﻿/**
 * @class Router
 * Роутер приложения. Обрабатывает все маршруты.
 *
 */
define(
    [
        'backbone',
        'underscore',
        'jquery',
        'settings',
        'state',
        'urlBuilder',
        'pubSub',
        'authentication',
        'srv',
        'log',
		'storage',
        'utils/uiManager',
        'utils/formatter',
        'utils/scrollStack',
        'views/login/LoginView',
        'views/activities/ActivitiesView',
        'views/discussions/DiscussionListView',
        'views/discussions/DiscussionView',
        'views/discussions/DiscussionCardView',
        'views/discussions/NewDiscussionView',
        'views/files/FolderListingView',
        'views/files/FolderCardView',
        'views/files/FileCardView',
        'views/tasks/TaskListView',
        'views/tasks/TaskView',
        'views/tasks/TaskCardView',
        'views/calendar/CalendarEventView',
        'views/calendar/CalendarEventCardView',
        //'views/calendar/EditCalendarEventView',
        'views/calendar/CalendarView',
        'views/calendar/TodayAgendaView',
        'views/calendar/CalendarDayAgendaView',
        'views/calendar/EditCalendarEventView',
        'views/search/SearchView',
        'views/users/ProfileView',
        'views/maintenance/FeedbackView',
        'views/dev/DevelopmentSettingsView',
        'views/dev/TestView',
        'views/dev/EventTestView',
		'views/notifications/NotificationView',
    ],
    function (
        Backbone,
        _,
        $,
        settings,
        state,
        urlBuilder,
        pubSub,
        authentication,
        srv,
        log,
		storage,
        uiManager,
        formatter,
        scrollStack,
        LoginView,
        ActivitiesView,
        DiscussionListView,
        DiscussionView,
        DiscussionCardView,
        NewDiscussionView,
        FolderListingView,
        FolderCardView,
        FileCardView,
        TaskListView,
        TaskView,
        TaskCardView,
        CalendarEventView,
        CalendarEventCardView,
        //EditCalendarEventView,
        CalendarView,
        TodayAgendaView,
        CalendarDayAgendaView,
        EditCalendarEventView,
        SearchView,
        ProfileView,
        FeedbackView,
        DevelopmentSettingsView,
        TestView,
        EventTestView,
		NotificationView
        )
    {
    	return Backbone.Router.extend({

    		currentView: null,
    		views: null,

    		/**
             * Инициализирует роутер, устанавливает обрабатываемые маршруты
             */
    		initialize: function ()
    		{
    			var that = this;
    			this.views = {};
    			this.route(/^(!(\/))?$/, 'redirectToHome');
    			this.route(this._makeRegExp('sample'), 'sample');
    			this.route(this._makeRegExp('login'), 'login');
    			this.route(this._makeRegExp('logout'), 'logout');
    			this.route(this._makeRegExp('activities'), 'activities');
    			this.route(this._makeRegExp('tasks'), 'tasks');
    			this.route(this._makeRegExp('calendarEvents'), 'calendarEvents');
    			this.route(this._makeRegExp('discussions'), 'discussions');
    			this.route(this._makeRegExp('files'), 'files');
    			this.route(this._makeRegExp('today'), 'today');
    			this.route(this._makeRegExp('calendar'), 'calendar');
    			this.route(this._makeRegExp('search'), 'search');
    			this.route(this._makeRegExp('profile'), 'profile');
    			this.route(this._makeRegExp('feedback'), 'feedback');

    			this.route(this._makeRegExp('dev'), 'dev');
    			//this.route(this._makeRegExp('discussions'), 'discussions');

    			this.route(this._makeRegExp('notifications'), 'notifications');

    			pubSub.on('login:success', function ()
    			{
    				that.redirectToHome();
    			});
    			pubSub.on('login:loggedOut', function ()
    			{
    				that.redirectToLogin();
    			});

    			pubSub.on('routing:doNavigate', function (args)
    			{
    				that.navigate(args.fragment, args.options || { trigger: true, navigate: false });
    			});

    		},

    		route: function (route, name)
    		{
    			var that = this;

    			if (name === 'login')
    			{
    				Backbone.Router.prototype.route.apply(this, arguments);
    			}
    			else
    			{
    				var callbackFunction = this[name];
    				var wrapperCallback = function ()
    				{
    					if (!authentication.isLoggedIn())
    					{
    						that.redirectToLogin();
    					}
    					else
    					{
    						that.saveActiveTool(name);
    						log.debug('Router: navigating to ' + document.location.hash);

    						scrollStack.remember(that.currentView);

    						pubSub.trigger('router:navigate');

    						callbackFunction.call(that, that.parseArgs(arguments[0]));
    					}
    				}
    				Backbone.Router.prototype.route.call(that, route, name, wrapperCallback);
    			}
    		},

    		/**
             * Генерирует регулярное выражение для семейства маршрутов.
             * @param {String} routeName наименование, задающее семейство маршрутов
             * @returns {RegExp} регулярное выражение
             * @private
             */
    		_makeRegExp: function (routeName)
    		{
    			return new RegExp('^!/' + routeName.replace(/\//g, '\\/') + '((/.*?)*)/*$');
    		},


    		redirectToHome: function ()
    		{
    			//                this.navigate(urlBuilder.activities(), {trigger: true, replace: false});
    			this.navigate(urlBuilder.tool(this.getLastActiveTool()), { trigger: true, replace: false });
    		},

    		redirectToLogin: function ()
    		{
    			this.navigate(urlBuilder.login(), { trigger: true, replace: false });
    		},

    		navigateBack: function ()
    		{
    			//scrollStack.backNavigationDetected();
    			if (this.currentView)
    			{
    				if (this.currentView.model && this.currentView.model.id)
    				{
    					if (this.currentView.options.activityId)
    						this.prevId = this.currentView.options.activityId;
    					else
    						this.prevId = this.currentView.model.id;
    				}

    				if (this.currentView instanceof CalendarDayAgendaView && this.currentView.options && this.currentView.options.date)
    					this.prevId = this.currentView.options.date;
    			}

    			history.back();
    		},

    		sample: function (args)
    		{
    			alert(JSON.stringify(args));
    		},

    		login: function ()
    		{
    			var that = this;
    			pubSub.trigger('login:begin');
    			this.flipView(function ()
    			{
    				return that.prepareView(LoginView);
    			});
    		},

    		logout: function ()
    		{
    			authentication.logout();
    			this.redirectToHome();
    		},

    		activities: function (args)
    		{
    			var that = this;
    			this._lastToolRoute = document.location.hash;
    			this.flipView(function ()
    			{
    				return that.prepareView(ActivitiesView, undefined, args);
    			});
    		},

    		tasks: function (args)
    		{
    			var that = this;
    			if (args['edit'])
    			{
    				this.flipView(function ()
    				{
    					var opts = {};
    					//(args['edit'] !== 'new') && (opts.entityId = args['edit']);
    					opts.entityId = args['edit'];
    					args['area'] && (opts.areaId = args['area']);
    					args['workspace'] && (opts.workspaceId = args['workspace']);
    					args['date'] && (opts.date = args['date']);
    					args['activityId'] && (opts.activityId = args['activityId']);
    					return that.prepareView(TaskCardView, opts);
    				});
    			}
    			else if (args['view'])
    			{
    				this.flipView(function ()
    				{
    					/*return that.prepareView(TaskView, {
                            entityId: args['view']
                        });*/
    					return that.prepareView(TaskCardView, {
    						entityId: args['view']
    					});
    				});
    			}
    			else
    			{
    				this._lastToolRoute = document.location.hash;

    				this.flipView(function ()
    				{
    					return that.prepareView(TaskListView, TaskListView.prototype.routeArgsToOptions(args));
    				});
    			}

    		},

    		calendarEvents: function (args)
    		{
    			var that = this;
    			if (args['edit'])
    			{
    				this.flipView(function ()
    				{
    					var opts = {};
    					//(args['edit'] !== 'new') && (opts.entityId = args['edit']);
    					opts.entityId = args['edit'];
    					args['area'] && (opts.areaId = args['area']);
    					args['workspace'] && (opts.workspaceId = args['workspace']);
    					args['date'] && (opts.date = args['date']);
    					args['activityId'] && (opts.activityId = args['activityId']);
    					return that.prepareView(CalendarEventCardView, opts);
    				});
    			}
    			else if (args['view'])
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(CalendarEventCardView, {
    						entityId: args['view']
    					});
    				});
    			}
    		},

    		discussions: function (args)
    		{
    			var that = this;
    			//if (args['edit'] === 'new')
    			//{
    			//	this.flipView(function ()
    			//	{
    			//		return that.prepareView(NewDiscussionView, undefined, args);
    			//	});
    			//}
    			//else if (args['view'])
    			//{
    			//	this.flipView(function ()
    			//	{
    			//		return that.prepareView(DiscussionCardView, {
    			//			entityId: args['view']
    			//		}, args);
    			//	});
    			//}
    			//else
    			//{
    			//	this.flipView(function ()
    			//	{
    			//		return that.prepareView(DiscussionListView, undefined, args);
    			//	});
    			//}
    			if (args['edit'])
    			{
    				this.flipView(function ()
    				{
    					var opts = {};
    					//(args['edit'] !== 'new') && (opts.entityId = args['edit']);
    					opts.entityId = args['edit'];
    					args['area'] && (opts.areaId = args['area']);
    					args['workspace'] && (opts.workspaceId = args['workspace']);
    					args['activityId'] && (opts.activityId = args['activityId']);
    					return that.prepareView(DiscussionCardView, opts);
    				});
    			}
    			else if (args['view'])
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(DiscussionCardView, {
    						entityId: args['view']
    					});
    				});
    			}
    			else
    			{
    				this._lastToolRoute = document.location.hash;
    				this.flipView(function ()
    				{
    					return that.prepareView(DiscussionListView, TaskListView.prototype.routeArgsToOptions(args));
    				});
    			}
    		},

    		files: function (args)
    		{
    			var that = this;
    			if (args['view'])
    			{
    				this.flipView(function ()
    				{
    					if (args['type'] === 'file')
    					{
    						// file
    						return that.prepareView(FileCardView, {
    							entityId: args['view'],
    							type: args['type']
    						}, args);
    					}
    					else
    					{
    						// folder
    						return that.prepareView(FolderCardView, {
    							entityId: args['view'],
    							type: args['type']
    						}, args);
    					}

    				});
    			}
    			else
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(FolderListingView, {
    						folder: args['folder']
    					}, args);
    				});
    			}
    		},

    		calendar: function (args)
    		{
    			var that = this;
    			this._lastToolRoute = document.location.hash;
    			if (args['date'])
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(CalendarDayAgendaView, args, args);
    				});
    			}
    			else
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(CalendarView, args, args);
    				});
    			}
    		},

    		today: function (args)
    		{
    			var that = this;
    			this._lastToolRoute = document.location.hash;
    			this.flipView(function ()
    			{
    				return that.prepareView(TodayAgendaView, undefined, args);
    			});
    		},

    		search: function (args)
    		{
    			var that = this;
    			this.flipView(function ()
    			{
    				return that.prepareView(SearchView, undefined, args);
    			});
    		},

    		profile: function ()
    		{
    			var that = this;
    			this.flipView(function ()
    			{
    				return that.prepareView(ProfileView);
    			});
    		},

    		feedback: function ()
    		{
    			var that = this;
    			this.flipView(function ()
    			{
    				return that.prepareView(FeedbackView);
    			});
    		},

    		notifications: function (args)
    		{
    			var that = this;
    			this.flipView(function ()
    			{
    				return that.prepareView(NotificationView, { open: args && args['open'] });
    			});
    		},

    		/**
             * Show development views
             */
    		dev: function (args)
    		{
    			var that = this;
    			if (args['view'] === 'settings')
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(DevelopmentSettingsView);
    				});
    			}
    			else if (args['view'] === 'testError')
    			{
    				var m = new Backbone.Model();
    				m.urlRoot = 'incorrect';
    				srv.save({
    					//collection: new Backbone.Collection(),
    					model: m/*,
                        url: 'incorrect'*/
    				});
    			}
    			else if (args['view'] === 'test')
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(TestView);
    				});
    			}
    			else if (args['view'] === 'events')
    			{
    				this.flipView(function ()
    				{
    					return that.prepareView(EventTestView);
    				});
    			}
    		},

    		prepareView: function (viewClass, options, args)
    		{
    			options = options || {};
    			_.defaults(options, this.locationOptionsFromRouteArgs(args), this.defaultViewOptions());
    			var view = new viewClass(options);
    			view.render && view.render();
    			return view;
    		},

    		defaultViewOptions: function ()
    		{
    			return {
    				router: this,
    				doRender: false
    			}
    		},

    		flipView: function (createViewCallback)
    		{
    			var that = this;

    			function setNewView()
    			{
    				if (createViewCallback)
    				{
    					var view = createViewCallback();
    					that.setPageView(view);
    				}
    			}

    			function rollbackUrl()
    			{
    				// TODO implement viewUrl() ?
    				//that.navigate(that.currentView.viewUrl(), {trigger: false, replace: true});
    			}

    			if (this.currentView)
    			{
    				if (this.currentView.beforeSave)
    				{
    					this.currentView.beforeSave();
    				}
    				if (this.currentView.hasUnsavedChanges && this.currentView.hasUnsavedChanges())
    				{
    					// TODO show dialog, ask to save the changes
    					that.currentView.remove();
    					setNewView();

    				}
    				else
    				{
    					that.currentView.remove();
    					setNewView();
    				}
    			}
    			else
    			{
    				setNewView();
    			}
    		},

    		setPageView: function (view)
    		{
    			this.currentView && this.currentView.remove && this.currentView.remove();
    			this.currentView = view;
    			//$('#tb-content').empty().append(view.el);
    			//view.$el.enhanceWithin();
    			// TODO use view.$el ?
    			//$.mobile.changePage(view.$el, {changeHash:false});
    			//$(document.body).prepend(view.$el);
    			//afterbegin
    			document.body.insertAdjacentElement('afterBegin', view.el);
    		},

    		parseArgs: function (argsStr)
    		{
    			var args = _.compact((argsStr || '').split('/')), result = {}, arg;
    			for (var i = 0, l = args.length; i < l - 1; i += 2)
    			{
    				arg = args[i];
    				result[args[i]] = this.escapeParam(args[i + 1]);
    			}
    			return result;
    		},

    		escapeParam: function (param)
    		{
    			param = decodeURIComponent(param);
    			return param.replace('<', '&lt;').replace('>', '&gt;').replace('&', '&amp;');
    		},

    		locationOptionsFromRouteArgs: function (args)
    		{
    			var options = {};
    			if (args)
    			{
    				args['area'] && (options.areaId = args['area']);
    				args['workspace'] && (options.workspaceId = args['workspace']);
    			}
    			return options;
    		},


    		reloadCurrentRoute: function ()
    		{
    			Backbone.history.loadUrl(Backbone.history.fragment);
    		},

    		tryForceUpdateCurrentView: function ()
    		{
    			this.currentView && this.currentView.update && this.currentView.update({ forceUpdate: true });
    		},


    		saveActiveTool: function (routeName)
    		{
    			var storageKeyName = '$lastActiveTool';
    			var saveableRouteNames = ['tasks', 'calendar', 'activities', 'discussions'];
    			if (_.indexOf(saveableRouteNames, routeName) != -1)
    			{
    				storage.set(storageKeyName, routeName);
    			}
    		},

    		getLastActiveTool: function ()
    		{
    			var storageKeyName = '$lastActiveTool';
    			var toolName = 'tasks';

    			var storedTool = storage.get(storageKeyName);
    			if (storedTool)
    				toolName = storedTool;

    			return toolName;
    		},
    	});
    });
