﻿// Конфигурация RequireJS
require.config({
    // подключаем библиотеки
    paths: {
        modernizr: 'lib/modernizr/modernizr',
        jquery: 'lib/jquery/jquery-2.1.1',
        underscore: 'lib/underscore/underscore',
        backbone: 'lib/backbone/backbone',
        backboneModelBinder: 'lib/backbone/backbone-modelbinder-mod',
        text: 'lib/require/text-improved',
        textOriginal: 'lib/require/text',
        domReady: 'lib/require/domready',
        i18n: 'lib/require/i18n',
        moment: 'lib/momentjs/moment',
        uuid: 'lib/uuid/uuid',
        clndr: 'lib/clndr/clndr',
        //IScroll: 'lib/iscroll/iscroll-lite',
        autosize: 'lib/jquery/plugins/jquery.autosize-mod',
        jsizes: 'lib/jquery/plugins/jquery.sizes',
        //iframeTransport: 'lib/jquery/plugins/jquery.iframe-transport',
        fastclick: 'lib/fastclick/fastclick',
        Autolinker: 'lib/autolinker/Autolinker',
        mprogress: 'lib/mprogress/mprogress',
        //touche: 'lib/touche/touche'
    },

    // прописываем зависимости и экспортируемые объекты для библиотек, не поддерживающих RequireJS
    shim: {

        'modernizr': {
            exports: 'Modernizr'
        },

        'underscore': {
            exports: '_'
        },

        'jquery': {
            exports: '$'
        },

        //'jqueryMockjax': {
        //    deps: ['jquery']
        //},

        moment: {
            deps: ['jquery']
        },

        'backbone': {
            deps: ['underscore', 'jquery'],
            exports: 'Backbone'
        },

        'i18n': {
            deps: ['moment']
        },

        'backboneModelBinder': {
            deps: ['backbone']
        },

        clndr: {
            deps: ['underscore', 'jquery', 'moment']
        },

        //IScroll: {
        //    exports: 'IScroll'
        //},

        jsizes: {
            deps: ['jquery']
        },

        autosize: {
            deps: ['jquery', 'jsizes']
        }



    },

    waitSeconds: 120,

    config: {
        i18n: {
            locale: 'ru'
        }
    }
});

// Main entry point
require([
    'underscore',
    'jquery',
    'backbone',
	'pushActivity',
    'Router',
    'settings',
    'setup',
    'authentication',
    'pubSub',
    'log',
    'srv',
    'utils/ui',
    'views/sidebar/SidebarView',
    'utils/photoBank',
    'utils/customPopupRegistry'
], function (_, $, Backbone, pushActivity, Router, settings, setup, authentication, pubSub, log, srv, ui, SidebarView, photoBank, customPopupRegistry)
{
	/*alert(_.keys(window));
     alert(_.keys(window.cordova));
     alert(_.keys(window.File));
     alert('dataDirectory: ' + window.cordova.file.dataDirectory);*/

	// чтобы ripple не ругался
	//var annoyingDialog = parent.document.getElementById('exec-dialog');
	//if (annoyingDialog) annoyingDialog.outerHTML = "";

	var $document = $(document);
	var $body = $(document.body);

	var router = new Router();
	window._appRouter = router;
	var defaultOptions = {
		router: router,
		doRender: false
	};

	var sidebarView = new SidebarView(_.defaults({ el: $('#sidebar') }, defaultOptions));

	authentication.isLoggedIn() && pubSub.trigger('tool:progress') && sidebarView.render();

	pubSub.on('login:success', function ()
	{
		sidebarView.prepareRefData();
		sidebarView.render();
		$('input').blur();
	});

	pubSub.on('login:loggedOut', function ()
	{
		sidebarView.setUnavailable();
	});


	pubSub.on('structure:updated', function ()
	{
		sidebarView.prepareRefData();
		sidebarView.render();
	});

	pubSub.on('router:navigate', function ()
	{
		ui.popup.closeAll();
	});

	pubSub.on('sidebar:show', function ()
	{
		if (sidebarView.available)
		{
			ui.popup.sidebar({
				retainableSelector: '#sidebar'
			});
		}
		else
		{
			var text;
			if (srv.isOnline())
			{
				text = 'Меню еще не загрузилось, подождите...';
			}
			else
			{
				text = 'Нет соединения с сетью';
			}
			ui.popup.message({
				text: text
			});

		}

	});

	// Subscribe to Cordova events
	$document.on('online', function ()
	{
		pubSub.trigger('network:online');
	});

	$document.on('offline', function ()
	{
		pubSub.trigger('network:offline');
	});


	$document.on('searchbutton', function ()
	{
		// Android only
		router.navigate('#!/search', { trigger: true, replace: false });
	});

	$document.on('menubutton', function ()
	{
		// Android and Blackberry only
		// TODO call sidebar via sidebar.show()
		ui.popup.closeAll();
		if (!(router.currentView && router.currentView.noSidebar))
		{
			ui.popup.sidebar({
				retainableSelector: '#sidebar'
			});
		}

	});

	$document.on('backbutton', function ()
	{
		// Android, BlackBerry, Windows Phone
		if (ui.popup.anyPopupsAreVisible() || customPopupRegistry.hasAny())
		{
			ui.popup.closeAll();
			customPopupRegistry.closeAll();
		}
		else if (router.currentView && router.currentView.onBackButton)
		{
			router.currentView.onBackButton.call(router.currentView);
		}
		else
		{
			router.navigateBack();
		}
		return false;
	});

	pubSub.on('error:apiCall', function (params)
	{
		/*alert('statusCode = ' + params.statusCode);
         alert('all params = ' + JSON.stringify(params));*/
		if (params.statusCode === 502 || params.statusCode === 504 || params.statusCode === 0)
		{
			// Ignore Bad Gateway errors
			return;
		}
		var text = 'Ой! Ошибочка вышла! Помогите нам улучшить приложение. Нажмите на кнопку "Отправить", и вы отправите нам подробную информацию об ошибке.';
		ui.popup.error({
			text: text,
			sendReportButtonCaption: 'Отправить',
			onSendReport: function ()
			{
				log.error('API call error', params);
				log.sync();
			}
		});
	});

	pubSub.on('error:networkError', function (params)
	{
		ui.popup.message({
			text: 'Нет соединения с сетью'
		});
		log.debug('Network error', params);
	});

	pubSub.on('error:unhandled', function (params)
	{
		var text = 'Ой! Ошибочка вышла! Помогите нам улучшить приложение. Нажмите на кнопку "Отправить", и вы отправите нам подробную информацию об ошибке.';
		ui.popup.error({
			text: text,
			sendReportButtonCaption: 'Отправить',
			onSendReport: function ()
			{
				log.error('Unhandled client error', params);
				log.sync();
				router.redirectToHome();
			}
		});
	});

	pubSub.on('error:unauthorizedApiCall', function (params)
	{
		log.warn('Unauthorized API call', params);
		router.redirectToLogin();
	});

	//window.addEventListener('orientationchange', function ()
	//{
	//	$(document.body).hide();

	//	//document.location.reload(true);
	//	//setTimeout(function ()
	//	//{
	//	//	$(document.body).show();
	//	//	$(window).trigger('resize');
	//	//}, 200);
	//});

	/*$body.swipeDown(function() {
     //alert('swipeDown');
     router.currentView && router.currentView.updateWithIndicator && router.currentView.updateWithIndicator();
     });*/

	// запускаем историю Backbone
	Backbone.history.start();

});
