﻿define(['underscore', 'jquery', 'settings', 'pubSub', 'srv', 'storage', 'log'], function (_, $, settings, pubSub, srv, storage, log)
{
	var storageKeyName = '$authenticationToken';
	var _token = null;
	//var _userName = null;

	var authentication = {
		login: function (login, password)
		{
			var that = this;

			function raiseLoginSuccess()
			{
				pubSub.trigger('login:success');
				pubSub.trigger('login:tokenChanged', _token);
			}

			//if (settings.development.skipAuthentication)
			//{
			//	_token = '12345678';
			//	raiseLoginSuccess();
			//}
			//else
			//{
			//if (settings.development.fakeAuthentication && _.isEmpty(login))
			//{
			//	login = fakeLogin;
			//	password = fakePassword;
			//}
			$.ajax({
				url: settings.authenticationUrlBase + 'account/token',
				timeout: settings.authenticationTimeout,
				type: 'POST',
				data: {
					username: login,
					password: password,
					rememberMe: false,
					grant_type: 'password'
				},
				success: function (data, textStatus, jqXHR)
				{
					if (data.token_type === 'bearer' && data.access_token)
					{
						_token = data.access_token;
						//_userName = data.userName;
						storage.set(storageKeyName, _token);
						srv.setAuthenticationToken(_token);
						log.info('Successfully logged in with login ' + login);
						raiseLoginSuccess();
					}
					else
					{
						log.warn('Something is wrong with authentication', {
							serverResponse: data
						});
						pubSub.trigger('login:fail', { reason: that.loginFailedReasons.unknown });
					}

				},
				error: function (jqXHR, textStatus, errorThrown)
				{
					var reason = that.loginFailedReasons.unknown;
					var errorInfo = (jqXHR && jqXHR.responseJSON) || {};
					if (jqXHR.status === 504 || jqXHR.status === 502 || jqXHR.status === 0)
					{
						reason = that.loginFailedReasons.cannotConnectToService;
					}
					else if (jqXHR.status === 400)
					{
						if (errorInfo.error === 'invalid_username')
						{
							reason = that.loginFailedReasons.invalidLogin;
						}
						else if (errorInfo.error === 'invalid_password')
						{
							reason = that.loginFailedReasons.invalidPassword;
						}
						else
						{
							reason = that.loginFailedReasons.invalidCredentials;
						}
					}

					log.info('Login failed for ' + login, {
						login: login,
						reason: reason,
						errorDescription: errorInfo.error_description
					});

					pubSub.trigger('login:fail', {
						reason: reason,
						errorDescription: errorInfo.error_description
					});
				}
			});
			//}
		},

		logout: function ()
		{
			$.ajax({
				url: settings.authenticationUrlBase + 'account/logout',
				timeout: settings.authenticationTimeout,
				type: 'POST',
				complete: function ()
				{
					_token = null;
					srv.setAuthenticationToken(undefined);
					storage.unset(storageKeyName);
					log.info('Logged out');
					pubSub.trigger('login:loggedOut');
				}
			});
		},

		isLoggedIn: function ()
		{
			return !_.isEmpty(_token);
		},

		addAuthTokenToHeaders: function (headers)
		{
			if (headers)
			{
				headers['Authorization'] = 'Bearer ' + _token;
			}
		},

		loginFailedReasons: {
			cannotConnectToService: "cannotConnectToService",
			invalidCredentials: "invalidCredentials",
			invalidLogin: "invalidLogin",
			invalidPassword: "invalidPassword",
			unknown: "unknown"
		}
	};

	var storedToken = storage.get(storageKeyName);
	if (storedToken)
	{
		_token = storedToken;
		srv.setAuthenticationToken(_token);
		//raiseLoginSuccess();
	}

	/*var authenticationCookie = getAuthenticationCookie();
     if (authenticationCookie) {
     _token = authenticationCookie;
     }*/

	return authentication;


});