﻿/**
 * Модуль логирования.
 */
define(['underscore', 'jquery', 'settings', 'version', 'utils/formatter', 'pubSub'],
    function (_, $, settings, version, formatter, pubSub)
    {

    	var levels = {
    		critical: 50,
    		error: 40,
    		warn: 30,
    		info: 20,
    		debug: 10
    	};

    	var logLevelValue = (settings.logging && levels[settings.logging.logLevel]) || levels.debug;
    	var syncLevelValue = (settings.logging && levels[settings.logging.logLevel]) || levels.warn;
    	var messages = [];

    	var versionNumber = version.get();

    	function getStackTrace()
    	{
    		var err = new Error('StackTrace');
    		return err.stack;
    	}

    	var currentUser;
    	pubSub.on('profile:updated', function (currentUserData)
    	{
    		currentUser = currentUserData;
    	});

    	return {
    		log: function (level, message, params)
    		{
    			var levelValue = levels[level];
    			if (levelValue >= logLevelValue)
    			{
    				var timeAsString = formatter.nowAsString();
    				var messageObject = {
    					level: level,
    					levelValue: levelValue,
    					dateTime: timeAsString,
    					message: message,
    					version: versionNumber
    				};
    				if (params && params.includeStackTrace)
    				{
    					params.stackTrace = getStackTrace();
    				}
    				_.extend(messageObject, params);
    				messages.push(messageObject);
    				if (console && console.log)
    				{
    					console.log(timeAsString + ' (' + level + '): ' + message);
    				}
    				// TODO save to local file or localStorage?
    			}
    		},

    		critical: function (message, params)
    		{
    			this.log('critical', message, params);
    		},

    		error: function (message, params)
    		{
    			this.log('error', message, params);
    		},

    		warn: function (message, params)
    		{
    			this.log('warn', message, params);
    		},

    		info: function (message, params)
    		{
    			this.log('info', message, params);
    		},

    		debug: function (message, params)
    		{
    			this.log('debug', message, params);
    		},

    		sync: function (options)
    		{
    			var that = this;
    			//var messagesToSync = _.filter(messages, function (item) {
    			//    return item.levelValue >= syncLevelValue;
    			//});

    			var errors = _.filter(messages, function (item) { return item.levelValue == levels.error; });
    			var debugs = _.last(_.filter(messages, function (item) { return item.levelValue == levels.debug; }), 5);

    			var deviceInfo = window.device ? window.device : 'No device information available';
    			$.ajax({
    				url: settings.restUrlBase + 'logEntry',
    				type: 'POST',
    				contentType: 'application/json; charset=utf-8',
    				data: JSON.stringify({
    					scope: 'mobile-client',
    					category: 'log-sync',
    					message: 'User sent log sync from mobile application',
    					//createdDateTime: formatter.nowAsString(),

    					details: {
    						clientErrors: errors,

    						device: deviceInfo,
    						userAgent: navigator.userAgent,
    						userEmail: currentUser ? currentUser.email : 'No email',
    						userName: currentUser ? currentUser.name : 'No name',
    						userMessage: (options && options.userMessage) ? options.userMessage : undefined,

							zDebugs: debugs
    					},
    					userId: (currentUser && currentUser.id) ? currentUser.id : undefined

    				}),
    				success: function ()
    				{
    					messages = [];
    					that.debug('Sync completed successfully.');
    					options && _.isFunction(options.success) && options.success.call(that);
    				},
    				error: function ()
    				{
    					options && _.isFunction(options.error) && options.error.call(that);
    				}

    			});
    		},

    		_messages: function () { return messages; }
    	}

    });