﻿define(['underscore', 'log', 'pubSub', 'srv', 'storage'], function (_, log, pubSub, srv, storage)
{
	// глобальный обьект - потому что в методах регистрации указываются строковые имена хендлеров а не ссылки
	window.tbPushClient = {
		// флаг включения - выключения логики связанной с уведомлениями
		enable: true,
		htmlStack: [],
		// кэш сообщений - крайнее сообщение всегда первое в массиве
		messages: [],
		// ид зарегистрированного устройтва в системе пуш-уведомлений GCM или APN
		currentRegId: null,
		// ид подписки пользователя и устройства на нашем сервере
		subscriptionId: null,
		
		// энумератор платформ
		platformType: {
			android: 1,
			ios: 2
		},

		// платформа устройства: 1 - Android, 2 - iOS
		platform: null,

		// Android - Google Cloud Messaging
		onNotificationGCM: function (e)
		{
			switch (e.event)
			{
				case 'registered':
					if (e.regid.length > 0)
					{
						log.debug("--PUSH: Regid " + e.regid);

						this.currentRegId = e.regid;

						pubSub.trigger('Push:Registered');
					}
					break;

				case 'message':
					// this is the actual push notification. its format depends on the data model from the push server
					// if this flag is set, this notification happened while we were in the foreground.
					// you might want to play a sound to get the user's attention, throw up a dialog, etc.

					var message = { viewed: false, time: new Date().getTime() };
					message.data = _.extend({}, e.payload);

					if (e.foreground)
					{
						message.viewed = false;
					}
					else
					{   // otherwise we were launched because the user touched a notification in the notification tray.

						message.viewed = true;

						if (e.coldstart)
						{
							
						}
						else
						{
							
						}
					}

					this.messages.unshift(message);

					//pubSub.trigger('Push:Message:Recieved', message);
					tbPushClient.onMessageRecieved(message);
					break;

				case 'error':
					log.error('--PUSH: --GCM error = ' + e.msg);
					break;

				default:
					log.debug('--PUSH: --An unknown GCM event has occurred');
					break;
			}
		},
		// iOS APN
		onNotificationAPN: function (event)
		{
			var normalizedPayload = tbPushClient.serializePayload(event);

			var message = _.extend({ viewed: false, time: new Date().getTime() }, normalizedPayload);

			if (event.foreground == 1)
				message.viewed = false;
			else
				message.viewed = true;

			tbPushClient.messages.unshift(message);

			tbPushClient.onMessageRecieved(message);//pubSub.trigger('Push:Message:Recieved', message);
		},

		// превращаем данные из плоского обьекта в древовидный - нужно только для Apple APN
		serializePayload: function (eventObj)
		{
			var result = {};

			tbPushClient.htmlStack.push('<li>' + JSON.stringify(eventObj) + '</li>');

			//result.data = JSON.parse(eventObj.data);

			//return result;

			var keys = _.keys(eventObj);

			_.each(keys, function (key)
			{
				var splitted = key.split('_');
				if (splitted.length == 2)
				{
					if (!result[splitted[0]])
						result[splitted[0]] = {};

					result[splitted[0]][splitted[1]] = eventObj[key];
				}

				if (splitted.length == 3)
				{
					if (!result[splitted[0]])
						result[splitted[0]] = {};

					if (!result[splitted[0]][splitted[1]])
						result[splitted[0]][splitted[1]] = {};

					result[splitted[0]][splitted[1]][splitted[2]] = eventObj[key];
				}
			});

			var tempIds = result.data.activityIDs;
			result.data.activityIDs = JSON.parse(tempIds);

			return result;
		},

		// хендлер андороид успешной регистрации
		successHandler: function (result)
		{
			log.debug("--PUSH: successHandler " + result);
		},

		// хендлер ошибок
		errorHandler: function (error)
		{
			log.error('--PUSH: --errorHandler: ' + error);
		},

		// хендлер при успешной регистрации девайса пользователя в Apple APN
		tokenHandler: function (result)
		{
			// Your iOS push server needs to know the token before it can push to this device
			// here is where you might want to send it the token for later use.
			if (tbPushClient.currentRegId == result)
				return;

			tbPushClient.currentRegId = result;

			setTimeout(function () { pubSub.trigger('Push:Registered'); }, 100);
		},

		// регистрация девайса пользователя для пуш-уведомлений на GCM и APN
		register: function ()
		{
			var pushNotification = window.plugins.pushNotification;
			if (device.platform == 'android' || device.platform == 'Android')
			{
				tbPushClient.platform = tbPushClient.platformType.android;
				pushNotification.register(tbPushClient.successHandler, tbPushClient.errorHandler, { "senderID": "373835752893", "ecb": "tbPushClient.onNotificationGCM" });
			}
			else
			{
				tbPushClient.platform = tbPushClient.platformType.ios;
				pushNotification.register(tbPushClient.tokenHandler, tbPushClient.errorHandler, { "badge": "true", "sound": "true", "alert": "true", "ecb": "tbPushClient.onNotificationAPN" });
			}
		},

		// сохраняем подписку устройства пользователя у нас
		saveUserRegistration: function (success)
		{
			var PushDisabled = storage.get('PushDisabled');

			if (!PushDisabled && currentUser && tbPushClient.currentRegId)
			{
				var deviceModel = device.model + '_' + device.version;
				srv.post({
					url: 'push/subscription',
					postData: {
						userId: currentUser.id,
						deviceToken: tbPushClient.currentRegId,
						deviceModel: deviceModel,
						deviceType: tbPushClient.platform,
						deviceId: device.uuid
					},
					success: function (response)
					{
						log.debug('--PUSH:<li>-device token registered-</li>');
						tbPushClient.subscriptionId = response.result.id;
						success && success();
					},
				});
			}
		},
		// отписка устройства пользователя от пуш-уведомлений
		deleteUserRegistration: function ()
		{
			var defer = $.Deferred();
			if (currentUser && tbPushClient.currentRegId && tbPushClient.subscriptionId)
			{
				srv.del({
					entityType: 'push/subscription',
					entityId: tbPushClient.subscriptionId,
					success: function ()
					{
						log.debug('--PUSH:<li>-device token unregistered-</li>');
						tbPushClient.subscriptionId = null;

						defer.resolve();
					},
					error: function () {
						defer.reject();
					}
				});
			} else {
				defer.reject();
			}
			return defer.promise();
		},

		// количество непросмотренных уведомлений
		getNotViewedLength: function ()
		{
			return _.filter(tbPushClient.messages, function (item) { return item.viewed === false; }).length;
		},

		// выставляем что уведомления просмотрены и обновляем подсветку
		setAllViewed: function ()
		{
			_.each(tbPushClient.messages, function (item) { item.viewed = true; });

			_.defer(tbPushClient.highLightPushCounter);
		},

		// срабатывает при получении уведомления - обьект уже нормализованный - данные в поле data
		onMessageRecieved: function (recievedMessage)
		{
			log.debug('--PUSH: onMessageRecieved: ' + JSON.stringify(recievedMessage));

			// чистить кэши и проча движуха в зависимости от сущности
			_.defer(tbPushClient.messagePostAction, recievedMessage)
		},

		// обновление подсветки об уведомлении
		highLightPushCounter: function ()
		{
			var pushCounter = tbPushClient.getNotViewedLength();

			if (tbPushClient.platform == tbPushClient.platformType.ios)
			{
				window.plugins.pushNotification.setApplicationIconBadgeNumber(tbPushClient.successHandler, tbPushClient.errorHandler, pushCounter);
			}

			$('.j-new-push-counter').each(function ()
			{
				var $badge = $(this).removeClass('empty');
				if (pushCounter > 0)
					$badge.text(pushCounter);
				else
					$badge.text('').addClass('empty');
			});
		},

		// пост обработка при получении уведомления
		messagePostAction: function (message)
		{
			if (message.data && message.data.activityIDs && message.data.activityIDs.length > 0)
			{
				tbPushClient.clearCache(message.data);

				srv.post({
					url: 'activity',
					postData: [message.data.activityIDs[0]],
					success: function (response)
					{
						if (response && response.result && response.result.length > 0)
						{
							message.activity = _.extend({}, response.result[0]);
							// подсветить везде где можно новое сообщение
							_.defer(tbPushClient.highLightPushCounter);

							pubSub.trigger('Push:Message:Recieved', message);

							// если пользователь пришел по клику из статус бара телефона то открываем режим уведомлений сразу
							if (message.viewed === true)
							{
								message.open = true;
								pubSub.trigger('routing:doNavigate', {
									fragment: '#!/notifications/',
									options: { trigger: true, replace: false }
								});
							}
						}
					},
				});
			}
		},
		clearCache: function (messageData)
		{
			// почистить ленту
			srv.clearCache('ajax_activity');
			srv.clearCache('ajax_area/' + messageData.areaID + '/workspace/' + messageData.workspaceID + '/activity');

			// почистить запросы по сущностям
			switch (messageData.entityType)
			{
				case 'task':
					srv.clearCache('ajax_planning/today');

					srv.clearCache('ajax_task/' + messageData.entityID);

					srv.clearCache('ajax_task?');
					srv.clearCache('ajax_area/' + messageData.areaID + '/workspace/' + messageData.workspaceID + '/task?');

					srv.clearCache('ajax_calendar?');
					srv.clearCache('ajax_area/' + messageData.areaID + '/workspace/' + messageData.workspaceID + '/calendar?');
					break;
				case 'discussionTopic':
					srv.clearCache('ajax_discussionTopic/' + messageData.entityID);

					srv.clearCache('ajax_discussionTopic');
					srv.clearCache('ajax_area/' + messageData.areaID + '/workspace/' + messageData.workspaceID + '/discussionTopic');
					break;
				case 'calendarEvent':
					srv.clearCache('ajax_planning/today');

					srv.clearCache('ajax_calendarEvent/' + messageData.entityID);

					srv.clearCache('ajax_calendar?');
					srv.clearCache('ajax_area/' + messageData.areaID + '/workspace/' + messageData.workspaceID + '/calendar?');
					break;
			}
		},
	};

	function onDeviceReady(event)
	{
		if (tbPushClient.enable === true)
			tbPushClient.register();
	};

	document.addEventListener('deviceready', onDeviceReady, false);

	var currentUser = null;
	if (tbPushClient.enable === true)
	{
		// когда есть обьект текущего юзера сохраняем подписку на нашем сервере
		pubSub.on('profile:updated', function (currentUserData)
		{
			currentUser = currentUserData;

			if (tbPushClient.currentRegId === null)
				pubSub.once('Push:Registered', function () { tbPushClient.saveUserRegistration(); });
			else
				tbPushClient.saveUserRegistration();
		});

		// подвязываем обработчик на получение уведомления
		//pubSub.on('Push:Message:Recieved', function (recievedMessage)
		//{
		//	_.defer(tbPushClient.onMessageRecieved, recievedMessage);
		//});
	}

	return null;
});

//var a = {
//	title: '',
//	message: '',
//	data: {
//		dependentEntityRef: {id:'', caption:'', type:''},
//		entityRef: {id:'', caption:'', type:''},
//		areaRef: {id:'', caption:''},
//		workspaceRef: {id:'', caption:''},
//		operation: 'created, updated, deleted'
//	},
//};