﻿/**
 * Данный модуль выполняет начальную инициализацию приложения при старте.
 */
define(
    [
        /* В зависимостях указан модуль-плагин domReady,
         чтобы запуск данного модуля произошел только после
         готовности DOM-дерева
         */
        'domReady!',
        'i18n!nls/global',
        'settings',
        'jquery',
        'utils/loader',
        'moment',
        'log',
        'pubSub',
        'environment',
        'fastclick',
        'utils/uiManager',
    ],
    function (domReady, i18n, settings, $, loader, moment, log, pubSub, environment, FastClick, uiManager) {


        function onTouchStart(evt) {
            //alert('onTouchStart');
            if(environment.isAndroid()) {
                //alert('onTouchStart: Yes, it is Android');
                evt.preventDefault();
            }
        }

        var tapHandled = false;

        function doTrickyThings() {
            if (environment.isMobileBrowser()) {
                var initialZeptoOn = $.fn.on;

                //alert('before overriding');
                $.fn.on = function (eventName) {
                    //alert('in new on');
                    var onArguments = arguments;
                    var onThis = this;
                    if (eventName && eventName.indexOf('tap') === 0) {
                        var tapArguments;
                        (function () {

                            var clickEventName = 'click' + eventName.slice(3);
                            var clickPreArguments = [clickEventName].concat(_.rest(onArguments));
                            var originalHandler = _.find(clickPreArguments, function(arg) { return _.isFunction(arg); });
                            function cleverHandler(evt) {
                                if (evt.type === 'click') {
                                    if (tapHandled) {
                                        //alert('click blocked');
                                        evt.preventDefault();
                                        evt.stopPropagation();
                                        return true;
                                    }
                                }
                                tapHandled = true;
                                originalHandler.apply(this, arguments);
                                //evt.type === 'click' && alert('handled: ' + evt.type);
                                //evt.preventDefault();
                                //evt.stopPropagation();
                                setTimeout(function() {
                                    tapHandled = false;
                                    //alert('tapHandled = false');
                                }, 1000);
                                return true;
                            }
                            function enhanceArgs(args) {
                                return _.map(args, function(arg) {
                                    if (arg === originalHandler) {
                                        return cleverHandler;
                                    } else {
                                        return arg;
                                    }
                                });
                            }

                            var clickArguments = enhanceArgs(clickPreArguments);
                            initialZeptoOn.apply(onThis, clickArguments);

                            tapArguments = enhanceArgs(onArguments);

                        })();
                        return initialZeptoOn.apply(onThis, tapArguments);

                    }
                    /*if (eventName && eventName.indexOf('swipe') === 0) {
                        var touchStartEventName = 'touchstart';
                        var touchStartArguments = [touchStartEventName].concat(_.rest(onArguments));
                        touchStartArguments = _.map(touchStartArguments, function (arg) {
                            if (_.isFunction(arg)) {
                                return onTouchStart;
                            } else {
                                return arg;
                            }
                        });
                        initialZeptoOn.apply(this, touchStartArguments);
                        // don't return here, continue to call original 'on'
                    }*/

                    /*if (eventName && eventName.indexOf('touchstart') === 0) {
                        *//*var touchStartEventName = 'touchstart';
                        var touchStartArguments = [touchStartEventName].concat(_.rest(onArguments));*//*
                        var touchStartArguments = _.map(onArguments, function (arg) {
                            if (_.isFunction(arg)) {
                                return function(evt) {
                                    onTouchStart.apply(this, arguments);
                                    //alert('onTouchStart called');
                                    return arg.apply(this, arguments);
                                };
                            } else {
                                return arg;
                            }
                        });
                        initialZeptoOn.apply(this, touchStartArguments);
                        // don't return here, continue to call original 'on'
                    }*/

                    return initialZeptoOn.apply(this, onArguments);
                }
            }
        }

        function prevent300ms() {

            var isMobileBrowser = environment.isMobileBrowser();

            var initialZeptoOn = $.fn.on;
            $.fn.on = function (eventName) {
                if (eventName && isMobileBrowser && eventName.indexOf('click') === 0) {
                    var tapEventName = 'tap' + eventName.slice(5);
                    var tapArguments = [tapEventName].concat(_.rest(arguments));
                    initialZeptoOn.apply(this, tapArguments);

                    var blockClickArguments = _.map(arguments, function (arg) {
                        if (_.isFunction(arg)) {
                            return function(evt) {
                                var tagName = $(evt.currentTarget).prop('tagName').toLowerCase();
                                if (tagName === 'input' || tagName === 'textarea') {
                                    return true;
                                } else {
                                    evt.preventDefault();
                                    evt.stopPropagation();
                                    return false;
                                }
                            };
                        } else {
                            return arg;
                        }
                    });
                    initialZeptoOn.apply(this, blockClickArguments);

                    var touchStartEventName = 'touchstart';// +  + eventName.slice(5);
                    var touchStartArguments = [touchStartEventName].concat(_.rest(arguments));
                    touchStartArguments = _.map(touchStartArguments, function (arg) {
                        if (_.isFunction(arg)) {
                            return onTouchStart;
                        } else {
                            return arg;
                        }
                    });
                    return initialZeptoOn.apply(this, touchStartArguments);
                } else {
                    return initialZeptoOn.apply(this, arguments);
                }
            }
        }

        function enableEventDiagnostics() {
            var initialZeptoOn = $.fn.on;
            $.fn.on = function (eventName) {
                var argsToDump = _.map(arguments, function(arg) {
                    if (_.isNull(arg)) {
                        return 'null';
                    } else if (_.isUndefined(arg)) {
                        return 'undefined';
                    } else {
                        return arg.toString();
                    }
                });
                var message = argsToDump.join(' | ');
                log.debug(message, {includeStackTrace: true});

                return initialZeptoOn.apply(this, arguments);
            }
        }

        function extendStandardLibrary() {
            String.prototype.endsWith = function(suffix) {
                return this.indexOf(suffix, this.length - suffix.length) !== -1;
            };
        }

        function hideKeyboardAccessoryBar() {
            // iOS only
            window.cordova && cordova.plugins && cordova.plugins.Keyboard && cordova.plugins.Keyboard.hideKeyboardAccessoryBar && cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
        }

        function shouldActivateFastClick() {
            // TODO make version comparison algorithm more universal
            return !(window.device && device.platform === 'Android' && (device.version === '4.4.2' || device.version !== '4.4.3' || device.version !== '4.4.3'));
        }

        function animateViaRaf(jQuery)
        {
        	var animating,
				lastTime = 0,
				vendors = ['webkit', 'moz'],
				requestAnimationFrame = window.requestAnimationFrame,
				cancelAnimationFrame = window.cancelAnimationFrame;

        	for (; lastTime < vendors.length && !requestAnimationFrame; lastTime++)
        	{
        		requestAnimationFrame = window[vendors[lastTime] + "RequestAnimationFrame"];
        		cancelAnimationFrame = cancelAnimationFrame ||
					window[vendors[lastTime] + "CancelAnimationFrame"] ||
					window[vendors[lastTime] + "CancelRequestAnimationFrame"];
        	}

        	function raf()
        	{
        		if (animating)
        		{
        			requestAnimationFrame(raf);
        			jQuery.fx.tick();
        		}
        	}

        	if (requestAnimationFrame)
        	{
        		// use rAF
        		window.requestAnimationFrame = requestAnimationFrame;
        		window.cancelAnimationFrame = cancelAnimationFrame;
        		jQuery.fx.timer = function (timer)
        		{
        			if (timer() && jQuery.timers.push(timer) && !animating)
        			{
        				animating = true;
        				raf();
        			}
        		};

        		jQuery.fx.stop = function ()
        		{
        			animating = false;
        		};
        	}
        	else
        	{
        		// polyfill
        		window.requestAnimationFrame = function (callback, element)
        		{
        			var currTime = new Date().getTime(),
						timeToCall = Math.max(0, 16 - (currTime - lastTime)),
						id = window.setTimeout(function ()
						{
							callback(currTime + timeToCall);
						}, timeToCall);
        			lastTime = currTime + timeToCall;
        			return id;
        		};

        		window.cancelAnimationFrame = function (id)
        		{
        			clearTimeout(id);
        		};
        	}
        }
        /**
         * Форматирует сообщение о возникшей необработанной ошибке
         * @param msg сообщение
         * @param url URL-адрес
         * @param line номер строки, где произошла ошибка
         * @returns {string} форматированное сообщение для записи в лог
         */
        function formatErrorMessage(msg, url, line) {
            return url + ' : line #' + line + ' : ' + msg;
        }

        extendStandardLibrary();

        // TODO why did I add this line with overflow:hidden?
        var isMobileBrowser = environment.isMobileBrowser();
        //isMobileBrowser && $(document.body).css('overflow', 'hidden');

        //$(document.body).height($(window).height());

    	shouldActivateFastClick() && FastClick.attach(document.body);


        // Если обработка ошибок выключена в настройках, то будут использованы стандартные средства браузера
        // Если включена, то используем наш обработчик
        if (settings.development.errorHandlingEnabled) {
            // Устанавливаем глобальный обработчик ошибок
            window.onerror = function (msg, url, line) {
                // Заносим ошибку в лог
                log.error(formatErrorMessage(msg, url, line));
                pubSub.trigger('loader:hide');
                pubSub.trigger('error:unhandled', {
                    msg: msg,
                    url: url,
                    line: line
                });
                return true;
            };
        }

        //prevent300ms();
        //doTrickyThings();
        //enableEventDiagnostics();

        hideKeyboardAccessoryBar();

        uiManager.initLayout();

        window.addEventListener('native.keyboardshow', function (e)
        {
       		cordova.plugins.Keyboard.disableScroll(true);

        	$(document.body).addClass('tb-keyboard-open');
        });
        window.addEventListener('native.keyboardhide', function (e)
        {
        	$(document.body).removeClass('tb-keyboard-open');

       		cordova.plugins.Keyboard.disableScroll(false);
        });

        return null;
    }
);


