﻿define([
    'underscore',
    'jquery',
    'backbone',
    'settings',
    'log',
    'pubSub',
    'storage',
    'utils/modelUtil',
    'utils/validationUtil',
    'uuid'
], function (_, $, Backbone, settings, log, pubSub, storage, modelUtil, validationUtil, uuid)
{
	//storage.clearAllAjax();

	// префиксы событий
	var pubSubAjaxSuccesEventPrefix = 'AjaxURLSuccess:';
	var pubSubAjaxErrorEventPrefix = 'AjaxURLError:';
	var ajaxUrlsInProgress = []; // коллекция урлов которые запрашиваются в текущий момент

	var cacheKeyPrefix = 'ajax_';
	var defaultTimeout = 60000;


	function appendSlashIfNeeded(s)
	{
		return (s[s.length - 1] === '/') ? s : s + '/';
	}

	function makeUrl(baseUrl, args)
	{
		if (!_.isEmpty(args))
		{
			var argArray = [];
			_.each(args, function (value, name)
			{
				(value && name) && argArray.push(encodeURI(name) + '=' + encodeURI(value));
			});
			argArray = _.sortBy(argArray, function (item)
			{
				return item;
			});
			return baseUrl + '?' + argArray.join('&');
		}
		else
		{
			return baseUrl;
		}
	}

	// убирает из списка запрашиваемых в текущий момент урлов
	function UnsetUrlInProgress(url)
	{
		ajaxUrlsInProgress = _.filter(ajaxUrlsInProgress, function (item) { return item != url; });
	}
	// добавляет в список запрашиваемых в текущий момент урлов
	function setUrlInProgress(url)
	{
		ajaxUrlsInProgress.push(url);
	}
	// проверяет урл запрашивается или нет - если нет то выход - если да то слушает ивенты pubSub(ждет аякс который уже идет по этому урлу) - и не дает аяксу идти
	function isUrlInProgress(options)
	{
		if (_.find(ajaxUrlsInProgress, function (item) { return item == options.url; }))
		{
			if (settings.ajaxDebugging)
			{
				options.ajaxDebuggingId = uuid.v4();
				options.ajaxDebuggingStartMillis = moment().unix();

				log.debug('Dublicate AJAX request detected: ' + options.url, {
					ajaxDebuggingId: options.ajaxDebuggingId,
					ajaxDebuggingStartMillis: options.ajaxDebuggingStartMillis,
					url: options.url
				});
			}

			if (options.error)
			{
				pubSub.once(pubSubAjaxErrorEventPrefix + options.url, function (context, args)
				{
					options.error && options.error.apply(context, args);
				});
			}

			if (options.success)
			{
				pubSub.once(pubSubAjaxSuccesEventPrefix + options.url, function (context, args)
				{
					options.success && options.success.apply(context, args);
				});
			}

			if (options.success || options.error)
				return true;
		}

		return false;
	}

	// TODO pass errorContext to options (the readable message that must shown to user)

	var _online = true;
	// Initializing network status via Cordova
	if (window.Connection && navigator.connection)
	{
		_online = (navigator.connection.type !== Connection.UNKNOWN && navigator.connection.type !== Connection.NONE);
		//alert('Cordova network init: online = ' + _online + ', connection = ' + navigator.connection.type);
	}

	pubSub.on('network:online', function ()
	{
		_online = true;
		//alert('network:online');
	});

	pubSub.on('network:offline', function ()
	{
		_online = false;
		//alert('network:offline');
	});

	var _authenticationToken;

	var _originalAjax = $.ajax;
	$.ajax = function (ajaxSettings)
	{
		setUrlInProgress(ajaxSettings.url);

		// generate cache key
		var cacheKey;
		if (ajaxSettings.url.indexOf(settings.restUrlBase) === 0)
		{
			cacheKey = cacheKeyPrefix + ajaxSettings.url.substring(settings.restUrlBase.length);
		}
		else
		{
			cacheKey = cacheKeyPrefix + ajaxSettings.url;
		}

		if (settings.ajaxDebugging)
		{
			ajaxSettings.ajaxDebuggingId = uuid.v4();
			ajaxSettings.ajaxDebuggingStartMillis = moment().unix();

			log.debug('Starting AJAX request: ' + ajaxSettings.url, {
				ajaxDebuggingId: ajaxSettings.ajaxDebuggingId,
				ajaxDebuggingStartMillis: ajaxSettings.ajaxDebuggingStartMillis,
				url: ajaxSettings.url
			});
		}

		var originalComplete = ajaxSettings.complete;
		ajaxSettings.complete = function (jqXHR, textStatus)
		{
			UnsetUrlInProgress(ajaxSettings.url);
			//console.log(ajaxUrlsInProgress);
			//pubSub.trigger('AjaxURLComplete:' + ajaxSettings.url);
			originalComplete && originalComplete.apply(this, arguments);
		};
		var originalError = ajaxSettings.error;
		ajaxSettings.error = function (jqXHR, textStatus, errorThrown)
		{
			//pubSub.trigger('AjaxURLComplete:' + ajaxSettings.url);
			originalError && originalError.apply(this, arguments);
			pubSub.trigger(pubSubAjaxErrorEventPrefix + ajaxSettings.url, this, arguments);
		};
		// prepare callback that will put the new data into cache
		var originalSuccess = ajaxSettings.success;
		ajaxSettings.success = function (data, textStatus, xhr)
		{
			if (settings.ajaxDebugging)
			{
				var finishMillis = moment().unix();
				var deltaMillis = finishMillis - ajaxSettings.ajaxDebuggingStartMillis;
				log.debug('AJAX request finished: ' + ajaxSettings.url, {
					ajaxDebuggingId: ajaxSettings.ajaxDebuggingId,
					ajaxDebuggingStartMillis: ajaxSettings.ajaxDebuggingStartMillis,
					ajaxDebuggingFinishMillis: finishMillis,
					ajaxDebuggingDeltaMillis: deltaMillis,
					url: ajaxSettings.url,
					contentType: xhr && xhr.getResponseHeader ? xhr.getResponseHeader('Content-Type') : undefined,
					contentLength: xhr && xhr.getResponseHeader ? xhr.getResponseHeader('Content-Length') : undefined
				});
			}

			ajaxSettings.type === 'GET' && (!ajaxSettings.fromCache) && storage.set(cacheKey, data);
			originalSuccess && originalSuccess.apply(this, arguments);
			pubSub.trigger(pubSubAjaxSuccesEventPrefix + ajaxSettings.url, this, arguments);
		};

		//(!ajaxSettings.timeout) && (ajaxSettings.timeout = defaultTimeout);
		(_.isUndefined(ajaxSettings)) && (ajaxSettings.timeout = defaultTimeout);

		// Debugging alert:
		/*if (ajaxSettings.url.indexOf('current') !== -1) { alert('ajax: current: online = ' + _online + ', forceUpdate = ' + ajaxSettings.forceUpdate); }*/

		if (_.isObject(ajaxSettings) && (!_online || !ajaxSettings.forceUpdate) && (ajaxSettings.type === 'GET' || _.isEmpty(ajaxSettings.type)))
		{
			// try to get data from cache
			var cachedValue = storage.get(cacheKey);
			if (!_.isNull(cachedValue) && !_.isUndefined(cachedValue))
			{
				ajaxSettings.fromCache = true;
				UnsetUrlInProgress(ajaxSettings.url);
				ajaxSettings.success && ajaxSettings.success.call($, JSON.parse(cachedValue));
				// TODO return fake xhr
				//pubSub.trigger('loader:stop');
				return {};
			}
		}

		if (_authenticationToken)
		{
			ajaxSettings.headers = {
				'Authorization': 'Bearer ' + _authenticationToken
			};
		}
		return _originalAjax.apply($, arguments);
	};


	function defaultAjaxErrorHandler(statusCode, statusText, responseBody, xhr, options)
	{
		var params = {};
		//options === window && (options = {});
		options && options.url && (params.url = options.url);
		if (settings.ajaxDebugging)
		{
			var deltaMillis = options ? (options.ajaxDebuggingStartMillis - moment().unix()) : undefined;
			_.extend(params, {
				statusCode: statusCode,
				statusText: statusText,
				responseBody: responseBody,
				includeStackTrace: true,
				ajaxDebuggingId: options ? options.ajaxDebuggingId : undefined,
				ajaxDebuggingDeltaMillis: deltaMillis,
				contentType: xhr && xhr.getResponseHeader ? xhr.getResponseHeader('Content-Type') : undefined,
				contentLength: xhr && xhr.getResponseHeader ? xhr.getResponseHeader('Content-Length') : undefined

			});
		}
		pubSub.trigger('update:error');
		if (statusCode == 401)
		{
			// not authorized
			pubSub.trigger('error:unauthorizedApiCall', params);
			return;
		}
		if (statusCode == 502 || statusCode == 504 || statusCode == 0)
		{
			// network issues
			pubSub.trigger('error:networkError', params);
			return;
		}
		log.error('Ajax error occurred with statusCode = ' + statusCode + ', statusText = ' + statusText, params);
		pubSub.trigger('error:apiCall', params);
	}

	var defaultBackboneFetchOptions = {
		error: function (modelOrCollection, response, options)
		{
			defaultAjaxErrorHandler(response.status, response.statusText, response.responseText, options.xhr, options);
		}
	};


	function processUrl(options, modelOrCollection)
	{
		if (!_.isEmpty(options.urlParams))
		{
			var url = _.result(options, 'url');
			(!url && modelOrCollection) && (url = _.result(modelOrCollection, 'url'));
			url && (options.url = makeUrl(url, options.urlParams));
		}
	}

	/**
     * @deprecated
     * @param message
     * @param options
     * @param modelOrCollection
     */
	function logAjaxCall(message, options, modelOrCollection)
	{
		var url = _.result(options, 'url');
		if (!url && modelOrCollection)
		{
			url = _.result(modelOrCollection, 'url');
		}
		log.debug(message + ' URL = ' + url);
	}


	function fetchModel(options)
	{
		var model = options.model;
		delete options.model;
		_.defaults(options, defaultBackboneFetchOptions);
		processUrl(options, model);
		//settings.ajaxDebugging && logAjaxCall('Fetching model', options, model);
		
		if (isUrlInProgress(options) === true) // проверка может уже запрашивается урл
			return;
		model.fetch(options);
	}

	function fetchCollection(options)
	{
		var collection = options.collection;
		delete options.collection;
		_.defaults(options, defaultBackboneFetchOptions);
		processUrl(options, collection);
		//settings.ajaxDebugging && logAjaxCall('Fetching collection', options, collection);

		if (isUrlInProgress(options) === true) // проверка может уже запрашивается урл
			return;
		collection.fetch(options);
	}

	function ajaxGet(options)
	{
		var defaultAjaxOptions = {
			error: function (jqXHR, textStatus, errorThrown)
			{
				defaultAjaxErrorHandler(jqXHR.status, jqXHR.statusText, jqXHR.responseText, jqXHR, options);
			}
		};
		var originalSuccess = options.success;
		_.defaults(options, {
			type: 'GET'
		}, defaultAjaxOptions);
		options.success = function (data, textStatus, jqXHR)
		{
			originalSuccess && originalSuccess(data ? data.result : undefined);
		};
		processUrl(options);
		//settings.ajaxDebugging && logAjaxCall('Fetching AJAX data', options);

		if (isUrlInProgress(options) === true) // проверка может уже запрашивается урл
			return;
		$.ajax(options);
	}

	function saveModel(options)
	{
		var model = options.model;
		delete options.model;
		_.defaults(options, {
			error: function (model, response, options)
			{
				if (response.status === 422)
				{ // 422 - Unprocessable Entity
					var parsedResponse = JSON.parse(response.responseText);
					var validatedView = options.validatedView;
					if (validatedView)
					{
						validationUtil.parseValidationResponse(validatedView,
                            parsedResponse,
                            validatedView.customValidationHandlers);
					} else
					{
						log.warn('Validation response got without associated view', {
							parsedResponse: parsedResponse
						});
					}

					return;

				}
				options = _.omit(options, 'validatedView');
				options.url || (options.url = _.result(model, 'url'));
				defaultAjaxErrorHandler(response.status, response.statusText, response.responseText, options.xhr, options);
			}
		});

		var originalSuccess = options.success;
		options.success = function (model, response, options)
		{
			// TODO handle validation with validatedView
			//console.log(JSON.stringify(arguments));
			originalSuccess && originalSuccess(model, response, options);
		};
		model.save(options.attributes, options);
	}

	return {
		fetch: function (options)
		{
			//options = options || {};
			if (options.model)
			{
				fetchModel(options);
			}
			else if (options.collection)
			{
				fetchCollection(options);
			}
			else
			{
				ajaxGet(options);
			}
		},

		save: function (options)
		{
			if (options.model)
			{
				saveModel(options);
			}
			else
			{
				// TODO handle this situation
			}
		},

		del: function (options)
		{
			if (options.customData)
			{
				$.ajax({
					type: 'DELETE',
					url: settings.restUrlBase + options.entityType,
					data: JSON.stringify(options.customData),
					dataType: 'json',
					contentType: 'application/json',
					success: function ()
					{
						options.success && options.success();
					},
					error: function (jqXHR, textStatus, errorThrown)
					{
						defaultAjaxErrorHandler(jqXHR.status, jqXHR.statusText, jqXHR.responseText, jqXHR, options);
					}
				});
			}
			else
			{
				$.ajax({
					type: 'DELETE',
					url: settings.restUrlBase + options.entityType + '/' + options.entityId,
					success: function ()
					{
						options.success && options.success();
					},
					error: function (jqXHR, textStatus, errorThrown)
					{
						options.error && options.error(jqXHR, textStatus, errorThrown);
						if (jqXHR && jqXHR.errorAlreadyHandled === true)
							return;

						defaultAjaxErrorHandler(jqXHR.status, jqXHR.statusText, jqXHR.responseText, jqXHR, options);
					}
				});
			}

		},

		post: function (options)
		{
			// TODO
			$.ajax({
				type: 'POST',
				url: settings.restUrlBase + options.url,
				data: JSON.stringify(options.postData),
				dataType: 'json',
				contentType: 'application/json',
				success: function (data)
				{
					options.success && options.success(data);
				},
				error: function (jqXHR, textStatus, errorThrown)
				{
					defaultAjaxErrorHandler(jqXHR.status, jqXHR.statusText, jqXHR.responseText, jqXHR, options);
				}
			});
		},

		clearCache: function (pattern, forceClear)
		{
			(_online || forceClear) && storage.unsetContaining(pattern);
		},

		setAuthenticationToken: function (token)
		{
			_authenticationToken = token;
		},

		isOnline: function ()
		{
			return _online;
		},

		makeUrlWithLocationContext: function (locationContext, entityType, entityId)
		{
			function translateLocationContext(source)
			{
				var target = {};
				var keyMappings = {
					'areaId': 'area', 'workspaceId': 'workspace',
					'area': 'area', 'workspace': 'workspace'
				};
				_.each(keyMappings, function (keyTo, keyFrom)
				{
					source[keyFrom] && (target[keyTo] = source[keyFrom]);
				});
				return target;
			}
			var url = settings.restUrlBase;
			if (locationContext)
			{
				locationContext = translateLocationContext(locationContext);
				_.each(_.sortBy(_.pairs(locationContext), modelUtil.urlComponentSorter()), function (pair)
				{
					(pair[0] && pair[1]) && (url += pair[0] + '/' + pair[1] + '/');
				});
			}
			if (entityType)
			{
				url = appendSlashIfNeeded(url);
				url += entityType;
			}
			if (entityId)
			{
				url = appendSlashIfNeeded(url);
				url += entityId;
			}
			return url;
		}
	};


});