﻿/**
 * Модуль позволяет строить параметризованные URL-адреса для конкретных режимов и функций приложения.
 */
define(['underscore'], function (_) {

    return {

        // TODO do we need entityId support here?
        tool: function (toolName, areaId, workspaceId, entityId) {
            if (entityId && areaId && workspaceId) {
                return '#!/' + toolName + '/area/' + areaId + '/workspace/' + workspaceId + '/entity/' + entityId;
            } else if (areaId && workspaceId) {
                return '#!/' + toolName + '/area/' + areaId + '/workspace/' + workspaceId;
            } else if (entityId) {
                return '#!/' + toolName + '/entity/';
            } else {
                return '#!/' + toolName;
            }
        },

        tasks: function (args) {
            return this.generic('tasks', args);
        },

        task: function (entityId, openingMode, locationContext) {
            return this.entity('tasks', entityId, openingMode, locationContext);
        },

        discussion: function(entityId, openingMode, locationContext) {
            return this.entity('discussions', entityId, openingMode, locationContext);
        },

        calendarEvent: function(entityId, openingMode, locationContext) {
            return this.entity('calendarEvents', entityId, openingMode, locationContext);
        },

        entity: function (uiMode, entityId, openingMode, locationContext, activityId) {
            openingMode = openingMode || 'view';
            locationContext = locationContext || {};

            var args = {};
            (locationContext.areaId) && (args['area'] = locationContext.areaId);
            (locationContext.workspaceId) && (args['workspace'] = locationContext.workspaceId);
            args[openingMode] = entityId;
			
            activityId && (args['activityId'] = activityId);

            return this.generic(uiMode, args);
        },

        generic: function (name, args) {
            var argsStr = '';
            if (args) {
                _.each(_.pairs(args), function (nameValue) {
                    if (nameValue[0] && nameValue[1]) {
                        argsStr += ('/' + nameValue[0] + '/' + nameValue[1]);
                    }
                });
            }
            return '#!/' + name + argsStr;

        },

        structure: function () {
            return '#!/structure';
        },

        activities: function () {
            return '#!/activities';
        },

        login: function() {
            return '#!/login';
        }

    };

});