﻿define(['underscore', 'jquery', 'moment', 'srv', 'log', 'pubSub'], function(_, $, moment, srv, log, pubSub) {

    var _defaultExpirationTimeoutSeconds = 6 * 60; // two minutes

    return function(url, expirationTimeoutSeconds) {
        this.url = url;
        this.expirationTimeoutSeconds = expirationTimeoutSeconds ? expirationTimeoutSeconds : _defaultExpirationTimeoutSeconds;
        this.lastUpdated = null;
        this.data = null;
        var instance = this;

        this.refDataLoader = function(refDataName) {
            // This function is called with view as this
            // so put reference to view instance to that
            var that = this;
            instance.updateIfExpired(function(data) {
                that.onDataLoaded(refDataName, data);
            });
        };

        this.clear = function() {
            this.lastUpdated = null;
            this.data = null;
        };

        this.updateIfExpired = function(successCallback) {
            if (this.isExpired()) {
                this.update(successCallback);
            } else {
                successCallback && successCallback.call(this, this.data, false);
            }
        };

        this.isExpired = function() {
            return this.lastUpdated ? (moment().unix() - this.lastUpdated.unix() > this.expirationTimeoutSeconds) : true;
        };

        this.get = function() {
            return this.data;
        };

        this.update = function(successCallback) {
            var that = this;
            srv.fetch({
                url: this.url,
                forceUpdate: true,
                success: function(result) {
                    that.data = result;
                    that.lastUpdated = moment();
                    that.onUpdated && that.onUpdated(that.data);
                    successCallback && successCallback.call(that, that.data, true);
                },
                error: function(jqXHR, textStatus, errorThrown) {
                    log.info('Cannot load cachable data: ' + that.url + '  ' + jqXHR.status + ' ' + jqXHR.statusText);
                    if (jqXHR.status == 401) {
                        // not authorized
                        pubSub.trigger('error:unauthorizedApiCall', {});
                        return;
                    }
                }
            });
        }
    };

});