﻿define(['underscore', 'utils/ui', 'log', 'srv', 'settings', 'environment'], function (_, ui, log, srv, settings, environment)
{
    return {
        showAttachUI: function (options) {

            if (!window.Camera) {
                ui.popup.message({
                    text: 'К сожалению, работа с камерой не поддерживается на данном устройстве',
                    classes: 'ui-popup-message ui-popup-cardArea'
                });
                return;
            }

            function takePhoto() {
                var cameraOptions = {
                    destinationType: Camera.DestinationType.FILE_URI,
                    sourceType: Camera.PictureSourceType.CAMERA,
                    mediaType: Camera.MediaType.PICTURE,
                    allowEdit: false,
                    saveToPhotoAlbum: false
                };
                navigator.camera.getPicture(function(imageUrl) {
                    //alert(imageUrl ? imageUrl : 'No image');
                    options.onAttachmentCreated && options.onAttachmentCreated(imageUrl);

                }, function() {
                    log.info('Cannot take a photo or operation was cancelled');
                }, cameraOptions);

            }

            function loadFromGallery() {
                var cameraOptions = {
                    destinationType: Camera.DestinationType.FILE_URI,
                    sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
                    mediaType: Camera.MediaType.PICTURE,
                    allowEdit: false,
                    saveToPhotoAlbum: false
                };
                navigator.camera.getPicture(function(imageUrl) {
                    options.onAttachmentCreated && options.onAttachmentCreated(imageUrl);
                }, function() {
                    throw 'Не удалось сделать снимок';
                }, cameraOptions);

            }

            var items = options.items || [
                {
                    value: 'takePhoto',
                    caption: 'Сделать снимок',
                    icon: 'camera'
                },
                {
                    value: 'loadFromGallery',
                    caption: 'Загрузить из галереи',
                    icon: 'photo'
                }
            ];
            ui.popup.list({
                items: items,
                classes: 'ui-popup-list ui-popup-list-largePadding ui-popup-cardArea',
                onSelected: function (value, item) {
                    switch (value) {
                        case 'takePhoto':
                            takePhoto();
                            break;
                        case 'loadFromGallery':
                            loadFromGallery();
                            break;
                        default:
                            var selectedItem = _.find(items, function(item) {
                                return item.value === value;
                            });
                            selectedItem && selectedItem.onClick && selectedItem.onClick();
                    }
                }
            });
        },

        processAttachmentWithTicket: function(attachmentId, callback) {
            srv.fetch({
                url: settings.restUrlBase + 'ticket/' + attachmentId,
                forceUpdate: true,
                success: function(result) {
                    var ticket = result.key;
                    var expires = result.expires;
                    callback && callback(attachmentId, ticket, expires);
                }
            });
        },
        processAttachmentWithTicketAndDownload: function (options)
        {
        	var that = this;

        	var fileName = options.attach.title ? options.attach.title : options.attach.caption;

        	var callback = function (attachmentId, ticket, expires)
        	{
        		var documentUrl = that._documentUrlWithTicket(attachmentId, ticket, expires);

        		window.requestFileSystem = window.requestFileSystem || window.webkitRequestFileSystem;

        		if (window.requestFileSystem && window.FileTransfer)
        		{
        			window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, gotFS, fail);

        			function fail(error)
        			{
        				log.error('Ошибка при скачивании файла ' + fileName + ', ' + (error ? error.toString() : '.'));
        				options.error && options.error();
        			}

        			function gotFS(fileSystem)
        			{
        				fileSystem.root.getDirectory("TeamBridge", { create: true, exclusive: false }, gotDir, fail);
        			}

        			function gotDir(dirEntry)
        			{
        				dirEntry.getFile(fileName, { create: true, exclusive: false }, gotFile, fail);
        			}

        			function gotFile(file_entry)
        			{
        				//log.debug('gotFile');
        				var fileTransfer = new FileTransfer();
        				var uri = encodeURI(documentUrl);

        				fileTransfer.download(
							uri,
							file_entry.toURL(),
							function (entry)
							{
								var resFile = entry.toURL();
								log.debug("Файл скачан: " + resFile);

								options.success && options.success();

								openFile(entry);
								//window.open(resFile, "_system");
								//var ref = window.open(resFile, '_blank', 'location=no,enableViewportScale=yes');
							},
							fail, true);
        			};

        			function openFile(fileEntry)
        			{
        				tbPushClient.htmlStack.push('<li>--openFile--</li>');

        				fileEntry.file(function (file)
        				{
        					var fPath = fileEntry.toNativeURL();

        					tbPushClient.htmlStack.push('<li>--name:' + file.name + ' mime-type: ' + file.type + ' toNativeURL:' + fPath + '--</li>');

        					var p = '';
        					var fileOpenerPluginInterface = null;

        					if (environment.isIOS())
        					{
        						p = fPath;
        						fileOpenerPluginInterface = cordova.plugins.fileOpener2a;
        					}
        					else
        					{
        						p = decodeURIComponent(fPath);//file.fullPath;//fileEntry.toNativeURL();
        						fileOpenerPluginInterface = cordova.plugins.fileOpener2;
        					}

        					if (p && fileOpenerPluginInterface)
        					{
        						var t = file.type;
        						var ext = file.name.split('.');

        						fileOpenerPluginInterface.open(p, t, {
        							error: function (errorObj)
        							{
        								tbPushClient.htmlStack.push('<li>path:' + p + ' Error status: ' + errorObj.status + ' - Error message: ' + errorObj.message + '</li>');
        								alert('Не найдено приложения для открытия файлов с расширением *.' + ext[ext.length - 1]);
        							},
        							success: function ()
        							{
        								tbPushClient.htmlStack.push('<li>file opened successfully</li>');
        							}
        						});
        					}

        				}, fail);
        			};
        		}
        		else
        		{
        			window.open(documentUrl, '_system', 'location=yes');
        			options.success && options.success();
        		}
        	};

        	that.processAttachmentWithTicket(options.attach.id, callback);
        },

        _documentUrlWithTicket: function (attachmentId, ticket, expires)
        {
        	return settings.restUrlBase + 'attachment/' + attachmentId + '?ticket=' + ticket + '&expires=' + expires;// + '&inline=true';
        	// var resUrl = file.url + '?ticket=' + data.result.key + '&expires=' + data.result.expires;
        },
    };
});
