﻿define(['underscore'], function(_) {

    function toStringSmartly(v) {
        if (_.isUndefined(v) || _.isNull(v)) {
            return '';
        } else {
            return v.toString();
        }
    }

    return {

        /**
         *
         * @param entityModel task or event or something else
         */
        locationContextCaption: function(entityModel) {
            var areaCaption = entityModel.has('areaRef') ? entityModel.get('areaRef').caption : '';
            var workspaceCaption = entityModel.has('workspaceRef') ? entityModel.get('workspaceRef').caption : '';
            var parentCaption = entityModel.has('parentTaskRef') ? entityModel.get('parentTaskRef').caption : '';
            return this.locationContextCaptionFromMultipleCaptions(areaCaption, workspaceCaption, parentCaption);
        },
        locationContextCaptionObj: function (entityObj)
        {
        	var areaCaption = entityObj.areaRef ? entityObj.areaRef.caption : '';
        	var workspaceCaption =  entityObj.workspaceRef ?  entityObj.workspaceRef.caption : '';

        	return this.locationContextCaptionFromMultipleCaptions(areaCaption, workspaceCaption);
        },

        locationContextCaptionFromMultipleCaptions: function(/* caption1, caption2, etc... */) {
            var captions = _.filter(arguments, function(item) { return item; });
            var smartCaptions = _.map(captions, function(caption) { return toStringSmartly(caption); });
            return smartCaptions.join(' > ');
        },

        /**
         * @deprecated use locationContextCaptionFromMultipleCaptions instead
         * @param areaCaption
         * @param workspaceCaption
         * @returns {*}
         */
        locationContextCaptionFromTwoCaptions: function(areaCaption, workspaceCaption) {
            if (areaCaption && workspaceCaption) {
                return areaCaption + ' > ' + workspaceCaption;
            } else {
                return toStringSmartly(areaCaption) + toStringSmartly(workspaceCaption);
            }
        },

        allWorkspacesForSelect: function(structure) {
            var that = this;
            var allWorkspaces = [];
            _.each(structure.areas, function (area) {
                _.each(area.workspaces, function (workspace) {
                    allWorkspaces.push({
                        id: workspace.id,
                        caption: that.locationContextCaptionFromTwoCaptions(area.title, workspace.title),
                        areaId: area.id
                    });
                });
            });
            return allWorkspaces;
        }
    };

});