﻿define(['underscore', 'backbone'], function (_, Backbone) {

    var urlComponentOrder = {
        area: 10,
        workspace: 20,
        task: 100, calendar: 100,
        discussion: 200,
        attachment: 1000
    };

    return {
        modelUpdater: function (model, attribute) {
            return function (response, newValue) {
                model && attribute && model.set(attribute, newValue);
            }
        },

        urlComponentSorter: function() {
            return function(pair) {
                var orderValue = urlComponentOrder[pair[0]];
                orderValue = orderValue || 10000;
                return orderValue;
            };
        },

        convertToCollectionIfNeeded: function (dataSource, collectionClass) {
            collectionClass = collectionClass || Backbone.Collection;
            var modelClass = collectionClass.prototype.model;

            function createModelsIfNecessary(array) {
                var result = [];
                _.each(array, function(item) {
                    result.push(item instanceof modelClass ? item : new modelClass(item));
                });
                return result;
            }
            if (dataSource) {
                if (dataSource instanceof collectionClass) {
                    return dataSource;
                } else if (_.isArray(dataSource)) {
                    return new collectionClass(createModelsIfNecessary(dataSource));
                }
            }
            return new collectionClass();


        }

    };

});