﻿define(['underscore', 'jquery', 'moment', 'utils/formatter', 'autosize', 'utils/photoBank', 'utils/customPopupRegistry', 'log', 'environment'],
	function (_, $, moment, formatter, autosize, photoBank, customPopupRegistry, log, environment)
	{

    var popupScrollers = [];

    function createPopupScroller(element) {
        var scroll = new IScroll(element, {
        	tap: false,
            preventDefault: true,
            preventDefaultException: { tagName: /^(INPUT|TEXTAREA|BUTTON|A|SELECT|LI)$/ },
            click: false,

            bounce: false,

        });
        popupScrollers.push(scroll);
        return scroll;
    }

    function cleanPopupScrollers() {
        _.each(popupScrollers, function(scroller) {
            scroller.destroy();
        });
        popupScrollers = [];
    }

    function isEmptyPickerValue(value) {
        return _.isNull(value) || _.isUndefined(value) || value === '' || (_.isArray(value) && value.length === 0);
    }

    function pickerIcon(options, value) {
        if (options && options.icon) {
            return '<i class="ui-pickerIcon fa fa-' + options.icon + (_.isEmpty(value) ? ' ui-notSpecified' : '') + '"></i>';
        } else {
            return '';
        }
    }

    function internalIndexOf(array, predicate) {
        for (var i = 0; i < array.length; i++) {
            var element = array[i];
            if (predicate.call(this, element)) {
                return i;
            }
        }
        return -1;
    }

    var controlNumber = 1;

    function newControlNumber() {
        return controlNumber++;
    }

    var defaultEmptyCaption = 'Не задано';
    var $body = $(document.body);

    function $pickerValueSpan(spanOptions) {
        var $span = $('<span class="ui-pickerValue"></span>');
        spanOptions && _.isEmpty(spanOptions.value) && $span.addClass('ui-notSpecified');
        return $span;
    }

    function padLeft(num, n, str) {
        return new Array(n - String(num).length + 1).join(str || '0') + num;
    }

    function textAreaRows($div) {
        var lineHeight = $div.css('line-height');
        if (_.isString(lineHeight)) {
            if (lineHeight.endsWith('px')) {
                lineHeight = lineHeight.substring(0, lineHeight.length - 2);
            }
            lineHeight = Number(lineHeight);
        }
        lineHeight = Math.floor(lineHeight);
        var height = $div.height();
        var rows = Math.floor((height + 1) / lineHeight);
        return rows;
    }

    /*function padNumber(num, maxChars) {

     maxChars = maxChars || 2;
     return ("0000" + n).slice(-4)
     }*/

    var instance = {

        /**
         * @deprecated
         * @param $container
         */
        createControls: function ($container) {
            instance.form.datePicker($container.find('.js-date'));
            instance.form.select($container.find('.js-select'));
            instance.form.checkbox($container.find('.js-checkbox'));
        },

        createLinks: function ($container, router, options) {
            instance.form.link($container.find('[data-link]'), router, options);
        },

        scrollToBottom: function (async, onDone)
        {
        	function doIt()
        	{
        		var $div = $('.ui-cardContent');
        		if ($div.length > 0)
        		{
        			var height = $div.height();
        			$div[0].scrollTop = height;
        			_.isFunction(onDone) && onDone.call(instance);
        		}
        	}
        	if (async)
        	{
        		setTimeout(function ()
        		{
        			doIt();
        		}, 1);
        	} else
        	{
        		doIt();
        	}
        },

        form: {

            autoSizeAllInContainer: function($container) {
                $container.find('.js-autosize').autosize();
            },

            editInPlace: function($hiddens, options) {
            	options = options || {};

            	if (_.isUndefined(options.canEdit))
            		options.canEdit = true;

                $hiddens.each(function () {
                    var $hidden = $(this);
                    var validatedControlNumber = newControlNumber();
                    $hidden.attr('validated-control', validatedControlNumber);
                    var placeholderText = options.emptyCaption || 'Введите значение';
                    var $root = $('<div></div>').addClass('ui-editInPlace-root').
                        addClass('js-validated-control-' + validatedControlNumber);
                    var initialValue = $hidden.val();
                    if (initialValue) {
                        $root.html(formatter.cleanTags(initialValue, true)).removeClass('ui-placeholder');
                    } else {
                        $root.html(placeholderText).addClass('ui-placeholder');
                    }

                    (options.viewerClasses) && ($root.addClass(options.viewerClasses));
                    options.classes && $root.addClass(options.classes);

                    if (options.canEdit === true)
                    	$root.click(function (evt)
                    	{
                    		evt.preventDefault();
                    		evt.stopPropagation();
                    		var $input = $('<textarea></textarea>').val($hidden.val()).
								attr('rows', textAreaRows($root)).
								attr('placeholder', placeholderText).
								addClass('ui-editInPlace-editor').
								insertAfter($root).focusout(function ()
								{
									applyEditorValue();
								})
								.click(function (evt)
								{
									evt.stopPropagation();
									if (environment.isMobileBrowser() && cordova.plugins.Keyboard.isVisible === false)
										cordova.plugins.Keyboard.show && cordova.plugins.Keyboard.show();
								}).autosize();
                    		setTimeout(function ()
                    		{
                    			$input.focus();
                    			$input.click();
                    		}, 300);
                    		$root.hide();
                    		$hidden.closest('.ui-card').click(onCardClicked);
                    		//_.isFunction(options.onClick) && options.onClick.call(instance, $root, options, $hidden);
                    	});

                    function applyEditorValue() {
                        var $input = $root.next('textarea.ui-editInPlace-editor');
                        var value = $input.val();
                        if (value) {
                            $root.html(formatter.cleanTags(value, true)).removeClass('ui-placeholder');
                        } else {
                            $root.html(placeholderText).addClass('ui-placeholder');
                        }

                        $hidden.closest('.ui-card').off('click', onCardClicked);
                        // TODO remove all $input event handlers
                        $input.remove();
                        $root.show();
                        $hidden.val($input.val()).trigger('change');
                    }
                    function onCardClicked(evt) {
                        evt && evt.preventDefault();
                        applyEditorValue();
                    }

                    options.decorateRoot && options.decorateRoot.call(instance, $root, options, $hidden);
                    $root.insertAfter($hidden);
                    $hidden.on('ui:refresh', function () {
                        _.isFunction(options.onRefresh) && options.onRefresh.call(instance, $root, options, $hidden);
                    });

                });
            },

            stateSwitch: function($hiddens, options) {
                options = options || {};
                var states = options.states || [];
                var stateMap = {};
                for (var i = 0; i < states.length; i++) {
                    var obj = {};
                    var nextIndex = (i < states.length - 1) ? i + 1 : 0;
                    obj.nextStateValue = states[nextIndex].value;
                    obj.stateIndex = i;
                    obj.nextStateIndex = nextIndex;
                    obj.state = states[i];
                    stateMap[states[i].value] = obj;
                }
                function switchToState(state, $root, $hidden) {
                    state && state.decorateState && state.decorateState.call(instance,
                        state, $root, options, $hidden);
                    $hidden.val(state.value).trigger('change');
                }
                instance.form.picker($hiddens, _.defaults(options, {
                    classes: 'ui-picker-stateSwitch',
                    decorateRoot: function($root, options, $hidden) {
                        options && options.initializeRoot && options.initializeRoot.call(instance, $root, options, $hidden);
                        var initialValue = $hidden.val();
                        var defaultState = _.find(states, function(state) {
                            return state.value === initialValue;
                        });
                        if (!defaultState) {
                            defaultState = _.find(states, function(state) {
                                return state.isDefault === true;
                            });
                        }
                        if (!defaultState && states.length > 0) {
                            defaultState = states[0];
                        }
                        switchToState(defaultState, $root, $hidden);
                    },
                    onClick: function ($root, options, $hidden) {
                        var currentValue = $hidden.val();
                        var nextStateValue = stateMap[currentValue].nextStateValue;
                        switchToState(stateMap[nextStateValue].state, $root, $hidden);
                    }
                }));
            },

            picker: function ($hiddens, options) {
                $hiddens.each(function () {
                    var $hidden = $(this);
                    var validatedControlNumber = newControlNumber();
                    $hidden.attr('validated-control', validatedControlNumber);
                    var $root = $('<div></div>').addClass('ui-picker').
                        addClass('js-validated-control-' + validatedControlNumber);
                    options.classes && $root.addClass(options.classes);
                    _.isFunction(options.builder) && options.builder.call(instance, $root, options);
                    $root.click(function (evt) {
                        evt.preventDefault();
                        evt.stopPropagation();
                        _.isFunction(options.onClick) && options.onClick.call(instance, $root, options, $hidden);
                    });
                    options.decorateRoot && options.decorateRoot.call(instance, $root, options, $hidden);
                    $root.insertAfter($hidden);
                    $hidden.on('ui:refresh', function () {
                        _.isFunction(options.onRefresh) && options.onRefresh.call(instance, $root, options, $hidden);
                    });
                });
            },

            destroy: function($hiddens) {
                /*$hiddens.each(function() {
                });*/
                $hiddens.next('.ui-picker').remove();
            },

            /**
             * Renders select controls based on hidden inputs.
             * Supported options: items, onRendered, onRefresh, classes, listClasses, emptyCaption
             * @param $hiddens
             * @param options
             */
            select: function ($hiddens, options) {
                instance.form.picker($hiddens, _.defaults(options, {
                    classes: 'ui-picker-select',
                    onClick: function ($selectRoot, options, $hidden) {
                        if (!options.items) {
                            return;
                        }
                        instance.popup.list({
                            items: options.items,
                            multiple: options.multiple,
                            onSelected: function (value, item) {
                                var caption;
                                if (isEmptyPickerValue(value)) {
                                    caption = options.emptyCaption || defaultEmptyCaption;
                                } else {
                                    if (_.isArray(value)) {
                                        caption = _.map(item, function(x) { return x.caption ? x.caption : x; }).join(', ');
                                    } else {
                                        caption = item.caption ? item.caption : value;
                                    }

                                }
                                $selectRoot.empty().append(pickerIcon(options, value)).append($pickerValueSpan({value: value}).text(caption));
                                $hidden.val(value).trigger('change');
                                options.decorateRoot && options.decorateRoot.call(instance, $selectRoot, options, $hidden);
                                options.onSelected && options.onSelected.call(instance, value, item);
                            },
                            classes: options.listClasses,
                            decorateItem: options.decorateItem,
                            usePhotos: options.usePhotos,
                            initialValues: $hidden.val().split(',')
                        });
                    },
                    onRefresh: function ($selectRoot, options, $hidden) {
                        var value = $hidden.val();
                        var item = _.find(options.items, function (item) {
                            return item == value || ((_.isObject(item)) && (item.value == value || item.id == value));
                        });
                        var caption = item ? (_.isObject(item) ? item.caption : item.toString()) :
                            (options.emptyCaption || defaultEmptyCaption);
                        caption && $selectRoot.empty().append(pickerIcon(options, value)).append($pickerValueSpan({value: value}).text(caption));

                    }
                }));
            },

            staticPicker: function($hiddens, options) {
                $hiddens.each(function () {
                    var $hidden = $(this);
                    var validatedControlNumber = newControlNumber();
                    $hidden.attr('validated-control', validatedControlNumber);
                    var $root = $('<div></div>').addClass('ui-picker').
                        addClass('js-validated-control-' + validatedControlNumber);
                    options.classes && $root.addClass(options.classes);
                    _.isFunction(options.builder) && options.builder.call(instance, $root, options);
                    options.decorateRoot && options.decorateRoot.call(instance, $root, options, $hidden);
                    $root.insertAfter($hidden);
                    var value = _.map(options.items, function(item) { return item.id; }).join(',');
                    $hidden.val(value);
                    _.each(options.items, function(item) {
                        $root.append(pickerIcon(options, item.id)).append($pickerValueSpan({value: item.id}).text(item.caption));
                    });
                });
            },

            listViewPicker: function($hiddens, options) {
            	instance.form.picker($hiddens, _.defaults(options, {
            		classes: 'ui-picker-select',
            		onClick: function ($root, options, $hidden)
            		{
            			if (!options.items)
            			{
            				return;
            			}
            			var listViewPageClass = options.listViewPageClass;
            			var originalApplyCallback = options.applyCallback;
            			var pageViewOptions = _.clone(options);
            			pageViewOptions.applyCallback = function (valueOrValues, itemOrItems)
            			{
            				if (_.isArray(valueOrValues))
            				{
            					$root.empty();
            					if (valueOrValues.length > 0)
            					{
            						var values = [];
            						var captions = [];

            						_.each(itemOrItems, function (item)
            						{
            							values.push(item.id);
            							captions.push(item.caption);
            						});
            						$root.append(pickerIcon(options, values.join(','))).append($pickerValueSpan({ value: values.join(',') }).text(captions.join(', ')));
            					}
            					else
            					{
            						$root.empty().append(pickerIcon(options, '')).append($pickerValueSpan({ value: '' }).text(options.emptyCaption || defaultEmptyCaption));
            					}

            					$hidden.val(valueOrValues.join(',')).trigger('change');

            				}
            				else
            				{
            					var value = itemOrItems.id;
            					var caption = itemOrItems.caption;
            					$root.empty().append(pickerIcon(options, value)).append($pickerValueSpan({ value: value }).text(caption));
            					$hidden.val(value).trigger('change');
            				}

            				originalApplyCallback && originalApplyCallback.call(this, valueOrValues, itemOrItems);
            				options.onSelected && options.onSelected.call(instance, value, itemOrItems);
            			}
            			if (options.multipleSelection)
            			{
            				var selectedIds = _.filter($hidden.val().split(','), function (item) { return item; });
            				pageViewOptions.preselectedIds = selectedIds;
            			}
            			else
            			{
            				var selectedId = $hidden.val();
            				selectedId && (pageViewOptions.preselectedIds = [selectedId]);
            			}

            			var pageView = new listViewPageClass(pageViewOptions);
            			instance.popup.popupPage(pageView, options.ignoreOtherElements);
            		},
            		onRefresh: function ($selectRoot, options, $hidden)
            		{
            			var valueOrValues = $hidden.val();
            			if (options.multipleSelection)
            			{
            				valueOrValues = valueOrValues.split(',');
            				var itemOrItems = _.filter(options.items, function (item)
            				{
            					return _.contains(valueOrValues, item.id);
            				});
            				$selectRoot.empty();
            				if (itemOrItems.length > 0)
            				{
            					var values = [];
            					var captions = [];

            					_.each(itemOrItems, function (item)
            					{
            						values.push(item.id);
            						captions.push(item.caption);
            					});
            					$selectRoot.append(pickerIcon(options, values.join(','))).append($pickerValueSpan({ value: values.join(',') }).text(captions.join(', ')));
            				}
            				else
            				{
            					$selectRoot.empty().append(pickerIcon(options, '')).append($pickerValueSpan({ value: '' }).text(options.emptyCaption || defaultEmptyCaption));
            				}
            			}
            			else
            			{
            				var value = $hidden.val();
            				var item = _.find(options.items, function (item)
            				{
            					return item == value || ((_.isObject(item)) && (item.value == value || item.id == value));
            				});
            				var caption = item ? (_.isObject(item) ? item.caption : item.toString()) :
                                (options.emptyCaption || defaultEmptyCaption);
            				caption && $selectRoot.empty().append(pickerIcon(options, value)).append($pickerValueSpan({ value: value }).text(caption));
            			}
            		}
            	}));

            },

            datePicker: function ($hiddens, options) {
                options = options || {};
                instance.form.picker($hiddens, _.defaults(options, {
                    classes: 'ui-picker-date',
                    icon: 'calendar',
                    onClick: function ($root, options, $hidden) {
                        instance.popup.calendar({
                            onSelected: function (selectedDate) {
                                var caption;
                                if (_.isEmpty(selectedDate)) {
                                    caption = options.emptyCaption || defaultEmptyCaption;
                                } else {
                                    caption = formatter.serviceDateToHuman(selectedDate);
                                }
                                $root.empty().append(pickerIcon(options, selectedDate)).append($pickerValueSpan({value: selectedDate}).text(caption));
                                $hidden.val(selectedDate).trigger('change');
                                options.onSelected && options.onSelected.call(instance, selectedDate);
                            },
                            selectedDate: $hidden.val(),
                            supportsEmptyValue: options.supportsEmptyValue,
                            emptyValueCaption: 'Дата не задана'
                        });
                    },
                    onRefresh: function ($root, options, $hidden) {
                        var strValue = $hidden.val();
                        var caption;
                        if (_.isEmpty(strValue)) {
                            caption = options.emptyCaption || defaultEmptyCaption;
                        } else {
                            var date = moment(strValue);
                            caption = formatter.serviceDateToHuman(date);
                        }
                        $root.empty().append(pickerIcon(options, strValue)).append($pickerValueSpan({value: strValue}).text(caption));
                    }
                }));
            },

            dateTimePicker: function($hiddens, options) {
                function applyResult(selectedDate, selectedTime, $hidden, $root) {
                    var selectedDateTime;
                    var caption;
                    if (selectedDate) {
                        var selectedMoment = moment(selectedDate + ' ' + selectedTime, 'YYYY-MM-DD HH:mm');
                        caption = selectedMoment.format(formatter.humanDateTimeFormat);
                        selectedDateTime = moment(selectedMoment).utc().format(formatter.serviceDateTimeFormat);
                    } else {
                        selectedDateTime = undefined;
                        caption = options.emptyCaption || defaultEmptyCaption;
                    }
                    $root.empty().append(pickerIcon(options, selectedDateTime)).append($pickerValueSpan({value: selectedDateTime}).text(caption));
                    $hidden.val(selectedDateTime).trigger('change');
                    options.onSelected && options.onSelected.call(instance, selectedDateTime);

                }
                function showMinuteSelector(selectedHour, selectedDate, $root, options, $hidden) {
                    var hour = selectedHour.substring(0, selectedHour.indexOf(':'));
                    function minuteFromDateTime(dateTime) {
                        return moment(dateTime).format('HH:mm');
                    }
                    instance.popup.minuteSelector({
                        hour: hour,
                        onSelected: function(selectedTime) {
                            applyResult(selectedDate, selectedTime, $hidden, $root);
                        },
                        selectedValue: $hidden.val() ? minuteFromDateTime($hidden.val()) : undefined
                    });
                }

                function showHourSelector(selectedDate, $root, options, $hidden) {
                    function hourFromDateTime(dateTime) {
                        return moment(dateTime).format('HH:00');
                    }
                    instance.popup.hourSelector({
                        onSelected: function(selectedHour) {
                            showMinuteSelector(selectedHour, selectedDate, $root, options, $hidden);
                        },
                        selectedValue: $hidden.val() ? hourFromDateTime($hidden.val()) : undefined
                    });
                }
                function showCalendar($root, options, $hidden) {
                    function dateFromDateTime(dateTime) {
                        return moment(dateTime).format('YYYY-MM-DD');
                    }
                    instance.popup.calendar({
                        onSelected: function (selectedDate) {
                            if (selectedDate) {
                                showHourSelector(selectedDate, $root, options, $hidden);
                            } else {
                                applyResult(undefined, undefined, $hidden, $root);
                            }

                        },
                        selectedDate: $hidden.val() ? dateFromDateTime($hidden.val()) : undefined,
                        supportsEmptyValue: options.supportsEmptyValue
                    });
                }
                options = options || {};
                instance.form.picker($hiddens, _.defaults(options, {
                    classes: 'ui-picker-date',
                    onClick: function ($root, options, $hidden) {
                        showCalendar($root, options, $hidden);
                    },
                    onRefresh: function ($root, options, $hidden) {
                        var strValue = $hidden.val();
                        var caption;
                        if (_.isEmpty(strValue)) {
                            caption = options.emptyCaption || defaultEmptyCaption;
                        } else {
                            var dateTimeMoment = moment(strValue);
                            caption = formatter.serviceDateTimeToHuman(dateTimeMoment);
                        }
                        $root.empty().append(pickerIcon(options, strValue)).append($pickerValueSpan({value: strValue}).text(caption));
                    }
                }));
            },

            timePicker: function() {
                // TODO need this?
            },

            checkbox: function ($hiddens) {
                $hiddens.each(function () {
                    var $hidden = $(this);
                    var initialValueAsString = $hidden.val();
                    var initialValue = (initialValueAsString === 'true');
                    var $root = $('<div></div>').addClass('ui-checkbox');

                    function makeOn() {
                        $root.addClass('ui-checkbox-on');
                        $root.children('i').removeClass('fa-square-o').addClass('fa-check-square-o');
                        $hidden.val(true);
                    }
                    function makeOff() {
                        $root.removeClass('ui-checkbox-on');
                        $root.children('i').removeClass('fa-check-square-o').addClass('fa-square-o');
                        $hidden.val(false);
                    }
                    $root.append($('<i class="fa"></i>'));
                    initialValue ? makeOn() : makeOff();
                    $root.click(function (evt) {
                        if ($root.hasClass('ui-checkbox-on')) {
                            makeOff();
                        } else {
                            makeOn();
                        }
                        $hidden.trigger('change');
                        evt.stopPropagation();
                        evt.preventDefault();
                    });
                    $root.insertAfter($hidden);
                });
            },

            link: function ($links, router, options) {
                $links.each(function () {
                    var $link = $(this);
                    var targetUrl = $link.attr('data-link');
                    $link.click(function (evt) {
                        options && options.onClick && options.onClick.call(instance, $link);
                        router.navigate(targetUrl, {trigger: true, replace: false});
                        evt.preventDefault();
                        evt.stopPropagation();
                        return false;
                    });
                });

            }
        },

        popup: {
        	addObstacle: function (options)
        	{
        		if ($('div.ui-ajax-obstacle').length > 0)
        			return;

        		$('input').add('textarea').blur().addClass('ui-temporarilyDisabled').prop('disabled', true);
        		var $obstacle = $('<div></div>').addClass('ui-ajax-obstacle');
        		options && options.obstacleClasses && $obstacle.addClass(options.obstacleClasses);

        		//$obstacle.on('touchstart touchend click', function (e)
        		//{
        		//	//var timeString = moment().format('HH:mm:ss.SSS');
        		//	//var eventType = e.type;
        		//	//var text = timeString + ' - ' + eventType + ' prevented';
        		//	//$('<div style="color:white;"></div>').text(text).prependTo($obstacle);
        		//	e.stopImmediatePropagation();
        		//	e.stopPropagation();
        		//	e.preventDefault();
        		//	console.log(e);
        		//	return false;
        		//});
        		//var $inp = $('<input type="text" style="width:100%; height:100%; opacity:0.01;" />').addClass('ui-temporarilyDisabled').prop('disabled', true);
        		//$obstacle.append($inp);
        		$obstacle.appendTo($body);
        		//$inp.focus();
        		//var allEvents = 'scroll seeking seeked show resize reset mousewheel mouseup mouseover mouseout mousemove mouseleave mouseenter mousedown keyup keypress keydown input focus ended drop dragstart dragover dragleave dragenter dragend drag dblclick contextmenu close click change abort wheel selectstart search paste cut copy beforepaste beforecut beforecopy touchstart MSPointerDown pointerdown MSGestureEnd touchmove MSPointerMove pointermove touchend MSPointerUp pointerup touchcancel MSPointerCancel pointercancel longTap doubleTap singleTap tap swipe swipeLeft swipeRight swipeUp swipeDown';
        		//$obstacle.bind(allEvents, function (evt)
        		//{
        		//	var timeString = moment().format('HH:mm:ss.SSS');
        		//	var eventType = evt.type;
        		//	var text = timeString + ' - ' + eventType;
        		//	$('<div style="color:white;"></div>').text(text).prependTo($obstacle);
        		//});
        	},
        	removeObstacle: function ()
        	{
        		$('div.ui-ajax-obstacle').remove();

        		$('.ui-temporarilyDisabled').removeAttr('disabled').removeClass('ui-temporarilyDisabled');
        	},

        	generic: function (options)
        	{
        		options = options || {};

        		$('input').add('textarea').blur().addClass('ui-temporarilyDisabled').prop('disabled', true);

        		var $obstacle = $('<div></div>').addClass('ui-popup-obstacle');//.appendTo($body);
        		options.obstacleClasses && $obstacle.addClass(options.obstacleClasses);
        		var closePopup = function ()
        		{
        			// TODO is this a correct implementation?
        			instance.popup.closeAll();
        			return false;
        		}
        		$obstacle.click(closePopup)/*.swipe(closePopup)*/;
        		$obstacle.on('touchstart touchend click', function (e) { closePopup(); e.stopPropagation(); e.preventDefault(); return false; });
        		$obstacle.appendTo($body);
        		var $popup = undefined;
        		if (options.retainableSelector)
        		{
        			$popup = $body.find(options.retainableSelector);
        		}
        		if (!$popup || $popup.length === 0)
        		{
        			$popup = $(document.createElement('div')).addClass('ui-popup');
        			options.classes && $popup.addClass(options.classes);
        			if (options.content)
        			{
        				var htmlContent = _.result(options, 'content');
        				$popup.append(htmlContent);
        			}
        			else if (_.isFunction(options.builder))
        			{
        				options.builder.call(instance, $popup, options);
        			}

        			$popup.css('display', 'none').appendTo(document.body);
        		}

        		//instance.helpers.requestAnimationFrame(function ()
        		//{
        			if (options.classes == 'ui-popup-sidebar')
        			{
        				var $toolbar = $popup.find('.ui-popup-sidebar-toolbar');
        				$toolbar.on('touchstart touchend click', function (e) { e.stopPropagation(); e.preventDefault(); return false; });
        				setTimeout(function () { $toolbar.off('touchstart touchend click'); }, 500);
        			}
        			$popup.show();
        			//instance.helpers.requestAnimationFrame(function ()
        			//{
        			//	$popup[0] && $popup.children().length > 0 && createPopupScroller($popup[0]);
        			//});
        		//});

        		//console.log(instance.helpers.requestAnimationFrame);
        		
        		return $popup;
        	},

            calendar: function (options) {
                var dayNames = ['пн', 'вт', 'ср', 'чт', 'пт', 'сб', 'вс'];
                options = options || {};
                function buildCalendarMarkup($root, options, startOfMonth) {
                    var endOfMonth = moment(startOfMonth).endOf('month').startOf('day');
                    var today = moment().startOf('day');
                    var selectedDate = options.selectedDate ? moment(options.selectedDate) : undefined;
                    var startOffset = startOfMonth.day() - 1;
                    (startOffset < 0) && (startOffset = 6);
                    var date = moment(startOfMonth);
                    date.add('days', -startOffset);
                    var $table = $('<table></table>').attr('startOfMonth', startOfMonth.format(formatter.serviceDateFormat));
                    var $navigationTr = $('<tr></tr>').addClass('ui-popup-calendar-navigation').appendTo($table);
                    var $navigationTd = $('<td></td>').attr('colspan', '7').css('border-bottom', 'solid 1px #e9e9e9').appendTo($navigationTr);

                    var navHtml = '<table style="width:100%;table-layout:fixed;border: 0;"><tbody><tr>'
					+ '<td style="vertical-align: top;">'
					   + '<a class="ui-button ui-button-inline ui-button-text js-previousMonth" style="height: auto; padding: 0; font-size: 100%">'
							+ '<i class="fa fa-arrow-left" style="margin-right: 0.5em;"></i>' + moment(startOfMonth).add('months', -1).format('MMMM')
						+ '</a>'
					+ '</td>'
					+ '<td style="vertical-align:top;">'
						+ '<h3 class="js-currentMonth" style="text-align: center;color: #676767">' + startOfMonth.format('MMMM, YYYY') + '</h3>'
					+ '</td>'
					+ '<td style="vertical-align: top;">'
						+ '<a class="ui-button ui-button-inline ui-button-text js-nextMonth" style="height: auto; padding: 0; font-size: 100%">'
							+ moment(startOfMonth).add('months', 1).format('MMMM') + '<i class="fa fa-arrow-right" style="margin-left: 0.5em;"></i>'
						+ '</a>'
					+ '</td>'
					+ '</tr></tbody></table>';
                    $navigationTd.html(navHtml)
						.on('click.ui-calendar-picker', 'a.js-previousMonth', function (e)
						{
							goToPreviousMonth($root, options);
						})
						.on('click.ui-calendar-picker', 'a.js-nextMonth', function (e)
						{
							goToNextMonth($root, options);
						});

                    var $dayNamesTr = $('<tr></tr>').addClass('ui-popup-calendar-dayNames').appendTo($table);
                    _.each(dayNames, function (dayName) {
                        $('<td></td>').text(dayName).appendTo($dayNamesTr);
                    });
                    var clickHandler = function (evt) {
                        var $td = $(this);
                        var selectedDate = $td.val();
                        instance.popup.closeAll();
                        _.isFunction(options.onSelected) && options.onSelected.call(instance, selectedDate);
                        evt.preventDefault();
                        evt.stopPropagation();
                    };
                    while (date.diff(endOfMonth, 'days') <= 0) {
                        var $tr = $('<tr></tr>').addClass('ui-popup-calendar-dates').appendTo($table);
                        for (var d = 1; d <= 7; d++) {
                            var $td = $('<td></td>').appendTo($tr);
                            $td.text(date.date());
                            $td.val(date.format(formatter.serviceDateFormat));
                            $td.click(clickHandler);
                            if (date.unix() < startOfMonth.unix() || date.unix() > endOfMonth.unix()) {
                                $td.addClass('ui-popup-calendar-otherMonth');
                            } else {
                                $td.addClass('ui-popup-calendar-mainMonth');
                            }
                            if (date.diff(today, 'days') === 0) {
                                $td.addClass('ui-popup-calendar-today');
                            }
                            if (selectedDate && date.diff(selectedDate, 'days') === 0) {
                                $td.addClass('ui-popup-calendar-selectedDate');
                            }
                            date.add('days', 1);
                        }
                    }

                    if (options.supportsEmptyValue)
                    {
                    	var $quickSelectTr = $('<tr></tr>').addClass('ui-popup-calendar-quickSelect').appendTo($table);
                    	var $quickSelectTd = $('<td></td>').attr('colspan', '7').appendTo($quickSelectTr);
                    	$('<div></div>').addClass('ui-button ui-button-text ui-popup-calendar-emptyDate').css('text-align', 'center').
                            appendTo($quickSelectTd).
                            text(options.emptyValueCaption || 'Дата не задана').
                            click(function (evt)
                            {
                            	instance.popup.closeAll();
                            	_.isFunction(options.onSelected) && options.onSelected.call(instance, undefined);
                            	evt.preventDefault();
                            	evt.stopPropagation();
                            });

                    }

                    $root.empty().append($table);
                }

                function goToOtherMonth($root, options, dMonth) {
                    var startOfMonth = moment($root.find('table').attr('startOfMonth'));
                    startOfMonth.add('months', dMonth);
                    buildCalendarMarkup($root, options, startOfMonth);
                }

                function goToNextMonth($root, options) {
                    goToOtherMonth($root, options, 1);
                }

                function goToPreviousMonth($root, options) {
                    goToOtherMonth($root, options, -1);
                }

                return instance.popup.generic(_.defaults(options, {
                    builder: function ($root, options) {
                        var startOfMonth = options.selectedDate ?
                            moment(options.selectedDate).startOf('month') : moment().startOf('month');
                        /*$root.swipeLeft(function () {
                            goToNextMonth($root, options);

                        });
                        $root.swipeRight(function () {
                            goToPreviousMonth($root, options);
                        });*/
                        buildCalendarMarkup($root, options, startOfMonth);
                    },
                    classes: 'ui-popup-calendar'
                }));
            },

            chooseFromTable: function (options) {
                options = options || {};
                if (!options.table) return;
                return instance.popup.generic(_.defaults(options, {
                    builder: function ($root, options) {

                        function clickHandler(evt) {
                            var $td = $(this);
                            var selectedValue = $td.val();
                            instance.popup.closeAll();
                            _.isFunction(options.onSelected) && options.onSelected.call(instance, selectedValue);
                            evt.stopPropagation();
                            evt.preventDefault();
                        }

                        var $table = $('<table></table>');
                        var rows = options.table;
                        for (var i = 0; i < rows.length; i++) {
                            var $tr = $('<tr></tr>').appendTo($table);
                            var columns = rows[i];
                            for (var j = 0; j < columns.length; j++) {
                                var $td = $('<td></td>').appendTo($tr);
                                var item = columns[j];
                                var text = _.isEmpty(item) ? '' :
                                    (_.isObject(item) ? (item.caption ? item.caption : item.value) : item.toString());
                                var value = _.isObject(item) ? (item.id ? item.id : item.value) : item;
                                (value === options.selectedValue) && ($td.addClass('ui-popup-table-selectedCell'));
                                item.classes && ($td.addClass(item.classes));
                                $td.text(text).val(value).click(clickHandler);
                                options.decorateItem && options.decorateItem.call(instance, $td, item, i, j);
                            }
                        }
                        $table.appendTo($root);
                    },
                    classes: 'ui-popup-table'
                }));
            },

            hourSelector: function (options) {
                options = options || {};
                return instance.popup.chooseFromTable(_.defaults(options, {
                    table: [
                        [
                            {value: '00:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '01:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '02:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '03:00', classes: 'ui-popup-table-rarelyUsed'}
                        ],
                        [
                            {value: '04:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '05:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '06:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '07:00', classes: 'ui-popup-table-rarelyUsed'}
                        ],
                        [
                            {value: '08:00'},
                            {value: '09:00'},
                            {value: '10:00'},
                            {value: '11:00'}
                        ],
                        [
                            {value: '12:00'},
                            {value: '13:00'},
                            {value: '14:00'},
                            {value: '15:00'}
                        ],
                        [
                            {value: '16:00'},
                            {value: '17:00'},
                            {value: '18:00'},
                            {value: '19:00'}
                        ],
                        [
                            {value: '20:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '21:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '22:00', classes: 'ui-popup-table-rarelyUsed'},
                            {value: '23:00', classes: 'ui-popup-table-rarelyUsed'}
                        ]
                    ]
                }));
            },

            minuteSelector: function (options) {
                options = options || {};
                options.hour = options.hour || 11;
                var table = [];
                var minutes = 0;
                var hour = padLeft(options.hour, 2);
                for (var i = 0; i < 4; i++) {
                    var columns = [];
                    for (var j = 0; j < 3; j++) {
                        var item = {};
                        var val = hour + ':' + padLeft(minutes, 2);
                        item.value = val;

                        if (val.endsWith('00') || val.endsWith('30')) {
                            item.classes = 'ui-popup-table-oftenUsed';
                        } else if (val.endsWith('05') || val.endsWith('25') ||
                            val.endsWith('35') || val.endsWith('55')) {
                            item.classes = 'ui-popup-table-rarelyUsed';
                        }

                        columns.push(item);
                        minutes += 5;
                    }
                    table.push(columns);
                }
                return instance.popup.chooseFromTable(_.defaults(options, {
                    table: table
                }));
            },

            error: function (options) {
                options = options || {};
                _.defaults(options, {
                    text: 'Ошибка вышла - вот о чем молчит наука. Хотели кока, а съели Кука.',
                    sendReportButtonCaption: 'Загрузить'
                });
                return instance.popup.generic(_.defaults(options, {
                    builder: function ($root, options) {
                        /*$('<div></div>').addClass('ui-popup-message-header').text(options.headerText).appendTo($root);
                         $('<div></div>').addClass('ui-popup-message-text').text(options.text).appendTo($root);*/
                        // TODO error message, button
                        $('<div></div>').addClass('ui-popup-error-text').text(options.text).appendTo($root);
                        $('<a></a>').attr('href', '#').addClass('ui-largeButton ui-important js-report').text(options.sendReportButtonCaption).appendTo($root).click(function (evt) {
                            options.onSendReport && options.onSendReport();
                            instance.popup.closeAll();
                            evt.preventDefault();
                            evt.stopPropagation();
                        });
                    },
                    classes: 'ui-popup-error'
                }));
            },

            message: function (options) {
                options = options || {};
                return instance.popup.generic(_.defaults(options, {
                    builder: function ($root, options) {
                        $('<div></div>').addClass('ui-popup-message-header').text(options.headerText).appendTo($root);
                        $('<div></div>').addClass('ui-popup-message-text').text(options.text).appendTo($root);
                    },
                    classes: 'ui-popup-message'
                }));
            },

            unsupportedMessage: function (options) {
                options = options || {};
                this.message(_.extend({}, {
                    text: 'Временно не поддерживается. Приносим извинения за доставленные неудобства.'
                }, options));
            },

            refreshIndicator: function (options) {
                options = options || {};
                return instance.popup.generic(_.defaults(options, {
                    builder: function ($root, options) {
                        $('<div></div>').addClass('ui-popup-refresh-text').text('Обновление... ').appendTo($root);
                        /*$('<div></div>').addClass('ui-popup-message-header').text(options.headerText).appendTo($root);
                         $('<div></div>').addClass('ui-popup-message-text').text(options.text).appendTo($root);*/
                    },
                    classes: 'ui-popup-refresh'
                }));
            },

            sidebar: function (options) {
                options = options || {};
                return instance.popup.generic(_.defaults(options, {
                    classes: 'ui-popup-sidebar',
                    obstacleClasses: 'ui-popup-sidebar-obstacle'
                    /*builder: function($root, options) {
                     $root.append('sadf sadf sadf sadf sadf sadf sadf sadf sadf sadf sadf sadf ');
                     }*/
                }));
            },

            list: function (options) {
                options = options || {};
                return instance.popup.generic(_.defaults(options, {
                    classes: 'ui-popup-list',
                    builder: function ($root, options) {
                        var items = options.items;
                        var $ul = $('<ul></ul>');
                        var onSelected = options.onSelected;
                        var selected = {
                            values: [],
                            items: [],
                            lis: []
                        };
                        if (options.multiple) {
                            var $applier = $('<li></li>').addClass('ui-list-menu');
                            $applier.append('<table><tr><td></td></tr></table>');
                            $applier.find('td').addClass('ui-apply').text('Выбрать');
                            $applier.click(function(evt) {
                                evt.preventDefault();
                                evt.stopPropagation();
                                onSelected && onSelected.call(instance, selected.values, selected.items, selected.lis, $root);
                                instance.popup.closeAll();
                            });
                            $applier.appendTo($ul);
                        }

                        _.each(items, function (item, itemIndex) {
                            var $li = $('<li></li>').attr('data-item-index', itemIndex).addClass('js-item');
                            if (_.isEmpty(item)) {
                                return;
                            } else if (_.isObject(item)) {
                                var itemValue = item.value ? item.value : item.id;
                                var itemCaption = item.caption;
                                if (item.icon) {
                                    $li.append($('<i></i>').
                                        addClass('tb-list-icon').addClass('fa fa-' + item.icon)).append(' ');
                                } else if (options.usePhotos) {
                                    $li.append($('<img/>').attr('src', photoBank.getPhotoUrl(itemValue)).attr('alt', itemCaption)).append(' ');
                                }
                                item.classes && $li.addClass(item.classes);
                                $li.attr('item-value', itemValue);
                                $('<span></span>').text(itemCaption).appendTo($li);
                            } else {
                                $li.attr('item-value', item.toString());
                                $('<span></span>').text(item.toString()).appendTo($li);
                            }

                            if (options.initialValues && _.contains(options.initialValues, itemValue)) {
                                selected.values.push(itemValue);
                                selected.items.push(item);
                                selected.lis.push($li);
                                $li.addClass('ui-popup-list-selectedItem');
                            }
                            options.decorateItem && options.decorateItem.call(instance, $li, item, $root);
                            $li.appendTo($ul);
                            $li[0].addEventListener('click', function (evt) {
                                evt.preventDefault();
                                evt.stopPropagation();
                                var $li = $(evt.currentTarget);
                                var item = items[$li.attr('data-item-index')];
                                if (options.multiple) {
                                    if ($li.hasClass('ui-popup-list-selectedItem')) {
                                        var indexToRemove = internalIndexOf(selected.lis, function($liItem) {
                                            return $li[0] === $liItem[0];
                                        });
                                        selected.lis.splice(indexToRemove, 1);
                                        selected.items.splice(indexToRemove, 1);
                                        selected.values.splice(indexToRemove, 1);
                                        $li.removeClass('ui-popup-list-selectedItem');
                                    } else {
                                        selected.values.push(itemValue);
                                        selected.items.push(item);
                                        selected.lis.push($li);
                                        $li.addClass('ui-popup-list-selectedItem');
                                    }
                                } else {
                                    onSelected && onSelected.call(instance, $li.attr('item-value'), item, $li, $root);
                                    instance.popup.closeAll();
                                }

                            }, false);
                        });
                        /*$root.on('click', '.js-item', 1, function (evt) {
                            evt.preventDefault();
                            evt.stopPropagation();
                            var $li = $(evt.currentTarget);
                            var item = items[$li.attr('data-item-index')];
                            if (options.multiple) {
                                if ($li.hasClass('ui-popup-list-selectedItem')) {
                                    var indexToRemove = internalIndexOf(selected.lis, function($liItem) {
                                        return $li[0] === $liItem[0];
                                    });
                                    selected.lis.splice(indexToRemove, 1);
                                    selected.items.splice(indexToRemove, 1);
                                    selected.values.splice(indexToRemove, 1);
                                    $li.removeClass('ui-popup-list-selectedItem');
                                } else {
                                    selected.values.push(itemValue);
                                    selected.items.push(item);
                                    selected.lis.push($li);
                                    $li.addClass('ui-popup-list-selectedItem');
                                }
                            } else {
                                onSelected && onSelected.call(instance, $li.attr('item-value'), item, $li, $root);
                                instance.popup.closeAll();
                            }

                        });*/
                        $ul.appendTo($root);
                    }
                }));
            },

            closeAll: function () {
                cleanPopupScrollers();
                $body.children('.ui-popup').not('.js-retainable').remove();
                $body.children('.ui-popup.js-retainable').hide();
                $body.children('.ui-popup-obstacle').remove();
                setTimeout(function() {
                    $('.ui-temporarilyDisabled').removeAttr('disabled').removeClass('ui-temporarilyDisabled');
                }, 550);

            },

            anyPopupsAreVisible: function() {
                var somePopupIsVisible = false;
                $body.children('.ui-popup').add($body.children('.ui-customPopup')).each(function() {
                    ($(this).css('display') !== 'none') && (somePopupIsVisible = true);
                });
                return somePopupIsVisible;
            },

            popupPage: function(pageView, ignoreOtherElements) {
                var $elementsToHide = $body.children().filter(function() {
                    return $(this).css('display') !== 'none' && (!$(this).hasClass('js-alwaysVisible'));
                });
                ignoreOtherElements || $elementsToHide.hide();
                _.extend(pageView, {
                    closePage: function() {
                        this.remove();
                        customPopupRegistry.unregister(this);
                        ignoreOtherElements || $elementsToHide.css('display', 'block'); // (we cannot use 'show' here)
                    },
                    close: function() {
                        this.closePage();
                    }
                });
                pageView.render();
                customPopupRegistry.register(pageView);
                $body.prepend(pageView.$el);
            }

        },

        helpers: {
            $pickerValueSpan: $pickerValueSpan,
            pickerIcon: pickerIcon,
            defaultEmptyCaption: defaultEmptyCaption,
            requestAnimationFrame: function (cb)
            {
            	var rAF = window.requestAnimationFrame ||
				window.webkitRequestAnimationFrame ||
				window.mozRequestAnimationFrame ||
				window.oRequestAnimationFrame ||
				window.msRequestAnimationFrame ||
				function (callback) { window.setTimeout(callback, 1000 / 60); };

            	return rAF(cb);
            }
        }

    };

    return instance;

});