﻿define(['underscore', 'jquery', 'utils/entityUtil', 'utils/urlParser', 'utils/customPopupRegistry', 'pubSub', 'urlBuilder', 'log', 'storage'],
	function (_, $, entityUtil, urlParserUtil, customPopupRegistry, pubSub, urlBuilder, log, storage)
	{
		var uiMode;
		var currentCardView;

		var cardScroller;

		var cardViewClasses = new Object(null);

		function guessUserInterfaceMode()
		{
			var userLayout = storage.get('UserLayoutMode');
			if (userLayout)
			{
				uiMode = userLayout;
				return;
			}

			var pixelRatio = getDevicePixelRatio();
			var $w = $(window);

			var actualWidth = $w.width() * pixelRatio;

			if (actualWidth >= 800)
			{
				uiMode = instance.modes.tablet;
			}
			else
			{
				uiMode = instance.modes.smartphone;
			}
		}

		function getDevicePixelRatio()
		{
			var ratio = 1;
			if (window.devicePixelRatio !== undefined)
			{
				ratio = window.devicePixelRatio;
			}
			else if (window.screen.systemXDPI !== undefined && window.screen.logicalXDPI !== undefined && window.screen.systemXDPI > window.screen.logicalXDPI)
			{// To account for zoom, change to use deviceXDPI instead of systemXDPI
				// Only allow for values > 1
				ratio = window.screen.systemXDPI / window.screen.logicalXDPI;
			}
			return ratio;
		};

		var instance = {
			mode: function ()
			{
				return uiMode;
			},
			modes: {
				smartphone: 'smartphone',
				tablet: 'tablet'
			},

			isTabletMode: function ()
			{
				return uiMode === this.modes.tablet;
			},

			isSmartphoneMode: function ()
			{
				return uiMode === this.modes.smartphone;
			},

			getDevicePixelRatio: function ()
			{
				return getDevicePixelRatio();
			},

			currentCardView: function ()
			{
				return currentCardView;
			},

			initLayout: function ()
			{
				guessUserInterfaceMode();

				var mode = this.mode();
				log.warn('Layout mode: ' + mode);
				$(document.body).removeClass('ui-layout-smartphone ui-layout-tablet').addClass('ui-layout-' + mode);
			},

			registerCardViewClass: function (entityType, cardViewClass)
			{
				cardViewClasses[entityType] = cardViewClass;
			},

			_showCard: function (cardView)
			{
				currentCardView && currentCardView.remove();
				$('#cardContainer').empty().append(cardView.$el);
				//cardView.$('.js-content').height($(window).height() - cardView.$('.js-header').height() * 2);
				currentCardView = cardView;
			},

			_updateScroller: function ()
			{
				return;
				if (cardScroller)
				{
					cardScroller.destroy();
				}
				if (currentCardView)
				{
					var $content = currentCardView.$('.js-content');
					if ($content && $content.length > 0)
					{
						cardScroller = new IScroll($content[0], {
							tap: false,
							preventDefault: true,
							//preventDefaultException: { tagName: /^(INPUT|TEXTAREA|BUTTON|A|SELECT|LI)$/ },
							click: false,

							bounce: false,
							//tap: 'click',
							//preventDefault: true,
							//click: true
							//preventDefaultException: { tagName: /^(INPUT|TEXTAREA|BUTTON|A|SELECT|LI)$/ }
						});
					}

				}

			},

			_refreshScroller: function ()
			{
				return;
				if (cardScroller)
				{
					cardScroller.refresh();
				}
			},

			/**
			 *
			 * @param entityType
			 * @param cardOptions can contain model or entityId properties
			 */
			openCard: function (entityType, cardOptions)
			{
				cardOptions = cardOptions || {};
				if (this.isSmartphoneMode())
				{
					if (!cardOptions.entityId /*&& !cardOptions.model*/)
					{
						// cannot open empty card in smartphone mode
						return;
					}
					else
					{
						var locationContext = {
							areaId: cardOptions.areaId,
							workspaceId: cardOptions.workspaceId
						};
						var replace = false;
						if (!_.isUndefined(cardOptions.replace))
							replace = cardOptions.replace;
						pubSub.trigger('routing:doNavigate', {
							fragment: cardOptions.explicitFragment || urlBuilder.entity(entityUtil.modeForEntity(entityType), cardOptions.entityId, 'edit', locationContext, cardOptions.activityId),
							options: { trigger: true, replace: replace }
						});
					}
				}
				else
				{
					// tablet mode
					var cardViewClass = cardViewClasses[entityType];
					var cardView = new cardViewClass(cardOptions);
					cardView.render();
					this._showCard(cardView);

					cardOptions.entityId && cardView.router && cardView.router.currentView && cardView.router.currentView.selectItem && cardView.router.currentView.selectItem(cardOptions.activityId || cardOptions.entityId);
				}
			},

			removeCard: function ()
			{
				currentCardView && currentCardView.remove();
			},

			onOrientationChange: function (event)
			{
				var prevMode = this.mode();

				this.initLayout();

				var currentMode = this.mode();

				if (prevMode == currentMode)
				{
					// экран недостаточно широкий или наоборот очень большой - режим не изменился делать ничего не нада
					log.debug('Screen Mode NotChanged: ' + prevMode + ' -> ' + currentMode);
					return;
				}

				customPopupRegistry.closeAll();

				log.debug('Screen Mode Changed: ' + prevMode + ' -> ' + currentMode);

				var lc = urlParserUtil.location().parse().asObject();
				console.log(lc);

				var router = window._appRouter;
				if (this.isSmartphoneMode())
				{
					// tablet -> smartphone
					// перевернули с 2ух колоноки одну
					var entityType = null;
					var modelId = null;
					if (currentCardView && currentCardView.model && currentCardView.model!=null)
					{
						entityType = currentCardView.entityType;
						modelId = currentCardView.model.id || 'new';
					}

					this.removeCard();

					if (entityType && modelId)
						this.openCard(entityType, { entityId: modelId, areaId: lc.area, workspaceId: lc.workspace });
				}
				else
				{
					// smartphone -> tablet
					// перевернули с одной колоноки в две
					if (!lc.edit && !lc.view)
					{
						router && router.currentView && router.currentView.cardEntityType && this.openCard(router.currentView.cardEntityType, _.extend({}, { router: router }));
					}
					else
					{
						if (router.currentView)
						{
							var entityType = router.currentView.entityType;
							router.navigate(router._lastToolRoute, { trigger: true, replace: true });
							this.openCard(entityType, _.extend({}, { router: router, entityId: lc.edit || lc.view, areaId: lc.area, workspaceId: lc.workspace }));
						}
					}
				}
			},
		};


		window.addEventListener('orientationchange', function ()
		{
			$(document.body).hide();

			setTimeout(function ()
			{
				instance.onOrientationChange();

				$(document.body).show();
				$(window).trigger('resize');

			}, 1000);
		});

		//pubSub.on('update:success', function() {
		//instance._updateScroller();
		//});

		//pubSub.on('card:resized', function() {
		//instance._refreshScroller();
		//});

		return instance;

	});
