﻿define(['underscore', 'jquery', 'storage'], function(_, $, storage) {

    var storageKey = '$activeThemeName';

    var themes = [
        {
            value: 'consensus',
            caption: 'Консенсус'
        },
        {
            value: 'greymood',
            caption: 'Московская осень'
        },
        {
            value: 'blue',
            caption: 'Лазурный берег'
        },
        {
            value: 'pink',
            caption: 'Розовые очки'
        },
        {
            value: 'moor',
            caption: 'Болотная площадь'
        }
    ];

    function saveActiveThemeName(themeName) {
        storage.set(storageKey, themeName);
    }

    function loadActiveThemeName() {
        var themeName = storage.get(storageKey);
        if (!themeName) {
            themeName = themes[0].value;
            saveActiveThemeName(themeName);
        }
        return themeName;
    }

    function removeThemeClasses($element) {
        var classes = $element[0].className.split(/\s/);
        var themeClasses = _.filter(classes, function(cls) { return cls.indexOf('ui-theme-') === 0; });
        (!_.isEmpty(themeClasses)) && $element.removeClass(themeClasses.join(' '));
    }

    function applyThemeToUI(themeName) {
        var $body = $('body');
        removeThemeClasses($body);
        $body.addClass('ui-theme-' + themeName);
    }

    (function() {
        var activeTheme = loadActiveThemeName();
        applyThemeToUI(activeTheme);
    })();



    return {
        supportedThemes: function() {
            return themes;
        },

        activateTheme: function(themeName) {
            applyThemeToUI(themeName);
            saveActiveThemeName(themeName);
        },

        activeThemeName: function() {
            return loadActiveThemeName();
        },

        removeThemeClasses: function($element) {
            removeThemeClasses($element);
        }

    };
});