﻿/**
 * @class BaseCardEditorView
 */
define([
    'views/BaseCardView',
    'views/refDataSupport',
    'views/bindingSupport',
    'models/BaseModel',
    'utils/validationUtil',
    'srv'
], function(
    BaseCardView,
    refDataSupport,
    bindingSupport,
    BaseModel,
    validationUtil,
    srv
    ) {

    return BaseCardView.extend({

        model: null,

        events: _.extend({}, BaseCardView.prototype.events),

        saveTriggerSelector: null,

        saving: null,

        initialize: function(options) {
            this.include(refDataSupport);
            this.include(bindingSupport);
            BaseCardView.prototype.initialize.apply(this, arguments);
            this.model = this.createModel();
            this.saving = false;
        },

        createModel: function() {
            // Переопределить в подклассе
            return new BaseModel();
        },

        updateData: function(updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            this.model || (this.model = this.createModel());
            if (this.options.entityId) {
                this.model.id = this.options.entityId;
                // Если задан идентификатор существующего объекта, загружаем его
                srv.fetch(_.defaults({
                    model: this.model,
                    forceUpdate: true,
                    success: function() {
                        that.onDataUpdated();
                    }
                }, updateOptions));
            } else {
                // Нечего загружать, сообщаем о готовности данных, чтобы перейти к рендерингу
                this.onDataUpdated();
            }
        },

        renderData: function() {
            this.bindData();
        },

        save: function() {
            var that = this;

            if (this.saving) {
                return;
            }

            this.saving = true;
            this.saveTriggerSelector && this.$(this.saveTriggerSelector).attr('disabled', true);

            this.$('input').blur();
            that.$('input').add('textarea').trigger('change');
            setTimeout(function () {
                that.$('input').add('textarea').trigger('change');

                that.beforeSave && that.beforeSave();


                // TODO check for unsaved changes

                srv.save({
                    //patch: true,
                    model: that.model,
                    validatedView: that,
                    success: function (model, response, options) {
                        // TODO navigate somewhere ?
                        that.onSaved && that.onSaved();
                    },
                    complete: function() {
                        that.saveTriggerSelector && that.$(that.saveTriggerSelector).removeAttr('disabled');
                        that.saving = false;
                    }
                });
            }, 150);
        },

        beforeSave: null,

        hasUnsavedChanges: function() {
            // TODO
        },

        isNewEntity: function() {
            return _.isEmpty(this.options.entityId);
        }
    });

});