﻿/**
 * @class BaseCompositeView
 * Базовое композитное представление (состоящее из нескольких дочерних частей).
 */
define([
    'underscore',
    'jquery',
    'views/BaseView'
], function(
    _,
    $,
    BaseView
    ) {
    return BaseView.extend({

        /**
         * Определения дочерних представлений.
         * Задаются в виде соответствий selector -> viewClass.
         * selector указывает на DOM-элемент внутри данного композитного представления,
         * где будет размещено дочернее представление.
         */
        childViewDefinitions: {},

        /**
         * Поле для объекта, где будут храниться экземпляры созданных дочерних представлений
         */
        childViewInstances: null,

        initialize: function(options) {
            var that = this;
            this.childViewInstances = {};
            BaseView.prototype.initialize.apply(this, arguments);
            _.each(this.childViewDefinitions, function(viewClass, selector) {
                var viewInstance = new viewClass(options);
                that.childViewInstances[selector] = viewInstance;
                if (_.isNull(viewInstance.container)) {
                    viewInstance.container = that;
                }
            });
        },

        /**
         * @deprecated not applicable in new approach
         */
        update: function() {
            var that = this;
            var thatArguments = arguments;

            // Вызываем базовый метод render, чтобы отобразить разметку, в которую будут встраиваться дочерние представления
            BaseView.prototype.render.apply(this, arguments);

            _.each(this.childViewInstances, function(viewInstance, selector) {
                if (!viewInstance.elementWasSet) {
                    viewInstance.setElement(that.$(selector));
                    viewInstance.elementWasSet = true;
                }
                if (viewInstance.update) {
                    viewInstance.update(thatArguments);
                } else if (viewInstance.render) {
                    viewInstance.render();
                }
            });
        },

        render: function() {
            _.each(this.childViewInstances, function(viewInstance) {
                if (viewInstance.render) {
                    viewInstance.render();
                }
            });
        },

        setInAllChildViews: function(attribute, value) {
            _.each(this.childViewInstances, function(viewInstance) {
                viewInstance[attribute] = value;
            });
        },

        callInAllChildViews: function(functionName) {
            var args = _.rest(arguments);
            _.each(this.childViewInstances, function(viewInstance) {
                viewInstance[functionName].apply(viewInstance, args);
            });
        },

        remove: function() {
            BaseView.prototype.remove.apply(this, arguments);
            _.each(this.childViewInstances, function(childView) {
                childView.remove && childView.remove();
            });
        }

    });
});