﻿/**
 * @class BaseEntityCardView
 */
define([
    'underscore',
    'jquery',
    'collections/BaseCollection',
    'views/BaseCardView',
    'utils/uiManager',
    'text!views/BaseEntityCardView_smartphoneHeader.html',
    'text!views/BaseEntityCardView_tabletHeader.html',
    'text!views/BaseEntityCardView_empty.html',
    'text!views/BaseEntityCardView_content.html',
    'settings',
    'srv','moment',
    'collections/activities/ActivityCollection',
    'views/common/EmbeddableActivitiesView',
    'views/common/EmbeddableAttachmentListView',
    'views/common/CommentComposerPageView',
    'views/common/EmbeddableCommentActivatorView',
    'views/common/BaseLowerToolbar',
    'views/common/CommentComposerPopupView',
    'views/deleteSupport',
    'utils/attachmentUtil',
    'utils/fileManager',
    'utils/entityUtil',
    'views/refDataSupport',
    'views/bindingSupport',
    'views/locationContextSupport',
    'environment'
], function(
    _,
    $,
    BaseCollection,
    BaseCardView,
    uiManager,
    smartphoneHeaderTemplate,
    tabletHeaderTemplate,
    emptyTemplate,
    contentTemplate,
    settings,
    srv,moment,
    ActivityCollection,
    EmbeddableActivitiesView,
    EmbeddableAttachmentListView,
    CommentComposerPageView,
    EmbeddableCommentActivatorView,
    BaseLowerToolbar,
    CommentComposerPopupView,
    deleteSupport,
    attachmentUtil,
    fileManager,
    entityUtil,
    refDataSupport,
    bindingSupport,
    locationContextSupport,
	environment
    ) {

    return BaseCardView.extend({

        entityType: null,

        emptyTemplate: emptyTemplate,
        contentTemplate: contentTemplate,

        supportsAttachments: true,
        supportsActivity: true,

        model: null,
        _modelLoaded: null,
        attachmentCollection: null,
        _attachmentCollectionLoaded: null,
        activityCollection: null,
        _activityCollectionLoaded: null,

        embeddableActivitiesView: null,
        embeddableAttachmentListView: null,

        saving: false,

        refData: {

        },

        events: {
            'click .js-cancelChanges': 'onCancelChangesClicked',
            'click .js-saveChanges': 'onSaveChangesClicked',
            'click .js-attach': 'onAttach',
            'click .js-comment': 'onComment',
            'click .js-delete': 'onDelete',
            'click .js-back': 'onBackClicked'

        },

        initialize: function(options) {
            this.include(refDataSupport);
            this.include(bindingSupport);
            this.include(deleteSupport);
            this.include(locationContextSupport);
            BaseCardView.prototype.initialize.apply(this, arguments);

            this.saving = false;

            if (uiManager.isTabletMode()) {
                this.headerTemplate = tabletHeaderTemplate;
            } else {
                this.headerTemplate = smartphoneHeaderTemplate;
            }
            if (this.isEmpty()) {
                this.template = this.emptyTemplate;
            }

            if (uiManager.isSmartphoneMode() && !this.isNew()) {
                this._initializeLowerToolbar();
            }
            if (options.date)
            	this.initialDate = options.date;

            var that = this;
            that.listenTo(that.pubSub, 'Push:Message:Recieved', function (message)
            {
            	if (that.editMode === true && that.model && that.model.id && that.model.id == message.data.entityID)
            	{
            		that.$('.js-saveChanges').hide();
            		that.$('.js-lowerToolbarButton').hide();

            		//alert('Данные карточки изменились. Отмените изменения.');
            		navigator.notification.alert('Кто-то вас опередил - данные были изменены другим участником. Мы не сможем сохранить внесенные вами правки, отмените ваши изменения.',
						function () { },
						'Данные были изменены',
						'Закрыть');
            	}
            });
        },

        isEmpty: function() {
            return !this.options.entityId && !this.options.model;
        },

        isNew: function() {
            return this.options.entityId === 'new';
        },

        entityLocationContext: function() {
            var locationContext;
            if (this.model && this.model.has('workspaceRef') && this.model.has('areaRef')) {
                locationContext = {
                    workspaceId: this.model.get('workspaceRef').id,
                    areaId: this.model.get('areaRef').id
                }
            } else {
                locationContext = this.locationContext();
            }
            return locationContext;
        },

        updateData: function (updateOptions)
        {
        	var that = this;

            if (this.isEmpty()) {
                this._modelLoaded = this._activityCollectionLoaded = this._attachmentCollectionLoaded = true;
                this.onDataUpdated();
                return;
            }

            this.pubSub.trigger('entity:progress');

            if (this.isNew())
            {
            	this.model = this.createModel();
            	if (this.initialDate)
            	{
            		if (this.entityType == 'task')
            			this.model.set('dueOn', { date: this.initialDate });
            		if (this.entityType == 'calendarEvent')
            		{
            			var h = moment().utc().hours();
            			var h1 = (h < 10 ? '0' : '') + h;
            			var h2 = h + 1;
            			h2 = (h2 < 10 ? '0' : '') + h2;
            			this.model.set({
            				startOn: { dateTime: this.initialDate + 'T' + h1 + ':00:00Z' },
            				dueOn: { dateTime: this.initialDate + 'T' + h2 + ':00:00Z' }
            			});
            		}
            		delete this.initialDate;
            	}

            	this._modelLoaded = this._activityCollectionLoaded = this._attachmentCollectionLoaded = true;
            	this.onDataUpdated();
            	this.startModelChangeTracking();
            	this.model.trigger('change');
            	this.focusedFieldSelector && setTimeout(function ()
            	{
            		that.$(that.focusedFieldSelector).click();
            	}, 200);
            	return;
            }

            this.model = this.createModel();
            this.model.id = this.options.entityId;
            this._modelLoaded = false;

            this.attachmentCollection = new BaseCollection(); // TODO use dedicated collection type?
            this._attachmentCollectionLoaded = !this.supportsAttachments;

            this.activityCollection = new ActivityCollection();
            this._activityCollectionLoaded = !this.supportsActivity;

            var customOptions = {};
            _.isFunction(that.fetchUrl) && (customOptions.url = that.fetchUrl());

            srv.fetch(_.defaults({
                model: this.model,
                success: function () {
                    that._modelLoaded = true;
                    that.onDataUpdated();
                    that.startModelChangeTracking();
                }
            }, customOptions, updateOptions));

            if (this.supportsAttachments) {
                srv.fetch(_.defaults({
                    collection: this.attachmentCollection,
                    url: that.model.url() + '/attachment',
                    success: function() {
                        that._attachmentCollectionLoaded = true;
                        that.onDataUpdated();
                    }
                }, updateOptions));
            } else {
                this._attachmentCollectionLoaded = true;
            }

            if (this.supportsActivity) {
                srv.fetch(_.defaults({
                    collection: this.activityCollection,
                    url: that.model.url() + '/activity',
                    success: function() {
                        that._activityCollectionLoaded = true;
                        that.onDataUpdated();
                    }
                }, updateOptions));
            } else {
                this._activityCollectionLoaded = true;
            }

        },

        onDataUpdated: function() {
            if (this._modelLoaded && this._attachmentCollectionLoaded && this._activityCollectionLoaded) {
                this.renderDataWrapper();
            }
        },

        render: function ()
        {
        	if (!this.isEmpty())
        		this.pubSub.trigger('entity:progress');
            BaseCardView.prototype.render.apply(this, arguments);
            this.$el.addClass('ui-entityCard');

            if (this.isEmpty())
            	this.pubSub.trigger('entity:stopProgress');
        },

        remove: function ()
        {
        	this.pubSub.trigger('entity:stopProgress');
        	BaseCardView.prototype.remove.apply(this, arguments);
        },

        renderData: function ()
        {
        	if (this.model)
        	{
        		this.$('.js-data').html(this.dataTemplate ? this.dataTemplate.call(this) : '');
        		this.bindData();
        		this.supportsAttachments && this.renderEmbeddableAttachmentListView();
        		this.supportsActivity && this.renderEmbeddableActivitiesView();
        		this._createControls();

        		var perms = this.model.get('permissions');
        		if (perms)
        		{
        			if (perms.canDelete === false)
        			{
        				if (uiManager.isTabletMode())
        					this.$('.js-viewer-button').find('.js-delete').parent().hide();
        				else
        					this.$('.js-lowerToolbarButton[item-name="delete"]').data('allow', false).find('i.fa').hide();
        			}

        			if (perms.canEdit === false)
        			{
        				if (uiManager.isTabletMode())
        					this.$('.js-viewer-button').find('.js-attach').parent().hide();
        				else
        					this.$('.js-lowerToolbarButton[item-name="attach"]').data('allow', false).find('i.fa').hide();
        			}
        		}
        	}
        },

        renderEmbeddableAttachmentListView: function() {
            if (this.embeddableAttachmentListView) {
                this.embeddableAttachmentListView.remove();
            }
            this.embeddableAttachmentListView = this.createChildView(EmbeddableAttachmentListView, {
            	collection: this.attachmentCollection,
            	parentPerms: _.clone(this.model.get('permissions'))
            });
            this.embeddableAttachmentListView.render();
            this.embeddableAttachmentListView.$el.appendTo(this.$('.js-attachments'));
        },

        renderEmbeddableActivitiesView: function() {
            if (this.embeddableActivitiesView) {
                this.embeddableActivitiesView.remove();
            }
            this.embeddableActivitiesView = this.createChildView(EmbeddableActivitiesView, {
                collection: this.activityCollection,
                forEntity: true,
				activityId: this.options.activityId
            });
            this.embeddableActivitiesView.render();
            var $activitiesContainer = this.$('.js-activities');
            $activitiesContainer.empty();
            this.embeddableActivitiesView.$el.appendTo($activitiesContainer);
        },

        updateActivities: function() {
            var that = this;
            this.activityCollection = new ActivityCollection();
            this._activityCollectionLoaded = false;
            srv.fetch({
                collection: this.activityCollection,
                url: that.model.url() + '/activity',
                success: function() {
                    that._activityCollectionLoaded = true;
                    that.renderEmbeddableActivitiesView();
                }
            });
        },

        startModelChangeTracking: function() {
            var that = this;
            if (!this.model) return;
            this.listenTo(this.model, 'change', function() {
                that.onModelChanged();
            });
        },

        onModelChanged: function() {
            this.$('.js-viewer-button').hide();
            this.$('.js-editor-button').show();
            this.editMode = true;
        },

        onModelChangesHandled: function() {
            this.$('.js-editor-button').hide();
            this.$('.js-viewer-button').show();
            this.editMode = false;
        },

        onCancelChangesClicked: function (evt)
        {
        	if (environment.isMobileBrowser() && cordova.plugins.Keyboard.isVisible === true)
        		cordova.plugins.Keyboard.close && cordova.plugins.Keyboard.close();

        	this.$('.js-saveChanges').show();
        	this.$('.js-lowerToolbarButton').show();

        	evt && evt.preventDefault();
        	this.onModelChangesHandled();
        	//if (uiManager.isTabletMode()) {
        	if (this.isNew() === true)
        	{
        		if (uiManager.isTabletMode())
        		{
        			uiManager.openCard(this.entityType);
        		}
        		else
        			this.router.navigateBack();
        	}
        	else
        		this.update();
        	//} else {
        	//this.router.navigateBack();
        	//}

        },

        onSaveChangesClicked: function (evt)
        {
        	if (environment.isMobileBrowser() && cordova.plugins.Keyboard.isVisible === true)
        		cordova.plugins.Keyboard.close && cordova.plugins.Keyboard.close();

        	evt && evt.preventDefault();

        	var $but = $(evt.currentTarget);

        	if ($but.hasClass('disabled'))
        		return false;

        	$but.html('<i class="fa fa-refresh fa-spin"></i>&nbsp;Сохранение...');
        	this.saveChanges();
        },

        saveChanges: function() {
            var that = this;

            if (this.saving) {
                return;
            }

            this.saving = true;
            this.$('input').blur();
            that.$('input').add('textarea').trigger('change');
            setTimeout(function () {
                that.$('input').add('textarea').trigger('change');

                that.beforeSave && that.beforeSave();

                if (!that.model.has('workspaceRef')) {
                    var locationContext = that.locationContext();
                    if (!_.isEmpty(locationContext)) {
                        that.model.set('workspaceRef', { id: locationContext.workspaceId });
                    }
                }

                srv.save({
                    //patch: true,
                    model: that.model,
                    validatedView: that,
                    success: function (model, response, options) {
                        that.onModelChangesHandled();
                        that.onSaved && that.onSaved();
                        that.reloadAfterSave();
                    },
                    complete: function ()
                    {
                    	that.$('.js-saveChanges').text('Сохранить');
                        that.saveTriggerSelector && that.$(that.saveTriggerSelector).removeAttr('disabled');
                        that.saving = false;
                    }
                });
            }, 150);
        },

        beforeSave: null,

        reloadAfterSave: function ()
        {
        	var that = this;
        	if (this.entityType)
        	{
        		var wasNew = that.isNew();
        		setTimeout(function ()
        		{
        				uiManager.openCard(that.entityType, _.defaults({
        					router: that.router,
        					entityId: that.model.id,
        					explicitFragment: that.reloadAfterSaveExplicitFragment ? that.reloadAfterSaveExplicitFragment() : undefined,
							replace: wasNew,
        				}, that.options));

        				if (uiManager.isTabletMode())
        				{
        					setTimeout(function ()
        					{
        						that.router && that.router.tryForceUpdateCurrentView && that.router.tryForceUpdateCurrentView();
        					}, 100);
        				}
        				else if (wasNew === false)
        					that.render();
        			//else
        			//{
        			//	that.router.reloadCurrentRoute();
        			//	//that.router.navigate(that.urlBuilder.generic(entityUtil.modeForEntity(that.entityType), {
        			//	//    view: that.model.id
        			//	//}), {trigger: true, replace: true});
        			//}

        		}, 100);
        	}

        },

        reloadAfterSaveExplicitFragment: null,

        onAttach: function (evt)
        {
        	var that = this;
        	evt && evt.preventDefault();
        	attachmentUtil.showAttachUI({
        		onAttachmentCreated: function (imageUrl)
        		{
        			that.pubSub.trigger('update:start');
        			that.pubSub.trigger('loader:start');
        			that.pubSub.trigger('entity:progress');
        			fileManager.uploadAttachment(imageUrl, {
        				target: {
        					entityType: that.model.entityType,
        					entityId: that.model.id
        				},
        				fileName: 'image-' + that.formatter.nowAsFileNameString() + '.jpg',
        				success: function ()
        				{
        					that.pubSub.trigger('loader:stop');
        					that.pubSub.trigger('entity:stopProgress');
        					that.update({ forceUpdate: true });
        					//that.updateAttachments();
        				},
        				error: function ()
        				{
        					that.pubSub.trigger('loader:stop');
        					that.pubSub.trigger('entity:stopProgress');
        				},
        			});
        		}
        	});
        },

        onComment: function(evt) {
            var that = this;
            evt && evt.preventDefault();
            that.ui.scrollToBottom(true, function() {
                var popupView = new CommentComposerPopupView({
                    commentTarget: {
                        entityType: that.model.entityType,
                        entityId: that.model.id
                    },
                    onPosted: function() {
                        that.updateActivities();
                    }
                });
                popupView.render();
            });
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh',
                            width: '25%'
                        },
                        {
                            builtin: 'delete',
                            width: '25%'
                        },
                        {
                            name: 'attach',
                            icon: 'paperclip',
                            onClicked: function($td, item) {
                                that.onAttach();
                            }
                        },
                        {
                            name: 'comment',
                            icon: 'comment-o',
                            onClicked: function($td, item) {
                                that.onComment();
                            }
                        }

                    ]
                });
            }
        },

        onBackClicked: function (e)
        {
        	if (uiManager.isSmartphoneMode())
        	{
        		this.router.navigateBack();
        	}
        	else
        		uiManager.openCard(this.entityType);

        	//if (e)
        	//{
        	//	e.preventDefault();
        	//	e.stopPropagation();
        	//}

        	//return false;
        }


    });

});
