﻿define([
    'underscore',
    'jquery',
    'views/BaseView',
	'utils/uiManager','utils/scrollStack',
    'text!views/BasePageView.html',
    'text!views/BasePageView_header.html'
], function(
    _,
    $,
    BaseView,
	uiManager, scrollStack,
    template,
    headerTemplate
    ) {

    return BaseView.extend({

        template: template,
        headerTemplate: headerTemplate,
        contentTemplate: null,

        lowerToolbarClass: null,
        lowerToolbar: null,

        events: {
            //'swipeDown .js-content': 'onRefreshRequested'
            // Swipe works when we have preventDefault in onTouchStart, but it blocks the scroll
            //'touchstart .js-content': 'onTouchStart'
            //'scroll js-content': 'onScroll'
        },

        initialize: function ()
        {
            BaseView.prototype.initialize.apply(this, arguments);
            this.$el.addClass('ui-page');
           	this.listenTo(this.pubSub, 'update:success', this.onUpdateSuccess);
        },

        onRefreshRequested: function ()
        {
        	this.onRefreshRequestedBase();

        	this.update({
        		forceUpdate: true
        	});
        },

        onRefreshRequestedBase: function ()
        {
        	if (uiManager.isSmartphoneMode())
        	{
        		this._lastSelectedId = null;
        		scrollStack.clear();
        	}
        },

        /*onTouchStart: function(evt) {
            evt.preventDefault();
        },

        onScroll: function(evt) {
            console.log('scroll');

        },*/

        /*updateWithIndicator: function() {
            if (this.updateData*//* || !_.isEmpty(this.refData)*//*) {
                this.ui.popup.refreshIndicator();
                this.update();
            }
        },*/

        render: function() {
            BaseView.prototype.render.apply(this, arguments);
            this.renderLowerToolbar();
        },

        $header: function() {
            return this.$('.js-header');
        },

        $content: function() {
            return this.$('.js-content');
        },

        renderLowerToolbar: function() {
            if (this.lowerToolbar) {
                this.lowerToolbar.remove();
            }
            this.$('.js-lowerToolbar').remove();
            if (!this.lowerToolbarClass) {
                return;
            }
            this.lowerToolbar = new this.lowerToolbarClass({
                parentView: this
            });
            this.lowerToolbar.render();
            this.$('.ui-content-wrapper').after(this.lowerToolbar.$el);
        },

        fitElements: function() {
            this.$('.js-fitToPage').height($(window).height() - this.$('.ui-pageHeader').height());
        },

        onUpdateSuccess: function (cid)
        {
        	if (this.notSelectableByCard === true)
        		return;

        	if (cid === this.cid)
        	{
        		var that = this;
        		that.ui.helpers.requestAnimationFrame(function ()
        		{
        			if (uiManager.isSmartphoneMode())
        			{
        				scrollStack.tryRestore(that);
        			}

        			if (that.router.prevId)
        			{
        				that._selectElementByEntityId(that.router.prevId);
        				delete that.router.prevId;
        			}
        			else
        				that._selectElementByEntityId(that._lastSelectedId);
        		});

        		that.pubSub.trigger('tool:stopProgress');
        	}
        },

        _selectElementByEntityId: function (entityId)
        {
        	var that = this;
        	if (!entityId)
        		return;

        	that._lastSelectedId = entityId;

        	var $list = this.$('.ui-content');
        	if ($list.length > 0)
        	{
        		var $li = null;

        		if (_.isUndefined(that.cardEntityType) || that.cardEntityType == 'task' || that.dayAgenda===true)
        			$li = $list.find('li[entity-id="' + entityId + '"]');
        		else if (that.cardEntityType == 'activity')
        			$li = $list.find('a[data-entity-id="' + entityId + '"]').parent();
				else if (that.cardEntityType == 'notification')
        			$li = $list.find('a[data-activity-id="' + entityId + '"]').parent();
        		else if (that.cardEntityType == 'discussion')
        			$li = $list.find('a[entity-id="' + entityId + '"]').parent();
        		else if (that.cardEntityType == 'calendarEvent')
        			$li = $list.find('td.calendar-day-' + entityId);

        		var needAutoRemove = false;
        		if (that.router.prevId || that.needAutoRemove === true)
        			needAutoRemove = true;

        		if ($li && $li.length > 0)
        		{
        			$li.addClass('selected');

        			if (needAutoRemove === true)
        				setTimeout(function ()
        				{
        					$li && $li.length > 0 && $li.removeClass('selected');
        				}, 5000);
        		}
        	}
        },

        selectItem: function (entityId)
        {
        	var $list = this.$('.ui-content');
        	if (this.cardEntityType == 'calendarEvent' && !this.dayAgenda)
        		$list.find('td.selected').removeClass('selected');
        	else
        		$list.find('li.selected').removeClass('selected');

        	this._selectElementByEntityId(entityId);
        },
    });

});
