﻿define([
    'underscore',
    'jquery',
    'views/BaseView'
], function(
    _,
    $,
    BaseView
    ) {

    return BaseView.extend({

        childViewDefinitions: {},

        activeView: null,
        activeViewName: null,
        containerSelector: null,

        initialize: function(options) {
            BaseView.prototype.initialize.apply(this, arguments);
            options.activeViewName && (this.activeViewName = options.activeViewName);
            options.activeViewName && this._createActiveView();
        },

        _createActiveView: function(options) {
            this._removeCurrentActiveView();
            if (this.activeViewName) {
                options = options || {};
                var viewConstructor = this.childViewDefinitions[this.activeViewName];
                if (viewConstructor) {
                    var view = new viewConstructor(_.defaults(options, _.omit(this.options, 'el')));
                    view.$el.appendTo(this.$(this.containerSelector));
                    _.isNull(view.container) && (view.container = this);
                    this.activeView = view;
                }
            }
        },

        _removeCurrentActiveView: function() {
            if (this.activeView) {
                this.activeView.remove && this.activeView.remove();
                this.activeView = null;
                this.$(this.containerSelector).empty();
            }
        },

        switchTo: function(viewName, options, recreate) {
            if (!recreate && viewName === this.activeViewName) return;
            this.activeViewName = viewName;
            this._createActiveView(options);
            return this;
        },

        updateActiveView: function() {
            if (this.activeView && this.activeView.update) {
                this.activeView.update.apply(this.activeView, arguments);
            } else {
                this.renderActiveView();
            }
        },

        renderActiveView: function() {
            this.activeView && this.activeView.render && this.activeView.render.apply(this.activeView, arguments);
        },

        remove: function() {
            BaseView.prototype.remove.apply(this, arguments);
            this._removeCurrentActiveView();
        }



    });

});