﻿/**
 * @class BaseToolView
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'text!views/BaseToolView_header.html',
    'settings',
    'utils/cache/structureCache',
    'utils/toolUtil',
    'views/refDataSupport',
    'views/locationContextSupport',
    'utils/uiManager'
], function (_, $, BasePageView, headerTemplate, settings, structureCache, toolUtil, refDataSupport, locationContextSupport, uiManager)
{
    return BasePageView.extend({

        cardEntityType: null,

        headerTemplate: headerTemplate,
        hasSidebar: true,

        events: _.extend({
            'click .js-toggleSidebar': 'onToggleSidebarClicked',
            //'swipe .js-toggleSidebar': 'onTestSwipe',
            'click .js-rightButton': 'onRightButtonClicked',
            'click .js-toggleToolSelector': 'onToggleToolSelector'
        }, BasePageView.prototype.events),

        refData: {
            structure: structureCache.refDataLoader
        },

        initialize: function (options) {
            var that = this;
            this.include(refDataSupport);
            this.include(locationContextSupport);
            BasePageView.prototype.initialize.apply(this, arguments);

            this.listenTo(this.pubSub, 'card:showDefault', function() {
                that.tryShowCard();
            });
        },

        rightButton: null,


        onToggleSidebarClicked: function(evt) {
        	evt.preventDefault();
        	this.showSidebar();
        	return false;
        },

        onRightButtonClicked: function(evt) {
            if (this.rightButton) {
                if (this.rightButton.href) {
                    this.router.navigate(this.rightButton.href, {trigger: true, replace: false});
                } else if (this.rightButton.handler) {
                    var handler = this[this.rightButton.handler];
                    handler && _.isFunction(handler) && handler.call(this, evt);
                }
            }
            evt.preventDefault();
        },

        onToggleToolSelector: function(evt) {
            this.toggleToolSelector();
            evt.preventDefault();
        },

        toggleToolSelector: function() {
            var that = this;
            var structureSection = this.structureSectionByLocationContext();
            var tools = structureSection ? (!_.isEmpty(structureSection.workspace.tools) ? structureSection.workspace.tools : structureSection.area.tools) : this.refData.structure.sharedTools;
            var popupListItems = [];
            _.each(tools, function(tool) {
                if (toolUtil.isToolSupported(tool.toolName)) {
                    var href = structureSection ? that.urlBuilder.tool(tool.toolName, structureSection.area.id, structureSection.workspace.id) : that.urlBuilder.tool(tool.toolName);
                    popupListItems.push({
                        value: tool.toolName,
                        caption: tool.caption,
                        href: href,
                        icon: toolUtil.toolIcon(tool.toolName)
                    });
                }

            });
            this.ui.popup.list({
                items: popupListItems,
                classes: 'ui-popup-list tb-popup-toolSelector',
                onSelected: function(value, item) {
                    that.router.navigate(item.href, {trigger: true, replace: false});
                }
            });

        },

        render: function ()
        {
        	this.pubSub.trigger('tool:progress');
            BasePageView.prototype.render.apply(this, arguments);
            this.tryShowCard();
        },

        tryShowCard: function(cardOptions) {
            if (!this.cardEntityType) {
                return;
            }
            uiManager.openCard(this.cardEntityType, _.extend({}, {router: this.router}, cardOptions));
        },

        renderLocationContextCaption: function() {
            this.$header().find('.js-locationContextCaption').text(this.locationContextCaption());
        },

        onBackButton: function() {
            // exit app if we run under Cordova
            navigator && navigator.app && navigator.app.exitApp && navigator.app.exitApp();
        },

        update: function ()
        {
        	this._mainDataLoaded = false;
        	this.pubSub.trigger('tool:progress');

        	BasePageView.prototype.update.apply(this, arguments);

        	//this.pubSub.trigger('tool:stopProgress');
        },

        onRefreshRequested: function ()
        {
        	this.pubSub.trigger('tool:progress');
        	this.onRefreshRequestedBase && this.onRefreshRequestedBase();

        	this.update({
        		forceUpdate: true
        	});

        	if (uiManager.isTabletMode())
        	{
        		var currentCardView = uiManager.currentCardView();
        		if (currentCardView && currentCardView.update)
        		{
        			currentCardView.update({
        				forceUpdate: true
        			});
        		}
        	}

        },
    });
});