﻿define([
    'underscore',
    'jquery',
    'backbone',
    'pubSub',
    'settings',
    'log',
    'urlBuilder',
    'i18n!nls/global',
    'utils/formUtil',
    'utils/formatter',
    'utils/ui',
    'utils/photoBank'], function (
    _,
    $,
    Backbone,
    pubSub,
    settings,
    log,
    urlBuilder,
    i18n,
    formUtil,
    formatter,
    ui,
    photoBank
    )
    {

    	return Backbone.View.extend({

    		template: null,

    		router: null,
    		pubSub: null,
    		log: null,
    		urlBuilder: null,
    		i18n: null,
    		formatter: null,
    		ui: null,
    		photoBank: photoBank,

    		views: null,

    		autoUpdate: true,

    		initialize: function (options)
    		{
    			var that = this;

    			options = options || {};
    			Backbone.View.prototype.initialize.apply(this, arguments);

    			_.isBoolean(options.autoUpdate) && (this.autoUpdate = options.autoUpdate);

    			// Устанавливаем настройк представления
    			_.isObject(options) && this.setOptions(options);

    			// Устанавливаем стандартные вспомогательные объекты
    			this.pubSub = pubSub;
    			this.log = log;
    			this.urlBuilder = urlBuilder;
    			this.i18n = i18n;
    			this.formatter = formatter;
    			this.ui = ui;
    			this.photoBank = photoBank;

    			// Инициализируем массив дочерних представлений
    			this.views = [];

    			this.prepareRefData && this.prepareRefData();

    			// Отображаем представление сразу, если это задано в настройках
    			if (this.options['doRender'])
    			{
    				//setTimeout(function ()
    				//{
    					that.render();
    				//}, 1);

    			}
    		},

    		/**
			 * Устанавливает настройки данного представления
			 * @param {Object} options настройки представления
			 */
    		setOptions: function (options)
    		{
    			var opts = _.extend({}, options);
    			if (opts.el)
    			{
    				this.setElement(opts.el);
    				opts.el = this.el;
    			}
    			opts.router && (this.router = opts.router);
    			this.options = opts;
    		},

    		/**
			 * Отображает представление.
			 * @param template (необязательный) шаблон, который необходимо отобразить.
			 * Если параметр template не указан, то отображается шаблон this.template
			 */
    		render: function (template)
    		{
    			var that = this;
    			template || (template = this.template);

    			if (template && that.$el && that.$el.length > 0)
    			{

    				that.$el[0].innerHTML = null;
    				that.$el.empty();

    				that.$el[0].insertAdjacentHTML('beforeend', template.call(that, that.options));

    				//that.rendered = true;

    			}

    			//that.ui.helpers.requestAnimationFrame(function ()
    			//{
    				
    				//this._createStandardControls();
    				if (that.autoUpdate)
    				{
    					setTimeout(function ()
    					{
    						that.update();
    					}, 1);

    				}

    			//});
    			
    		},

    		update: function (updateOptions)
    		{
    			var that = this;

    			if (this.updateData || !_.isEmpty(this.refData))
    			{
    				this.pubSub.trigger('update:start');
    			}

    			if (updateOptions && updateOptions.forceUpdate)
    			{
    				if (this.refData)
    				{
    					_.each(this.refData, function (value, key)
    					{
    						that.prepareRefData();
    					});
    				}
    				if (this._mainDataLoaded && this.updateData)
    				{
    					this._mainDataLoaded = false;
    				}
    			}

    			this.startRefDataLoading && this.startRefDataLoading();
    			this.updateData && this.updateData(updateOptions);
    		},

    		updateData: null,

    		renderData: function ()
    		{
    			// override
    		},

    		onDataUpdated: function ()
    		{
    			if (this.onDataLoaded)
    			{
    				this.onDataLoaded()
    			}
    			else
    			{
    				this.renderDataWrapper();
    			}
    		},

    		renderDataWrapper: function ()
    		{
    			var that = this;
    			that.ui.helpers.requestAnimationFrame(function ()
    			{
    				that.renderData();

    				that.triggerUpdateSuccess();
    			});
    		},

    		triggerUpdateSuccess: function ()
    		{
    			this.pubSub.trigger('update:success', this.cid);
    		},

    		include: function (supportObject)
    		{
    			_.defaults(this, supportObject);
    		},

    		/**
			 * @deprecated
			 * @private
			 */
    		_createStandardControls: function ()
    		{
    			//formUtil.editInPlace(this.$('.j-editInPlace'));
    		},

    		createChildView: function (viewConstructor, options)
    		{
    			options = options || {};
    			options = _.defaults(options, _.omit(this.options, 'el'));
    			var childView = new viewConstructor(options);
    			_.isNull(childView.container) && (childView.container = this);
    			this.views && this.views.push(childView);
    			return childView;
    		},

    		remove: function ()
    		{
    			_.each(this.views, function (view)
    			{
    				view.remove && view.remove();
    			});
    			this.views = null;

    			_.each(this.binders, function (binder)
    			{
    				binder.unbind();
    			});

    			this.stopListening();

    			Backbone.View.prototype.remove.apply(this, arguments);
    		},

    		result: function (functionOrValue)
    		{
    			if (_.isFunction(functionOrValue))
    			{
    				return functionOrValue.call(this);
    			} else
    			{
    				return functionOrValue;
    			}

    		},

    		showSidebar: function ()
    		{
    			this.pubSub.trigger('sidebar:show');
    		},

    		initFieldFromOption: function (name, options)
    		{
    			var opts = options || this.options;
    			(!_.isUndefined(opts[name])) && (this[name] = opts[name]);
    		},

    		blockEvent: function (evt)
    		{
    			evt.preventDefault();
    			evt.stopPropagation();
    		},


    		onBack: function (evt)
    		{
    			this.router.navigateBack();
    			evt.preventDefault();
    		},

    		renderPerson: function (oneOrMany, emptyCaption)
    		{
    			var that = this;
    			var html = '';
    			emptyCaption = emptyCaption || 'Не задано';
    			if (_.isArray(oneOrMany))
    			{
    				_.each(oneOrMany, function (person)
    				{
    					html += that.renderPerson(person);
    				});
    			} else
    			{
    				var person = oneOrMany;
    				html += '<img src="' + that.photoBank.getPhotoUrl(person ? person.id : undefined) + '" class="tb-smallInlinePhoto"/> ';
    				html += (person ? that.formatter.caption(person) : emptyCaption);
    				return html;
    			}
    		},

    		preventRepeatingClicks: function (evt)
    		{
    			return;
    			evt && $(evt.currentTarget).css('pointer-events', 'none');
    		}

    	});



    });