﻿/**
 * @class ActivitiesView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'views/common/EmbeddableActivitiesView',
    'settings',
    'srv',
    'text!views/activities/ActivitiesView.html',
    'text!views/activities/ActivitiesView_data.html',
    'collections/activities/ActivityCollection',
    'utils/entityUtil',
    'views/common/BaseLowerToolbar',
    'views/filterSupport',
    'views/activities/ActivitiesFiltersPopupView',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'views/activities/GenericActivityCardView'
], function (
    _,
    $,
    BaseToolView,
    EmbeddableActivitiesView,
    settings,
    srv,
    contentTemplate,
    dataTemplate,
    ActivityCollection,
    entityUtil,
    BaseLowerToolbar,
    filterSupport,
    ActivitiesFiltersPopupView,
    allUsersCache,
    currentUserCache,
    GenericActivityCardView
    ) {
    return BaseToolView.extend({

        contentTemplate: contentTemplate,

        cardEntityType: 'activity',

        collection: null,

        headerText: 'Лента',

        filterPopupClass: ActivitiesFiltersPopupView,

        embeddableActivitiesView: null,

        refData: _.extend({
            allUsers: allUsersCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        }, BaseToolView.prototype.refData),

        initialize: function (options) {
            this.include(filterSupport);
            BaseToolView.prototype.initialize.apply(this, arguments);
            this.restoreFilters();
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        },
                        {
                            builtin: 'filter'
                        }
                    ]
                });
            }
        },

        fetchUrl: function () {
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'activity');
        },

        fetchParams: function() {
            var params = {};
            if (this.filters) {
                this.filters.entityType && (params.entityType = this.filters.entityType);
                this.filters.initiator && (params.initiator = this.filters.initiator);
                this.filters.operationType && (params.operationType = this.filters.operationType);
            }
            return params;
        },

        updateData: function (updateOptions) {
            var that = this;
            this.collection = new ActivityCollection();
            this._mainDataLoaded = false;
            srv.fetch(_.defaults({
                collection: this.collection,
                url: this.fetchUrl(),
                urlParams: this.fetchParams(),
                success: function () {
                    that.onDataUpdated();
                }
            }, updateOptions));

        },

        renderData: function () {
            this.renderLocationContextCaption();
            //this.$content().html(dataTemplate.call(this));
            this.renderEmbeddableActivitiesView();
        },

        renderEmbeddableActivitiesView: function () {
            if (this.embeddableActivitiesView) {
                this.embeddableActivitiesView.remove();
            }
            this.embeddableActivitiesView = this.createChildView(EmbeddableActivitiesView, {
                collection: this.collection,
                forEntity: false
            });
            this.embeddableActivitiesView.render();
            this.embeddableActivitiesView.$el.appendTo(this.$('.js-embeddedActivities'));
        }

    });
});