﻿define([
    'underscore',
    'jquery',
    'backbone',
    'backboneModelBinder',
    'utils/formatter',
    'moment'
], function (_, $, Backbone, ModelBinder, formatter, moment) {

    var standardConverters = {

        timeOn: function (direction, value, attributeName) {
            if (direction === 'ViewToModel') {
                if (formatter.isServiceDate(value)) {
                    return { date: value };
                } else if (formatter.isServiceDateTime(value)) {
                    return { dateTime: value };
                } else {
                    return undefined;
                }

            } else {
                return value ? (value.date ? value.date : value.dateTime) : undefined;
            }
        },

        ref: function (direction, value, attributeName) {
            if (direction === 'ViewToModel') {
                return value ? { id: value } : undefined;
            } else {
                return value ? value.id : undefined;
            }
        },

        refs: function (direction, value, attributeName) {
            if (direction === 'ViewToModel') {
                if (value) {
                    var ids = value.split(',');
                    return _.map(ids, function(id) { return {id: id}; });
                } else {
                    return [];
                }
            } else {
                return _.isEmpty(value) ? undefined : _.map(value, function(ref) { return ref.id; }).join(',');
            }
        },

        boolean: function (direction, value) {
            if (direction === 'ViewToModel') {
                return value === 'true' ? true : (value === 'false' ? false : undefined);
            } else {
                return value === true ? 'true' : ( value === false ? 'false' : undefined);
            }
        }
    };

    return {
        binders: [],

        createBinder: function () {
            var binder = new ModelBinder();
            this.binders.push(binder);
            return binder;
        },

        defaultModelForBinding: function () {
            return this.model;
        },

        bindData: function (selector, modelForBinding) {
            var $root = selector ? this.$(selector) : this.$el;
            var model = modelForBinding ? modelForBinding : this.defaultModelForBinding();
            var binder = this.createBinder();
            var bindings = this.buildBindings($root);
            $root.each(function () {
                binder.bind(model, this, bindings);
            });
        },

        buildBindings: function ($root) {
            var bindings = {};
            $root.find('[bound-to]').each(function () {
                var $boundElement = $(this);
                var boundTo = $boundElement.attr('bound-to');
                var converterName = $boundElement.attr('converter');
                var binding = {
                    selector: '[bound-to="' + boundTo + '"]'
                };
                (converterName) && (binding.converter = standardConverters[converterName]);
                bindings[boundTo] = binding;
            });
            return bindings;
        }
    };

});