﻿/**
 * @class CalendarDayAgendaView
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'views/common/EmbeddableAgendaView',
    'text!views/calendar/CalendarDayAgendaView.html',
    'text!views/calendar/CalendarDayAgendaView_header.html',
    'settings',
    'srv',
    'utils/cache/structureCache',
    'utils/navigationUtil',
    'views/locationContextSupport',
    'views/refDataSupport',
    'moment',
    'views/common/BaseLowerToolbar'
], function (
    _,
    $,
    BasePageView,
    EmbeddableAgendaView,
    contentTemplate,
    headerTemplate,
    settings,
    srv,
    structureCache,
    navigationUtil,
    locationContextSupport,
    refDataSupport,
    moment,
    BaseLowerToolbar
    ) {
    return BasePageView.extend({

    	cardEntityType: 'calendarEvent',

    	dayAgenda: true,

        contentTemplate: contentTemplate,
        headerTemplate: headerTemplate,

        mainData: null,

        embeddableAgendaView: null,

        filters: null,

        events: _.extend({
        	'click .js-back': 'onBackButton',
            'click .js-add': 'onAddClicked'
        }, BasePageView.prototype.events),

        refData: {
            structure: structureCache.refDataLoader
        },

        initialize: function (options) {
            this.include(locationContextSupport);
            this.include(refDataSupport);
            BasePageView.prototype.initialize.apply(this, arguments);
            this.filters = {
                dataType: this.options.dataType,
                assigneeOrMember: this.options.assigneeOrMember
            };
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        }
                    ]
                });
            }
        },

        fetchParams: function () {
            var params = {};
            var dateMoment = this.options.date ? moment(this.options.date) : moment();
            var d = dateMoment.format(this.formatter.serviceDateFormat);
            //var dateTimeFromParam = moment(dateMoment).startOf('day').utc().format(this.formatter.serviceDateTimeFormat);
            //var dateTimeToParam = moment(dateMoment).endOf('day').utc().format(this.formatter.serviceDateTimeFormat);

            _.extend(params, {
            	dateFrom: d,
            	dateTo: d,
            });

            if (this.filters) {
                this.filters.dataType && (params.dataType = this.filters.dataType);
                this.filters.assigneeOrMember && (params.assigneeOrMember = this.filters.assigneeOrMember);
            }

            return params;
        },

        fetchUrl: function () {
            // TODO use locationContextSupport methods
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'calendar');
        },

        updateData: function(updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            srv.fetch(_.defaults({
                url: this.fetchUrl(),
                urlParams: this.fetchParams(),
                success: function(result) {
                    that.mainData = result;
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        renderData: function() {
            this.renderEmbeddableAgendaView();
            this.$header().find('.js-locationContextCaption').text(this.locationContextCaption());
        },

        renderEmbeddableAgendaView: function() {
            if (this.embeddableAgendaView) {
                this.embeddableAgendaView.remove();
            }
            this.embeddableAgendaView = this.createChildView(EmbeddableAgendaView, {
                tasks: this.mainData.tasks,
                calendarEvents: this.mainData.calendarEvents,
                supportsEmptyCaption: true,
                emptyCaption: 'На данный день нет никаких дел',
                showTaskDetails: false,
                showEventDetails: true,
                showLocationContext: true
            });
            this.embeddableAgendaView.render();
            this.embeddableAgendaView.$el.appendTo(this.$('.js-embeddedAgenda'));
        },


        onBackButton: function ()
        {
            this.router.navigateBack();
            //evt.preventDefault();
        },

        onAddClicked: function(evt) {
            //evt.preventDefault();
            var locationContext = this.locationContext();
            navigationUtil.showEntityCreationMenu(this.router, ['task', 'calendarEvent'], locationContext, { date: this.options.date });
        }




    });
});