﻿/**
 * @class CalendarEventCardView
 */
define(['underscore', 'jquery', 'views/BaseEntityCardView', 'models/calendar/CalendarEventModel', 'utils/uiManager', 'utils/cache/structureCache', 'utils/cache/currentUserCache', 'text!views/calendar/CalendarEventCardView_data.html', 'views/common/UserListViewPage', 'srv', 'views/common/WorkspaceListViewPage', 'utils/uiExtensions'], function (_, $, BaseEntityCardView, CalendarEventModel, uiManager, structureCache, currentUserCache, dataTemplate, UserListViewPage, srv, WorkspaceListViewPage, uiExtensions) {

    var CalendarEventCardView = BaseEntityCardView.extend({

        entityType: 'calendarEvent',

        dataTemplate: dataTemplate,

        focusedFieldSelector: '.js-calendarEventTitle',

        emptyCard: {
            caption: 'Выберите событие или задачу из списка, чтобы посмотреть более подробную информацию',
            icon: 'calendar'
        },

        refData: {
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        events: _.extend({
            'click .js-memberRefsPickerStub': 'onMemberRefsPickerStub'
        }, BaseEntityCardView.prototype.events),

        initialize: function (options) {
            BaseEntityCardView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new CalendarEventModel();
        },

        _createControls: function ()
        {
        	var that = this;

        	this.ui.form.editInPlace(this.$('[bound-to="title"]'), {
        		viewerClasses: 'tb-cardView-title js-calendarEventTitle',
        		emptyCaption: 'Название события...'
        	});


        	this.ui.form.editInPlace(this.$('[bound-to="description"]'), {
        		viewerClasses: 'tb-cardView-description',
        		emptyCaption: 'Описание события...'
        	});

        	this.ui.form.editInPlace(this.$('[bound-to="location"]'), {
        		viewerClasses: 'tb-cardView-description',
        		emptyCaption: 'Место события...'
        	});


        	uiExtensions.form.dateTimePicker(this.$('[bound-to="startOn"]'), {
        		emptyCaption: 'Начало',
        		icon: 'calendar',
        		supportsEmptyValue: false
        	});
        	uiExtensions.form.dateTimePicker(this.$('[bound-to="dueOn"]'), {
        		emptyCaption: 'Окончание',
        		icon: 'calendar',
        		supportsEmptyValue: false
        	});

        	var structureItems = WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
        		myRootCaption: 'Мои задачи'
        	});

        	this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
        		listViewPageClass: WorkspaceListViewPage,
        		items: structureItems,
        		icon: 'suitcase',
        		emptyCaption: 'Проект не выбран',
        		onSelected: function (value, item)
        		{
        			if (item.myRootWorkspace)
        			{
        				var user = that.refData.currentUser;
        				that.ui.form.destroy(that.$('[bound-to="memberRefs"]'));
        				that.ui.form.staticPicker(that.$('[bound-to="memberRefs"]'), {
        					items: [{
        						id: user.id,
        						caption: user.name
        					}],
        					icon: 'users'
        				});
        			}
        			else
        			{
        				that._updateUserList({
        					area: item.areaId,
        					workspace: item.id
        				});
        			}
        		}
        	});

        	var locationContext = this.entityLocationContext();
        	if (locationContext && locationContext.workspaceId)
        	{
        		var workspaceItem = _.find(structureItems, function (item)
        		{
        			return item.type === 'workspace' && item.id === locationContext.workspaceId;
        		});
        		if (workspaceItem && workspaceItem.myRootWorkspace)
        		{
        			var user = that.refData.currentUser;
        			that.ui.form.destroy(that.$('[bound-to="memberRefs"]'));
        			that.ui.form.staticPicker(that.$('[bound-to="memberRefs"]'), {
        				items: [{
        					id: user.id,
        					caption: user.name
        				}],
        				icon: 'users'
        			});
        		}
        		else
        		{
        			this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);
        			this._updateUserList(locationContext);
        		}
        	}
        	this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
        },

        _updateUserList: function(selectedLocationContext) {
            var that = this;
            if (_.isEmpty(selectedLocationContext)) {
                selectedLocationContext = this.locationContext();
            }
            var url = srv.makeUrlWithLocationContext(selectedLocationContext, 'user') + '?include=roles;allUsers';
            srv.fetch({
                url: url,
                success: function(users) {
                    that._renderUserList(users);
                }
            });
        },

        _renderUserList: function(users) {
            var $hidden = this.$('[bound-to="memberRefs"]');
            this.$('.js-memberRefsPickerStub').remove();
            this.ui.form.destroy($hidden);
            this.ui.form.listViewPicker($hidden, {
                listViewPageClass: UserListViewPage,
                items: UserListViewPage.prototype.usersToItems(users, 'Нет участников'),
                icon: 'users',
                usePhotos: true,
                emptyCaption: 'Участники',
                headerText: 'Участники',
                multipleSelection: true
            });
            $hidden.trigger('ui:refresh');
        },

        onMemberRefsPickerStub: function(evt) {
            //evt.preventDefault();
            this.ui.popup.message({
                text: 'Укажите проект, чтобы выбрать участников',
                classes: 'ui-popup-message ui-popup-cardArea'
            });
        },

        onSaved: function () {
            var that = this;
            srv.clearCache('calendarEvent');
            srv.clearCache('calendar');
            srv.clearCache('planning');
            //this.router.tryForceUpdateCurrentView();
        }

    });

    uiManager.registerCardViewClass(CalendarEventCardView.prototype.entityType, CalendarEventCardView);

    return CalendarEventCardView;
});
