﻿/**
 * @class EditCalendarEventView
 */
define([
    'underscore',
    'jquery',
    'views/BaseEditorView',
    'text!views/calendar/EditCalendarEventView_data.html',
    'text!views/calendar/EditCalendarEventView_header.html',
    'settings',
    'models/calendar/CalendarEventModel',
    'views/locationContextSupport',
    'utils/locationContextUtil',
    'utils/cache/structureCache',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'srv',
    'views/common/WorkspaceListViewPage',
    'views/common/UserListViewPage',
    'utils/uiExtensions'
], function (_, $, BaseEditorView, dataTemplate, headerTemplate, settings, CalendarEventModel, locationContextSupport, locationContextUtil, structureCache, allUsersCache, currentUserCache, srv, WorkspaceListViewPage, UserListViewPage, uiExtensions) {
    return BaseEditorView.extend({

        headerTemplate: headerTemplate,

        refData: {
            //users: allUsersCache.refDataLoader,
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader

        },

        events: {
            'click .js-save': 'onSaveClicked',
            'click .js-back': 'onBack',
            'click .js-memberRefsPickerStub': 'onMemberRefsPickerStub'
        },

        saveTriggerSelector: '.js-save',

        initialize: function (options) {
            this.include(locationContextSupport);
            BaseEditorView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new CalendarEventModel();
        },

        onSaveClicked: function (evt) {
            this.save();
            evt.preventDefault();
        },

        renderData: function () {
            var that = this;
            this.$content().html(dataTemplate.call(this));
            this.bindData();
            uiExtensions.form.dateTimePicker(this.$('[bound-to="startOn"]'), {
                emptyCaption: 'Начало',
                icon: 'calendar',
                supportsEmptyValue: false
            });
            uiExtensions.form.dateTimePicker(this.$('[bound-to="dueOn"]'), {
                emptyCaption: 'Окончание',
                icon: 'calendar',
                supportsEmptyValue: false
            });

            var structureItems = WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
                myRootCaption: 'Мои события'
            });

            this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
                listViewPageClass: WorkspaceListViewPage,
                items: structureItems,
                icon: 'suitcase',
                emptyCaption: 'Проект не выбран',
                onSelected: function(value, item) {
                    if (item.myRootWorkspace) {
                        var user = that.refData.currentUser;
                        that.ui.form.destroy(that.$('[bound-to="memberRefs"]'));
                        that.ui.form.staticPicker(that.$('[bound-to="memberRefs"]'), {
                            items: [{
                                id: user.id,
                                caption: user.name
                            }],
                            icon: 'user'
                        });
                    } else {
                        that._updateUserList({
                            area: item.areaId,
                            workspace: item.id
                        });
                    }
                }
            });

            var locationContext = this.locationContext();
            if (locationContext && locationContext.workspaceId) {
                var workspaceItem = _.find(structureItems, function(item) {
                    return item.type === 'workspace' && item.id === locationContext.workspaceId;
                });
                if (workspaceItem && workspaceItem.myRootWorkspace) {
                    var user = that.refData.currentUser;
                    that.ui.form.destroy(that.$('[bound-to="memberRefs"]'));
                    that.ui.form.staticPicker(that.$('[bound-to="memberRefs"]'), {
                        items: [{
                            id: user.id,
                            caption: user.name
                        }],
                        icon: 'user'
                    });
                } else {
                    this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);
                    this._updateUserList(locationContext);
                }

            }

            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
            this.ui.form.autoSizeAllInContainer(this.$el);
        },

        _updateUserList: function(selectedLocationContext) {
            var that = this;
            if (_.isEmpty(selectedLocationContext)) {
                selectedLocationContext = this.locationContext();
            }
            var url = srv.makeUrlWithLocationContext(selectedLocationContext, 'user') + '?include=roles;allUsers';
            srv.fetch({
                url: url,
                success: function(users) {
                    that._renderUserList(users);
                }
            });
        },

        _renderUserList: function(users) {
            var $hidden = this.$('[bound-to="memberRefs"]');
            this.$('.js-memberRefsPickerStub').remove();
            this.ui.form.destroy($hidden);
            this.ui.form.listViewPicker($hidden, {
                listViewPageClass: UserListViewPage,
                items: UserListViewPage.prototype.usersToItems(users),
                icon: 'user',
                usePhotos: true,
                emptyCaption: 'Участники',
                headerText: 'Участники',
                multipleSelection: true
            });
            $hidden/*.val('')*/.trigger('ui:refresh');
        },

        onMemberRefsPickerStub: function(evt) {
            evt.preventDefault();
            this.ui.popup.message({
                text: 'Укажите проект, чтобы выбрать участников'
            });
        },

        /**
         * @deprecated
         * @returns {Array}
         * @private
         */
        _allWorkspaces: function () {
            var allWorkspaces = [];
            _.each(this.refData.structure.areas, function (area) {
                _.each(area.workspaces, function (workspace) {
                    allWorkspaces.push({
                        id: workspace.id,
                        caption: locationContextUtil.locationContextCaptionFromTwoCaptions(area.title, workspace.title)
                    });
                });

            });
            return allWorkspaces;
        },

        beforeSave: function () {
            var locationContext = this.locationContext();
            if (!_.isEmpty(locationContext)) {
                this.model.set('workspaceRef', { id: locationContext.workspaceId });
            }
        },

        afterRender: function () {

        },

        onSaved: function () {
            srv.clearCache('calendarEvent');
            srv.clearCache('calendar');

            /*var locationContext = this.locationContext();
            var url;
            if (!_.isEmpty(locationContext)) {
                url = '#!/calendar/area/' + locationContext.areaId + '/workspace/' + locationContext.workspaceId;
            } else {
                url = '#!/calendar';
            }
            // TODO use urlBuilder
            this.router.navigate(url, {trigger: true, replace: false});*/
            this.router.navigate('#!/calendarEvents/view/' +  this.model.id, {trigger: true, replace: true});
        }

    });
});

