﻿/**
 * @class TodayAgendaView
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'views/common/EmbeddableAgendaView',
    'text!views/calendar/TodayAgendaView.html',
    'text!views/calendar/TodayAgendaView_header.html',
    'utils/cache/structureCache',
    'utils/navigationUtil',
	'utils/uiManager',
    'views/refDataSupport',
    'settings',
    'srv',
	'moment',
    'views/common/BaseLowerToolbar',
	'views/calendar/TodayAgendaCardView'
], function (
    _,
    $,
    BasePageView,
    EmbeddableAgendaView,
    contentTemplate,
    headerTemplate,
    structureCache,
    navigationUtil,
	uiManager,
    refDataSupport,
    settings,
    srv,
	moment,
    BaseLowerToolbar,
	TodayAgendaCardView
    ) {
    return BasePageView.extend({

    	cardEntityType: 'todayAgenda',

		dayAgenda:true,

        contentTemplate: contentTemplate,
        headerTemplate: headerTemplate,

        mainData: null,

        embeddableAgendaView: null,

        events: _.extend({
            'click .js-toggleSidebar': 'onToggleSidebarClicked',
            'click .js-add': 'onAddClicked'
        }, BasePageView.prototype.events),

        refData: {
            structure: structureCache.refDataLoader
        },

        initialize: function (options) {
            this.include(refDataSupport);
            BasePageView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();
            srv.clearCache('planning/today');

            if (uiManager.isTabletMode())
            	uiManager.openCard('todayAgenda', { router: this.router });
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        }
                    ]
                });
            }
        },

        updateData: function(updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            srv.fetch(_.defaults({
                url: settings.restUrlBase + 'planning/today',
                success: function(result) {
                    that.mainData = result;
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        renderData: function() {
            this.renderEmbeddableAgendaView();
        },

        renderEmbeddableAgendaView: function() {
            if (this.embeddableAgendaView) {
                this.embeddableAgendaView.remove();
            }
            this.embeddableAgendaView = this.createChildView(EmbeddableAgendaView, {
                tasks: this.mainData.tasks,
                calendarEvents: this.mainData.calendarEvents,
                supportsEmptyCaption: true,
                emptyCaption: 'На сегодня нет дел',
                showTaskDetails: false,
                showEventDetails: true,
                showLocationContext: true
            });
            this.embeddableAgendaView.render();
            this.embeddableAgendaView.$el.appendTo(this.$('.js-embeddedAgenda'));
        },

        onToggleSidebarClicked: function(evt) {
            evt.preventDefault();
            this.showSidebar();
        },

        onAddClicked: function(evt) {
            evt.preventDefault();
            navigationUtil.showEntityCreationMenu(this.router, ['task', 'calendarEvent'], null, { date: moment().format('YYYY-MM-DD') });
        }

    });
});