﻿/**
 * @class BaseFilterPopupView
 */
define([
    'underscore',
    'jquery',
    'views/BaseView',
    'views/refDataSupport',
    'views/bindingSupport',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'utils/customPopupRegistry',
    'storage'
], function (
    _,
    $,
    BaseView,
    refDataSupport,
    bindingSupport,
    allUsersCache,
    currentUserCache,
    customPopupRegistry,
    storage) {

    return BaseView.extend({

        // must override in subclass
        filterStorageKey: null,

        // must override in subclass
        filterModelClass: null,

        _renderFilters: function () {
            // overide in subclass, call ui methods to initialize filter controls
        },

        _mainDataLoaded: true,

        applyFiltersCallback: null,
        model: null,

        events: {
            'click .js-filtersObstacle': 'blockEvent',
            'click .js-cancel': 'onCancel',
            'click .js-apply': 'onApply'
        },

        refData: {
            allUsers: allUsersCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        initialize: function (options) {
            var that = this;
            this.include(refDataSupport);
            this.include(bindingSupport);
            BaseView.prototype.initialize.apply(this, arguments);
            this.initFieldFromOption('applyFiltersCallback');
            this.model = this.createModel();

            this.listenTo(this.pubSub, 'router:navigate', function () {
                that.remove();
            });

            customPopupRegistry.register(this);

        },

        createModel: function() {
            return new this.filterModelClass();
        },

        render: function () {
            this.$el.addClass('tb-lowerPopupView tb-lowerFilterPopupView ui-customPopup');
            BaseView.prototype.render.apply(this, arguments);
            this._renderFilters();
            //this.$('input[type="hidden"]').trigger('ui:refresh');
            this._renderObstacle();
            this.bindData();

            // init model from storage
            var storedFilterValue = storage.get(this.getFilterKey());
            storedFilterValue && this.model.set(JSON.parse(storedFilterValue));
            this.$('input[type="hidden"]').trigger('ui:refresh');

            this.$el.appendTo(document.body);
        },

        _renderObstacle: function () {
            var that = this;
            $('<div id="tb-filtersObstacle" class="js-filtersObstacle" style="background: transparent; z-index: 620; position: fixed; top: 0; bottom: 0; left: 0; right: 0">').appendTo(document.body).click(function (evt) {
                that.blockEvent(evt);
            });
        },

        _removeObstacle: function () {
            $('#tb-filtersObstacle').remove();
        },

        remove: function () {
            customPopupRegistry.unregister(this);
            BaseView.prototype.remove.apply(this, arguments);
            this._removeObstacle();
        },

        getFilterKey: function (options)
        {
        	var result = this.filterStorageKey;

        	var currentOpts = options || (this.options && this.options.parentView && this.options.parentView.options);
        	if (currentOpts && currentOpts.areaId && currentOpts.workspaceId)
        		result = 'area/' + currentOpts.areaId + '/workspace/' + currentOpts.workspaceId + result;

        	return result;
        },

        onCancel: function (evt) {
            evt.preventDefault();
            this.remove();
        },

        applyFilters: function () {
            if (this.applyFiltersCallback) {
                var filterObject = this.model.toJSON();
                storage.set(this.getFilterKey(), filterObject);
                this.applyFiltersCallback(filterObject);
            }
        },

        onApply: function (evt) {
            this.applyFilters();
            evt.preventDefault();
            this.remove();
        },

        /**
         * This method is to be called from customPopupRegistry
         */
        close: function () {
            this.remove();
        }

    });

});