﻿/**
 * @class BaseLowerToolbar
 */
define(['underscore', 'jquery', 'views/BaseView', 'pubSub'], function(_, $, BaseView, pubSub) {

    var builtinItems = {
        'refresh': {
            name: 'refresh',
            icon: 'refresh',
            refreshing: false,
            onClicked: function($td, item) {
                if (item.refreshing) {
                    return;
                }
                if (this.parentView && this.parentView.onRefreshRequested) {
                    this.parentView.onRefreshRequested();
                }

            },

            decorateItem: function($td, item) {
                this.listenTo(pubSub, 'update:success', function() {
                    // success callback
                    item.refreshing = false;
                    //$td.find('i').removeClass('fa-spin');
                });
                this.listenTo(pubSub, 'update:error', function() {
                    // error callback
                    item.refreshing = false;
                    //$td.find('i').removeClass('fa-spin');
                });
                this.listenTo(pubSub, 'update:start', function() {
                    // success callback
                    item.refreshing = true;
                    //$td.find('i').addClass('fa-spin');
                });
                this.listenTo(pubSub, 'loader:start', function() {
                    $td.find('i').addClass('fa-spin');
                });
                this.listenTo(pubSub, 'loader:stop', function() {
                    $td.find('i').removeClass('fa-spin');
                });
            }
        },
        'delete': {
            name: 'delete',
            icon: 'trash-o',
            onClicked: function($td, item) {
                if (this.parentView && this.parentView.onDelete) {
                    this.parentView.onDelete();
                }

            }
        },
        'clearNotification': {
        	name: 'clearNotification',
        	icon: 'trash-o',
			caption: 'Очистить',
        	onClicked: function ($td, item)
        	{
        		pubSub.trigger('Push:ClearNotifications');

        	}
        },
        'filter': {
            name: 'filter',
            icon: 'filter',
            onClicked: function($td, item) {
                var filterPopupClass = this.parentView.filterPopupClass;
                if (filterPopupClass && this.parentView.applyFilters) {
                    var view = new filterPopupClass({
                        applyFiltersCallback: _.bind(this.parentView.applyFilters, this.parentView),
                        parentView: this.parentView
                    });
                    view.render();
                }
            },
            decorateItem: function($td, item) {
                if (_.any(this.parentView.filters)) {
                    $td.addClass('tb-filtersActivator-active');
                } else {
                    $td.removeClass('tb-filtersActivator-active');
                }
            }

        }
    };

    return BaseView.extend({

    	parentView: null,
    	items: null,

    	events: {
    		'click .js-lowerToolbarButton': 'onButtonClicked'
    	},

    	initialize: function (options)
    	{
    		BaseView.prototype.initialize.apply(this, arguments);
    		this.initFieldFromOption('parentView');
    		this._prepareItems();
    	},

    	_prepareItems: function ()
    	{
    		//this.items = $.extend(true, {}, this.items);
    		this.items = _.map(this.items, function (item)
    		{
    			if (item.builtin)
    			{
    				return $.extend(true, $.extend(true, {}, item), builtinItems[item.builtin]);
    			} else
    			{
    				return $.extend(true, {}, item);
    			}
    		});
    	},

    	render: function ()
    	{
    		var that = this;
    		this.$el.addClass('tb-lowerToolbar js-lowerToolbar');
    		var $table = $('<table style="width:100%; table-layout:fixed;"></table>');
    		var $tr = $('<tr></tr>').appendTo($table);
    		_.each(this.items, function (item)
    		{
    			var $td = $('<td></td>').addClass('js-lowerToolbarButton');
    			item.name && ($td.attr('item-name', item.name));
    			item.classes && ($td.addClass(item.classes));
    			if (item.caption)
    			{
    				$('<span></span>').addClass('tb-lowerToolbar-itemCaption').text(item.caption).appendTo($td);
    			}
    			if (item.icon)
    			{
    				$('<i class="fa"></i>').addClass('fa-' + item.icon + (item.icon == 'refresh' && window._ajaxInProgress === true ? ' fa-spin' : '')).appendTo($td);
    			}
    			//if (item.width)
    			//{
    			//	$td.width(item.width);
    			//}
    			item.decorateItem && item.decorateItem.call(that, $td, item);
    			$td.attr('data-item-name', item.name).appendTo($tr);
    		});
    		$table.appendTo(this.$el);
    	},

    	getItemElement$: function (itemName)
    	{
    		return this.$('[item-name="' + itemName + '"]');
    	},

    	onButtonClicked: function (evt)
    	{
    		//evt.preventDefault();
    		var $td = $(evt.currentTarget);
    		var data = $td.data();
    		var itemName = data.itemName;
    		var allow = data.allow;
    		if (allow === false)
    			return;

    		var item = _.find(this.items, function (item) { return item.name === itemName; });
    		item.onClicked.call(this, $td, item);
    	}
    });

});