﻿/**
 * @class BasePlanItemView
 */
define([
    'underscore',
    'jquery',
    'backbone',
    'collections/BaseCollection',
    'views/BaseViewerView',
    'text!views/common/BasePlanItemView.html',
    'settings',
    'srv',
    'collections/activities/ActivityCollection',
    'views/common/EmbeddableActivitiesView',
    'views/common/EmbeddableAttachmentListView',
    'views/common/CommentComposerPageView',
    'views/common/EmbeddableCommentActivatorView',
    'views/common/BaseLowerToolbar',
    'views/common/CommentComposerPopupView',
    'views/deleteSupport',
    'utils/attachmentUtil',
    'utils/fileManager'
], function (
    _,
    $,
    Backbone,
    BaseCollection,
    BaseViewerView,
    contentTemplate,
    settings,
    srv,
    ActivityCollection,
    EmbeddableActivitiesView,
    EmbeddableAttachmentListView,
    CommentComposerPageView,
    EmbeddableCommentActivatorView,
    BaseLowerToolbar,
    CommentComposerPopupView,
    deleteSupport,
    attachmentUtil,
    fileManager
    ) {

    return BaseViewerView.extend({

        contentTemplate: contentTemplate,

        dataTemplate: null,

        headerText: '',

        _modelLoaded: null,
        attachmentCollection: null,
        _attachmentCollectionLoaded: null,
        activityCollection: null,
        _activityCollectionLoaded: null,

        embeddableActivitiesView: null,
        embeddableAttachmentListView: null,

        cachePatternsToClearOnDelete: ['calendar'],

        /**
         * @deprecated
         */
        embeddableCommentActivatorView: null,

        initialize: function (options) {
            this.include(deleteSupport);
            BaseViewerView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh',
                            width: '25%'
                        },
                        {
                            builtin: 'delete',
                            width: '25%'
                        },
                        {
                            name: 'attach',
                            icon: 'paperclip',
                            onClicked: function($td, item) {
                                attachmentUtil.showAttachUI({
                                    onAttachmentCreated: function(imageUrl) {
                                        that.pubSub.trigger('update:start');
                                        fileManager.uploadAttachment(imageUrl, {
                                            target: {
                                                entityType: that.model.entityType,
                                                entityId: that.model.id
                                            },
                                            fileName: 'image-' + that.formatter.nowAsFileNameString() + '.jpg',
                                            success: function() {
                                                that.update({forceUpdate: true});
                                                //that.updateAttachments();
                                            }
                                        });
                                    }
                                });

                            }
                        },
                        {
                            name: 'comment',
                            icon: 'comment-o',
                            onClicked: function($td, item) {
                                that.ui.scrollToBottom(true, function() {
                                    var popupView = new CommentComposerPopupView({
                                        commentTarget: {
                                            entityType: that.model.entityType,
                                            entityId: that.model.id
                                        },
                                        onPosted: function() {
                                            that.updateActivities();
                                        }
                                    });
                                    popupView.render();
                                });
                            }
                        }

                    ]
                });
            };
        },


        updateData: function (updateOptions) {
            var that = this;
            this.model = this.createModel();
            this.model.id = this.options.entityId;
            this._modelLoaded = false;

            this.attachmentCollection = new BaseCollection(); // TODO use dedicated collection type?
            this._attachmentCollectionLoaded = false;

            this.activityCollection = new ActivityCollection();
            this._activityCollectionLoaded = false;

            var customOptions = {};
            _.isFunction(that.fetchUrl) && (customOptions.url = that.fetchUrl());

            srv.fetch(_.defaults({
                model: this.model,
                success: function () {
                    that._modelLoaded = true;
                    that.onDataUpdated();
                }
            }, customOptions, updateOptions));

            srv.fetch(_.defaults({
                collection: this.attachmentCollection,
                url: that.model.url() + '/attachment',
                success: function() {
                    that._attachmentCollectionLoaded = true;
                    that.onDataUpdated();
                }
            }, updateOptions));

            srv.fetch(_.defaults({
                collection: this.activityCollection,
                url: that.model.url() + '/activity',
                success: function() {
                    that._activityCollectionLoaded = true;
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        onDataUpdated: function() {
            if (this._modelLoaded && this._attachmentCollectionLoaded && this._activityCollectionLoaded) {
                this.renderDataWrapper();
            }
        },

        renderData: function() {
            this.$content().find('.js-data').html(this.dataTemplate.call(this));
            this.renderEmbeddableAttachmentListView();
            this.renderEmbeddableActivitiesView();

            //this.renderEmbeddableCommentActivatorView();
            //this.$('.js-hiddenBeforeDataRendering').show();
            //activityCollection
        },

        renderEmbeddableAttachmentListView: function() {
            if (this.embeddableAttachmentListView) {
                this.embeddableAttachmentListView.remove();
            }
            this.embeddableAttachmentListView = this.createChildView(EmbeddableAttachmentListView, {
                collection: this.attachmentCollection
            });
            this.embeddableAttachmentListView.render();
            this.embeddableAttachmentListView.$el.appendTo(this.$content().find('.js-attachments'));
        },

        renderEmbeddableActivitiesView: function() {
            if (this.embeddableActivitiesView) {
                this.embeddableActivitiesView.remove();
            }
            this.embeddableActivitiesView = this.createChildView(EmbeddableActivitiesView, {
                collection: this.activityCollection,
                forEntity: true
            });
            this.embeddableActivitiesView.render();
            var $activitiesContainer = this.$content().find('.js-activities');
            $activitiesContainer.empty();
            this.embeddableActivitiesView.$el.appendTo($activitiesContainer);
        },

        updateActivities: function() {
            var that = this;
            this.activityCollection = new ActivityCollection();
            this._activityCollectionLoaded = false;
            srv.fetch({
                collection: this.activityCollection,
                url: that.model.url() + '/activity',
                success: function() {
                    that._activityCollectionLoaded = true;
                    that.renderEmbeddableActivitiesView();
                }
            });
        },

        /*updateAttachments: function() {
            var that = this;
            this.attachmentCollection = new BaseCollection(); // TODO use dedicated collection type?
            this._attachmentCollectionLoaded = false;
            srv.fetch(_.defaults({
                collection: this.attachmentCollection,
                url: that.model.url() + '/attachment',
                success: function() {
                    that._attachmentCollectionLoaded = true;
                    that.renderEmbeddableAttachmentListView();
                }
            }));
        },*/

        /**
         * @deprecated
          */
        renderEmbeddableCommentActivatorView: function() {

            var that = this;
            if (this.embeddableCommentActivatorView) {
                //this.stopListening(this.embeddableCommentActivatorView);
                this.embeddableCommentActivatorView.remove();
            }
            this.embeddableCommentActivatorView = this.createChildView(EmbeddableCommentActivatorView, {
                /*commentTarget: {
                    entityType: that.model.entityType,
                    entityId: that.model.id
                }*/
                onClicked: function() {
                    var pageView = new CommentComposerPageView({
                        commentTarget: {
                            entityType: that.model.entityType,
                            entityId: that.model.id
                        },
                        onPosted: function() {
                            that.updateActivities();
                        }
                    });
                    that.ui.popup.popupPage(pageView);
                }
            });
            this.embeddableCommentActivatorView.render();
            this.embeddableCommentActivatorView.$el.appendTo(this.$content().find('.js-commentActivator'));

        }




    });
});

