﻿/**
 * @class BaseSortPopupView
 */
define([
    'underscore',
    'jquery',
    'views/BaseView',
    //'views/refDataSupport',
    //'views/bindingSupport',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'utils/customPopupRegistry',
    'storage'
], function (
    _,
    $,
    BaseView,
    //refDataSupport,
    //bindingSupport,
    allUsersCache,
    currentUserCache,
    customPopupRegistry,
    storage) {

    return BaseView.extend({

        // must override in subclass
        sortStorageKey: null,

        // must override in subclass
        sortModelClass: null,

        _renderSortParameters: function () {
            // overide in subclass, call ui methods to initialize sort controls
        },

        _mainDataLoaded: true,

        applySortCallback: null,
        model: null,

        events: {
            'click .js-sortObstacle': 'blockEvent',
            'click .js-cancel': 'onCancel',
            'click .js-apply': 'onApply'
        },

        refData: {
            allUsers: allUsersCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        initialize: function (options) {
            var that = this;
            //this.include(refDataSupport);
            //this.include(bindingSupport);
            BaseView.prototype.initialize.apply(this, arguments);
            this.initFieldFromOption('applySortCallback');
            this.model = this.createModel();

            this.listenTo(this.pubSub, 'router:navigate', function () {
                that.remove();
            });

            customPopupRegistry.register(this);

        },

        createModel: function() {
            return new this.sortModelClass();
        },

        render: function () {
            this.$el.addClass('tb-lowerPopupView tb-lowerSortPopupView ui-customPopup');
            BaseView.prototype.render.apply(this, arguments);

            // init model from storage
            var storedSortValue = storage.get(this.sortStorageKey);
            storedSortValue && this.model.set(JSON.parse(storedSortValue));

            this._renderSortParameters();
            this._renderObstacle();

            this.$el.appendTo(document.body);
        },

        _renderObstacle: function () {
            var that = this;
            $('<div id="tb-sortObstacle" class="js-sortObstacle" style="background: transparent; z-index: 620; position: fixed; top: 0; bottom: 0; left: 0; right: 0">').appendTo(document.body).click(function (evt) {
                that.blockEvent(evt);
            });
        },

        _removeObstacle: function () {
            $('#tb-sortObstacle').remove();
        },

        remove: function () {
            customPopupRegistry.unregister(this);
            BaseView.prototype.remove.apply(this, arguments);
            this._removeObstacle();
        },



        onCancel: function (evt) {
            evt.preventDefault();
            this.remove();
        },

        applySort: function ()
        {
        	if (this.applySortCallback)
        	{
        		var sortObject = this.model.toJSON();
        		storage.set(this.sortStorageKey, sortObject);
        		this.applySortCallback(sortObject);
        	}
        },

        onApply: function (evt)
        {
        	evt.preventDefault();
        	var that = this;
        	that.remove();

        	//that.ui.helpers.requestAnimationFrame(function ()
        	//{
        	setTimeout(function () { that.applySort(); }, 1);
        		//that.applySort();
        	//});
        },

        /**
         * This method is to be called from customPopupRegistry
         */
        close: function () {
            this.remove();
        }

    });

});