﻿/**
 * @class CommentComposerPopupView
 */
define([
    'underscore',
    'jquery',
    'views/BaseView',
    'utils/customPopupRegistry',
    'storage',
    'text!views/common/CommentComposerPopupView.html',
    'srv',
    'settings',
    'models/common/CommentModel',
    'utils/attachmentUtil',
    'utils/fileManager',
    'environment'
], function (
    _,
    $,
    BaseView,
    customPopupRegistry,
    storage,
    template,
    srv,
    settings,
    CommentModel,
    attachmentUtil,
    fileManager,
    environment) {

    return BaseView.extend({

        template: template,

        callback: null,
        //model: null,

        events: {
            'click .js-composerObstacle': 'blockEvent',
            'click .js-cancel': 'onCancel',
            'click .js-apply': 'onApply',
            'click .js-attach': 'onAttach',
            'click .js-composerArea': 'onTextAreaClicked',
            'click .js-attachment': 'onAttachmentClicked'
            /*'focusin .js-composerArea': 'onComposerAreaFocusIn',
            'focusout .js-composerArea': 'onComposerAreaFocusOut'*/
        },

        //_attachmentCounter: null,

        initialize: function (options) {
            var that = this;
            BaseView.prototype.initialize.apply(this, arguments);
            this.initFieldFromOption('callback');

            //this._attachmentCounter = 0;

            this.listenTo(this.pubSub, 'router:navigate', function () {
                that.remove();
            });

            customPopupRegistry.register(this);

        },

        render: function () {
            var that = this;
            this.$el.addClass('tb-lowerPopupView tb-lowerCommentComposerPopupView ui-customPopup');
            BaseView.prototype.render.apply(this, arguments);
            this._renderObstacle();

            this.$el.appendTo(document.body);
            setTimeout(function() {
                //that.$('.js-composerArea').focus();
                that.ui.form.autoSizeAllInContainer(that.$el);
                that.$('.js-composerArea').click();
                if (environment.isMobileBrowser() && cordova.plugins.Keyboard.isVisible === false)
                	cordova.plugins.Keyboard.show && cordova.plugins.Keyboard.show();
            }, 100);
            //environment.isIOS() && $(document.body).addClass('ui-hideHeader');
        },

        _renderObstacle: function () {
            var that = this;
            $('<div id="tb-composerObstacle" class="js-composerObstacle" style="background: transparent; z-index: 620; position: fixed; top: 0; bottom: 0; left: 0; right: 0">').appendTo(document.body).click(function (evt) {
                //that.blockEvent(evt);
                that.remove();
            });
        },

        _removeObstacle: function () {
            $('#tb-composerObstacle').remove();
        },

        remove: function () {
            customPopupRegistry.unregister(this);
            this._removeObstacle();
            BaseView.prototype.remove.apply(this, arguments);
            setTimeout(function() {
                $(window).trigger('resize');
            }, 400);
            //$(document.body).removeClass('ui-hideHeader');
        },

        onTextAreaClicked: function(evt) {
            $(evt.currentTarget).focus();
        },

        onComposerAreaFocusIn: function(evt) {
            this.$('.js-composerArea').height($(window).height() - this.$('.tb-lowerPopupViewHeader').height() - 30);
            //window.scroll(0, 0);
        },

        onComposerAreaFocusOut: function(evt) {
            this.$('.js-composerArea').css('height', '8em');
        },

        onCancel: function (evt) {
            evt.preventDefault();
            this.remove();
        },

        /*applyFilters: function () {
            if (this.applyFiltersCallback) {
                var filterObject = this.model.toJSON();
                storage.set(this.filterStorageKey, filterObject);
                this.applyFiltersCallback(filterObject);
            }
        },*/

        _showBusyButton: function() {
            this.$('.js-apply').addClass('tb-commentPopupView-button-disabled').find('i').removeClass().addClass('fa fa-spin fa-refresh');
        },

        onApply: function (evt)
        {
        	var that = this;

        	var body = that.$('.js-composerArea').val();
        	if (body.length == 0)
        		return false;

        	this.preventRepeatingClicks(evt);

        	this._showBusyButton();

        	function onTotallyCompleted()
        	{
        		that.$('textarea').val('');
        		srv.clearCache(that.options.commentTarget.entityType);
        		that.options.onPosted && that.options.onPosted.call(that, comment);
        		that.remove();
        	}

        	function onCommentCreated(commentId)
        	{
        		var $attachments = that.$('.js-attachment');

        		function uploadSingleAttachment(attachmentIndex)
        		{
        			var $img = $($attachments[attachmentIndex]);
        			var localUrl = $img.attr('src');
        			//alert('Uploading attachment: ' + localUrl);
        			fileManager.uploadAttachment(localUrl, {
        				target: {
        					entityType: 'comment',
        					entityId: commentId
        				},
        				fileName: 'image-' + (attachmentIndex + 1) + '.jpg',
        				success: function ()
        				{
        					if (attachmentIndex >= $attachments.length - 1)
        					{
        						// Everything is uploaded
        						onTotallyCompleted();
        					} else
        					{
        						// Upload next file
        						uploadSingleAttachment(attachmentIndex + 1);
        					}

        				},
        				error: function ()
        				{
        					// TODO
        					alert('Cannot upload the file');
        				}
        			});
        		}

        		if ($attachments.length > 0)
        		{
        			var uploadIndex = 0;
        			uploadSingleAttachment(uploadIndex);

        		}
        		else
        		{
        			onTotallyCompleted();
        		}
        	}



        	var comment = new CommentModel();
        	//var body = this.$('.js-composerArea').val();

        	comment.set({
        		body: body,
        		parentEntityID: this.options.commentTarget.entityId,
        		parentEntityType: this.options.commentTarget.entityType,
        		permissions: {}
        	});

        	srv.save({
        		model: comment,
        		url: settings.restUrlBase + 'comment',
        		success: function (data)
        		{
        			if (environment.isMobileBrowser() && cordova.plugins.Keyboard.isVisible === true)
        				cordova.plugins.Keyboard.close && cordova.plugins.Keyboard.close();

        			onCommentCreated(data.id);
        		}
        	});

        	evt.preventDefault();



        },

        onAttach: function(evt) {
            var that = this;
            evt && evt.preventDefault();
            attachmentUtil.showAttachUI({
                onAttachmentCreated: function(imageUrl) {
                    var $attachment = $('<img>').attr('src', imageUrl).attr('alt', '').
                        addClass('attachedImagePreview js-attachment');

                    that.$('.js-attachmentContainer').append($attachment);
                    //$('<input type="file">').attr('name', 'attachment-' + (++that._attachmentCounter));
                }
            });
        },

        onAttachmentClicked: function(evt) {
            evt && evt.preventDefault();
            var $img = $(evt.currentTarget);
            /*this.ui.popup.list({
                items: [
                    'Просмотр', 'Удалить'
                ],
                onSelected: function(value) {
                    alert('Hey');
                }
            });*/
            this.ui.popup.list({
                items: [{
                    caption: 'Удалить вложение!',
                    classes: 'tb-confirmDelete',
                    confirmOperation: true
                }, {
                    caption: 'Оставить как есть'
                }],
                classes: 'ui-popup-list ui-popup-list-largePadding',
                onSelected: function(value, item, $li, $root) {
                    if (item.confirmOperation) {
                        $img.remove();
                    }
                }
            });
        },

        /**
         * This method is to be called from customPopupRegistry
         */
        close: function () {
            this.remove();
        }

    });

});