﻿define([
    'underscore',
    'jquery',
    'views/BaseView',
    'text!views/common/EmbeddableActivitiesView_generic.html',
    'text!views/common/EmbeddableActivitiesView_genericForEntity.html',
    'text!views/common/EmbeddableActivitiesView_commentForEntity.html',
    'text!views/common/EmbeddableActivitiesView_showAll.html',
    'utils/modelUtil',
    'utils/entityUtil',
    'utils/locationContextUtil',
    'collections/activities/ActivityCollection',
    'views/collectionReductionSupport',
    'views/common/EmbeddableAttachmentListView',
    'utils/uiManager'
], function(_, $, BaseView, genericTemplate, genericForEntityTemplate, commentForEntityTemplate, showAllTemplate, modelUtil, entityUtil, locationContextUtil, ActivityCollection, collectionReductionSupport, EmbeddableAttachmentListView, uiManager) {

    return BaseView.extend({

        tagName: 'ul',

        collection: null,

        autoUpdate: false,

        showLocationContext: true,
        showEntity: false,
        highlightComments: true,
        relativeTimingEnabled: true,

        locationContextUtil: locationContextUtil,
        entityUtil: entityUtil,

        events: {
            'click .js-showAll': 'onShowAll',
            'click .tb-link': 'onLinkClicked',
            'click .js-item': 'onItemClicked'
        },

        initialize: function(options) {
            this.include(collectionReductionSupport);
            BaseView.prototype.initialize.apply(this, arguments);
            this.collection = modelUtil.convertToCollectionIfNeeded(options.collection, ActivityCollection);
            this.initFieldFromOption('forEntity');
        },

        render: function ()
        {
        	// do not call base method
        	var that = this;
        	this.$el.addClass('tb-activities');
        	if (this.forEntity)
        	{
        		this.$el.addClass('tb-activitiesForEntity');
        	} else
        	{
        		this.$el.addClass('ui-listView');
        	}

        	var html = '';
        	var indices;
        	if (this.forEntity)
        	{
        		indices = this._displayingIndices();
        	} else
        	{
        		indices = {
        			indexFrom: 0,
        			indexToExclusive: this.collection.length,
        			reduced: false
        		};
        	}

        	if (indices.reduced)
        	{
        		html += showAllTemplate.call(this, {
        			extraItemCount: indices.indexFrom
        		});
        	}

        	for (var i = indices.indexFrom; i < indices.indexToExclusive; i++)
        	{
        		var index = that.forEntity ? indices.indexToExclusive - i - 1 : i;
        		var activityModel = this.collection.at(index);
        		var type = activityModel.get('type');
        		var newHtmlChunk;
        		if (that.forEntity)
        		{
        			var className = '';
        			if (that.hasActivityInPushMessages(activityModel.id) === true)
        				className = 'highlighted';

        			if (type === 'createComment')
        			{
        				if (activityModel.get('dependentEntityRef') && activityModel.get('dependentEntityRef').deleted)
        				{
        					continue;
        				}
        				newHtmlChunk = commentForEntityTemplate.call(that, {
        					m: activityModel,
        					index: index,
        					itemClass: className
        				});
        			}
        			else
        			{
        				newHtmlChunk = genericForEntityTemplate.call(that, {
        					m: activityModel,
        					index: index,
        					itemClass: className
        				});
        			}
        		}
        		else
        		{
        			// render generic template
        			newHtmlChunk = genericTemplate.call(that, {
        				m: activityModel,
        				index: index
        			});
        		}
        		//that.forEntity ? (html = newHtmlChunk + html) : (html += newHtmlChunk);
        		html += newHtmlChunk;
        	}
        	this.$el.html(html);
        	this._renderAttachments();
        },

        hasActivityInPushMessages: function (id)
        {
        	var result = false;

        	var currentActivity = this.options && this.options.activityId;
        	if (!currentActivity)
        		return result;

        	var has = _.find(tbPushClient.messages, function (item)
        	{
        		return item.activity && item.activity.id == currentActivity && item.data && item.data.activityIDs && item.data.activityIDs.indexOf(id) != -1;
        	});

        	if (has)
        		result = true;

        	return result;
        },

        _renderAttachments: function() {
            var that = this;
            this.$('.js-attachmentsBlock').each(function() {
                var $container = $(this);
                var commentIndex = $container.attr('data-comment-index');
                var comment = that.collection.at(commentIndex);
//                var attachments = comment.get('attachments');
                var attachments = that._attachmentsOfItem(comment);
                if (!_.isEmpty(attachments)) {
                    var attachmentListView = that.createChildView(EmbeddableAttachmentListView, {
                        attachments: attachments,
                        collection: null,
                        borderBottom: false
                    });
                    attachmentListView.render();
                    attachmentListView.$el.appendTo($container);
                }
            });

        },

        humanizeDateTime: function(dateTime) {
            // TODO !! нужно ли это вообще?
            var m = moment(dateTime);
            var now = moment();
            if (this.relativeTimingEnabled) {

            }
        },

        /*renderAttachments: function(model) {
            var attachments = this._attachmentsOfItem(model);
            if (!_.isEmpty(attachments)) {
                var attachmentListView = this.createChildView(EmbeddableAttachmentListView, {
                    attachments: attachments,
                    collection: null
                });
                attachmentListView.render();
                return attachmentListView.$el.html();
            }
            return '';
        },*/

        _attachmentsOfItem: function(model) {
            var additionalRefDictionary = model.get('additionalRefDictionary');
            return _.filter(additionalRefDictionary, function(ref, refId) {
                return ref.type === 'attachment' && ref.deleted !== true;
            });
        },

        generateActivityLink: function(m) {
            var entityRef = m.get('entityRef');
            if (entityRef) {
                if (entityRef.type && (!entityRef.deleted) && (entityUtil.isEntitySupported(entityRef.type))) {
                    return this.urlBuilder.entity(entityUtil.modeForEntity(entityRef.type), entityRef.id);
                }
            }
            return 'javascript:void(0)';
        },

        shouldGenerateLink: function(m) {
            var entityRef = m.get('entityRef');
            if (entityRef) {
                if (entityRef.type && (!entityRef.deleted) && (entityUtil.isEntitySupported(entityRef.type))) {
                    //return this.urlBuilder.entity(entityUtil.modeForEntity(entityRef.type), entityRef.id);
                    return true;
                }
            }
            return false;
        },

        onLinkClicked: function(evt) {
            evt.preventDefault();
            var href = $(evt.currentTarget).attr('href');
            window.open(href, '_system');
        },

        onItemClicked: function(evt) {
            evt && evt.preventDefault();
            var $target = $(evt.currentTarget);
            var entityType = $target.attr('data-entity-type');
            var entityId = $target.attr('data-entity-id');
            //var url = this.urlBuilder.entity(entityUtil.modeForEntity(entityType), entityId);
            uiManager.openCard(entityType, {
            	entityId: entityId,
				router: this.router
            });

        }
    });

});