﻿/**
 * @class EmbeddableAgendaView
 */
define([
    'underscore',
    'jquery',
    'backbone',
    'views/BaseView',
    'text!views/common/EmbeddableAgendaView_empty.html',
    'text!views/common/EmbeddableAgendaView_task.html',
    'text!views/common/EmbeddableAgendaView_event.html',
    'collections/tasks/TaskCollection',
    'collections/calendar/CalendarEventCollection',
    'utils/modelUtil',
    'utils/locationContextUtil',
    'utils/uiManager',
    'srv',
    'settings',
    'moment'
], function (
    _,
    $,
    Backbone,
    BaseView,
    emptyTemplate,
    taskTemplate,
    eventTemplate,
    TaskCollection,
    CalendarEventCollection,
    modelUtil,
    locationContextUtil,
    uiManager,
    srv,
    settings,
    moment
    )
{
	return BaseView.extend({

		tagName: 'ul',

		collection: null,

		autoUpdate: false,

		emptyCaption: 'Ничего нет',
		supportsEmptyCaption: true,
		showTaskDetails: false,
		showEventDetails: false,
		showLocationContext: false,

		todayAsMoment: null,
		nowAsMoment: null,

		locationContextUtil: locationContextUtil,

		currentYear: null,

		events: {
			'change .js-checkbox.js-completed': 'onCheckboxChanged',
			'click .js-task': 'onTaskClicked',
			'click .js-calendarEvent': 'onCalendarEventClicked'
		},

		initialize: function (options)
		{
			BaseView.prototype.initialize.apply(this, arguments);
			this.initFieldFromOption('emptyCaption');
			this.initFieldFromOption('supportsEmptyCaption');
			this.initFieldFromOption('sorter');
			this.initFieldFromOption('showTaskDetails');
			this.initFieldFromOption('showEventDetails');
			this.initFieldFromOption('showLocationContext');

			this.currentYear = moment().year();

			var tasks = modelUtil.convertToCollectionIfNeeded(options.tasks || [], TaskCollection);
			var calendarEvents = modelUtil.convertToCollectionIfNeeded(options.calendarEvents || [], CalendarEventCollection);

			var taskAndEventCollection = new Backbone.Collection();
			tasks.each(function (m) { m.set('type', 'task'); });
			taskAndEventCollection.add(tasks.models);
			calendarEvents.each(function (m) { m.set('type', 'calendarEvent'); });
			taskAndEventCollection.add(calendarEvents.models);
			this.sorter(taskAndEventCollection);
			this.collection = taskAndEventCollection;
		},

		render: function ()
		{
			// We don't call base method here
			var that = this;
			this.todayAsMoment = moment().startOf('day');
			this.nowAsMoment = moment();
			var html = '';

			this.$el.addClass('ui-listView').addClass('tb-agenda');

			if (!this.collection || this.collection.length === 0)
			{
				this.$el.addClass('tb-emptyList');
				this.supportsEmptyCaption && (html += emptyTemplate.call(this, arguments));
			}
			else
			{
				this.collection.each(function (itemModel)
				{
					var itemType = itemModel.get('type');
					if (itemType === 'task')
					{
						html += taskTemplate.call(that, {
							m: itemModel
						});
					} else if (itemType === 'calendarEvent')
					{
						html += eventTemplate.call(that, {
							m: itemModel
						});
					}
				});
			}
			this.$el.empty().html(html);
			this.ui.form.checkbox(this.$('.js-checkbox'));

		},

		cssClassesForTask: function (taskModel)
		{
			if (!taskModel) return '';
			var classes = [];
			if (taskModel.has('dueOn'))
			{
				var dueDayAsMoment = moment(this.formatter.timeOnToMoment(taskModel.get('dueOn'))).startOf('day');
				if (dueDayAsMoment.unix() < this.todayAsMoment.unix())
				{
					classes.push('tb-overdue');
				}
			}
			if (taskModel.has('priority') && taskModel.get('priority') === 'high')
			{
				classes.push('tb-important');
			}
			if (taskModel.has('status') && taskModel.get('status') === 'completed')
			{
				classes.push('tb-completed');
			}
			(!this.showLocationContext) && (!this.showTaskDetails) && (classes.push('tb-compact'));
			return classes.join(' ');
		},

		cssClassesForEvent: function (eventModel)
		{
			if (!eventModel) return '';
			var classes = [];
			(!this.showLocationContext) && (!this.showEventDetails) && (classes.push('tb-compact'));
			return classes.join(' ');
		},

		/*formatTimeRangeForEvent: function(eventModel) {
            return this.formatter.formatTimeRangeForEvent(eventModel);
        },*/

		sorter: function (collectionToSort)
		{
			var that = this;
			var futureMomentAsUnix = this.formatter.futureMoment().unix();
			collectionToSort.sortBy(function (m)
			{
				if (m.has('startOn'))
				{
					return that.formatter.timeOnToMoment(m.get('startOn')).unix();
				}
				else if (m.has('dueOn'))
				{
					return that.formatter.timeOnToMoment(m.get('dueOn')).unix();
				}
				else
				{
					return futureMomentAsUnix;
				}
			});
		},

		onCheckboxChanged: function (evt)
		{
			var that = this;
			srv.clearCache('task');
			srv.clearCache('calendar');
			var $input = $(evt.currentTarget);
			var checked = ($input.val() === 'true');
			var $li = $input.closest('li');
			if (checked !== $li.hasClass('tb-completed'))
			{
				var entityId = $li.attr('entity-id');
				setTimeout(function ()
				{
					var model = that.collection.get(entityId);
					model.set('status', checked ? 'completed' : 'active');
					srv.save({
						//patch: true,
						model: model,
						url: settings.restUrlBase + 'task/' + entityId
					});
				}, 1);
			}
			checked ? $li.addClass('tb-completed') : $li.removeClass('tb-completed');

		},

		onTaskClicked: function (evt)
		{
			var entityId = $(evt.currentTarget).attr('entity-id');
			//this.router.navigate(this.urlBuilder.task(entityId), {trigger: true, replace: false});
			uiManager.openCard('task', {
				entityId: entityId,
				router: this.router
			});
		},

		onCalendarEventClicked: function (evt)
		{
			var entityId = $(evt.currentTarget).attr('entity-id');
			//this.router.navigate(this.urlBuilder.calendarEvent(entityId), {trigger: true, replace: false});
			uiManager.openCard('calendarEvent', {
				entityId: entityId,
				router: this.router
			});
		}

	});

});

