﻿define([
    'underscore',
    'jquery',
    'backbone',
    'views/BaseView',
    'text!views/common/EmbeddableAttachmentListView.html',
    'srv',
    'settings',
    'utils/attachmentUtil'
], function (
    _,
    $,
    Backbone,
    BaseView,
    template,
    srv,
    settings,
    attachmentUtil
    )
{

	/*var googleViewerSupportedTypes = [
        'application/msexcel',
        'application/msword',
        'application/pdf',
        'text/plain'
    ];*/
	var googleViewerSupportedExtensions = [
        'jpeg', 'jpg', 'png', 'gif', 'tiff', 'bmp',
        'mpeg4', 'mp4', 'mov', 'avi', 'wmv', 'flv', 'ogg',
        'txt',
        'css', 'htm', 'html', 'php', 'c', 'cpp', 'h', 'hpp', 'js',
        'doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'xps',
        'pdf', 'svg', 'ps', 'eps', 'ttf',
        'ai', 'psd', 'dxf',
        'zip', 'rar', 'tar', 'gzip', 'gz'
	];

	return BaseView.extend({

		delegateEvents: function ()
		{
			BaseView.prototype.delegateEvents.apply(this, arguments);
		},

		template: template,

		//collection: null,
		attachments: null,

		borderBottom: true,

		autoUpdate: false,

		events: {
			'click .js-attachment': 'onDownload',
			'click .js-attachment-delete': 'onDelete',
		},

		parentPerms:{ canEdit:false, canDelete:false },

		initialize: function (options)
		{
			BaseView.prototype.initialize.apply(this, arguments);
			if (options.collection instanceof Backbone.Collection)
			{
				this.attachments = options.collection.toJSON();
			}
			else if (options.attachments)
			{
				this.attachments = options.attachments;
			}
			else
			{
				this.attachments = [];
			}
			this.initFieldFromOption('borderBottom');

			if (options.parentPerms)
				this.parentPerms = _.clone(options.parentPerms);
		},

		render: function ()
		{
			BaseView.prototype.render.apply(this, arguments);
			(this.borderBottom && this.attachments && this.attachments.length > 0) && (this.$el.addClass('ui-blockBorderBottom'));
		},

		onDownload: function (evt)
		{
			var that = this;

			if (that.isDownloading === true)
				return;

			that.isDownloading = true;

			var $fileLi = $(evt.currentTarget).closest('li');

			var attachmentId = $fileLi.data('attachmentId');
			var attachment = _.find(this.attachments, function (item) { return item.id == attachmentId; });
			if (!attachment)
			{
				that.isDownloading = false;
				return;
			}

			that.pubSub.trigger('entity:progress');

			$fileLi.find('.js-download-progress').show();
			$fileLi.find('.tb-attachmentList-attachment').hide();

			attachmentUtil.processAttachmentWithTicketAndDownload({
				attach: attachment,
				success: function ()
				{
					that.isDownloading = false;

					$fileLi.find('.js-download-progress').hide();
					$fileLi.find('.tb-attachmentList-attachment').show();

					that.pubSub.trigger('entity:stopProgress');
				},
				error: function ()
				{
					that.isDownloading = false;

					$fileLi.find('.js-download-progress').hide();
					$fileLi.find('.tb-attachmentList-attachment').show();

					that.pubSub.trigger('entity:stopProgress');
				}
			});
		},

		//onView: function (evt)
		//{
		//	return;
		//	var that = this;
		//	evt && evt.preventDefault();
		//	var attachmentId = $(evt.currentTarget).closest('.js-attachment').attr('data-attachment-id');
		//	attachmentUtil.processAttachmentWithTicket(attachmentId, function (attachmentId, ticket, expires)
		//	{
		//		that._openGoogleViewer(attachmentId, ticket, expires);
		//	});
		//},

		onDelete: function (evt)
		{
			var that = this;
			var $elem = $(evt.currentTarget);
			var attachmentId = $elem.closest('li').data('attachmentId');

			var attachment = _.find(this.attachments, function (item) { return item.id == attachmentId; });
			if (!attachment)
				return;

			var positioningClass = 'ui-popup-cardArea';
			this.ui.popup.list({
				items: [
					{
						caption: 'Подтверждаю, удалить файл ' + attachment.title,
						classes: 'tb-confirmDelete',
						confirmOperation: true
					},
					{
						caption: 'Отменить'
					}
				],
				classes: 'ui-popup-list ui-popup-list-largePadding ' + positioningClass,
				onSelected: function (value, item, $li, $root)
				{
					if (item.confirmOperation)
					{
						that.deleteAttachment(attachmentId, $elem);
					}
				}
			});
		},

		deleteAttachment: function (attachmentId, $elem)
		{
			var that = this;

			that.pubSub.trigger('entity:progress');

			var cachedUrl = this.collection.get(attachmentId).url.split(settings.restUrlBase)[1];

			$elem.find('i.fa').removeClass('fa-trash-o').addClass('fa-refresh fa-spin');

			srv.del({
				entityType: 'attachment',
				entityId: attachmentId,
				success: function ()
				{
					srv.clearCache(cachedUrl);
					srv.clearCache(cachedUrl.replace('attachment', 'activity'));
					$elem.closest('li[data-attachment-id]').remove();

					that.pubSub.trigger('entity:stopProgress');
				},
				error: function (jqXHR, textStatus, errorThrown)
				{
					srv.clearCache(cachedUrl);
					srv.clearCache(cachedUrl.replace('attachment', 'activity'));
					$elem.find('i.fa').removeClass('fa-refresh fa-spin').addClass('fa-trash-o');
					if (jqXHR.status == 404)
					{
						$elem.closest('li[data-attachment-id]').remove();
						jqXHR.errorAlreadyHandled = true;
					}

					that.pubSub.trigger('entity:stopProgress');
				},
			});
		},

		/*_processAttachmentWithTicket: function(attachmentId, callback) {
            var that = this;
            srv.fetch({
                url: settings.restUrlBase + 'ticket/' + attachmentId,
                forceUpdate: true,
                success: function(result) {
                    var ticket = result.key;
                    var expires = result.expires;
                    callback && callback(attachmentId, ticket, expires);
                }
            });
        },*/

		_documentUrlWithTicket: function (attachmentId, ticket, expires)
		{
			return settings.restUrlBase + 'attachment/' + attachmentId + '?ticket=' + ticket + '&expires=' + expires;// + '&inline=true';
			// var resUrl = file.url + '?ticket=' + data.result.key + '&expires=' + data.result.expires;
		},

		_openGoogleViewer: function (attachmentId, ticket, expires)
		{
			var documentUrl = this._documentUrlWithTicket(attachmentId, ticket, expires);
			var encodedDocumentUrl = encodeURIComponent(documentUrl);
			var googleUrl = 'https://docs.google.com/viewer?url=' + encodedDocumentUrl + '&embedded=false';
			window.open(googleUrl, '_blank', 'location=yes');
		},

		_canUseGoogleViewer: function (attachment)
		{
			if (!attachment)
			{
				return false;
			}

			var title = (attachment.title || '').toLowerCase();
			return _.some(googleViewerSupportedExtensions, function (supportedExtension)
			{
				return title.endsWith('.' + supportedExtension);
			});
		}

	});

});