﻿/**
 * @class ListViewPage
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'text!views/common/ListViewPage_header.html',
    'utils/photoBank',
    'utils/uiManager'
], function(
    _,
    $,
    BasePageView,
    headerTemplate,
    photoBank,
    uiManager
    ) {

    return BasePageView.extend({

        headerTemplate: headerTemplate,
        photoBank: photoBank,

        itemTemplate: null,
        decorateItem: null,

        applyCallback: null,
        multipleSelection: false,
        preselectedIds: [],
        selectCaption: 'Выбрать',
        usePhotos: false,
        defaultIcon: null,

        scroller: null,

        items: null,

		notSelectableByCard:true,

        events: {
            'click .js-apply': 'onApply',
            'click .js-item': 'onItemClicked',
            'click .js-close': 'onClose'
        },

        initialize: function(options) {
            BasePageView.prototype.initialize.apply(this, arguments);
            this.initFieldFromOption('items');
            this.initFieldFromOption('applyCallback');
            this.initFieldFromOption('multipleSelection');
            this.initFieldFromOption('preselectedIds');
            this.initFieldFromOption('itemTemplate');
        },

        render: function() {
            var that = this;
            this.$el.addClass('tb-listViewPage');
            uiManager.isTabletMode() && this._addObstacle();

            var args = arguments;
            //that.ui.helpers.requestAnimationFrame(function ()
            //{
            	BasePageView.prototype.render.apply(that, args);
            //});
            //setTimeout(function ()
        	//{
            //that.ui.helpers.requestAnimationFrame(function ()
            //{
            //	that.$('.js-content').height($(window).height() - that.$('.js-header').height());
            //	that.scroller = that._createScroller();
            //});
            //}, 300);
        },

        onClose: function() {
            this.closePage();
        },

        onApply: function(evt) {
            evt && evt.preventDefault();

            var $elements = this.$('.tb-listViewPage-selectedItem');
            var itemIds = _.map($elements, function(domElement) {
                return $(domElement).attr('data-item-id');
            });
            var items = _.filter(this.options.items, function(it) { return _.contains(itemIds, it.id); });
            this.applyCallback && this.applyCallback.call(this, itemIds, items);
            this.closePage();

        },

        onItemClicked: function(evt) {
            evt && evt.preventDefault();
            var $element = $(evt.currentTarget);
            var itemId = $element.attr('data-item-id');
            var item = _.find(this.options.items, function(it) { return it.id === itemId; });
            if (item.selectable === false) {
                return;
            }
            if (this.multipleSelection) {
                if ($element.hasClass('ui-empty')) {
                    this.$('.tb-listViewPage-selectedItem').removeClass('tb-listViewPage-selectedItem');
                    // TODO call this.applyCallback at once?
                } else {
                    $element.toggleClass('tb-listViewPage-selectedItem');
                }

            } else {
                this.applyCallback && this.applyCallback.call(this, itemId, item);
                this.closePage();
            }
        },

        contentTemplate: function() {
            var that = this;
            var html = '';

            if (that.itemTemplate) {
                _.each(that.items, function (item) {
                    // completely custom rendering
                    html += that.itemTemplate.call(that, {item: item});
                });

            } else {
                var $ul = $('<ul></ul>');
                _.each(that.items, function (item) {
                    var $li = $('<li></li>');
                    $li.addClass('js-item').attr('data-item-id', item.id);
                    (item.classes) && ($li.addClass(item.classes));
                    if (_.contains(that.preselectedIds, item.id)) {
                        $li.addClass('tb-listViewPage-selectedItem');
                    }
                    if (that.usePhotos && !item.icon) {
                        $li.append($('<img/>').attr('src', that.photoBank.getPhotoUrl(item.id)).
                            attr('alt', '').addClass('tb-listViewPage-itemImage tb-listViewPage-itemPhoto'));
                    } else {
                        var icon = item.icon ? item.icon : that.defaultIcon;
                        $li.append($('<i></i>').
                            addClass('tb-listViewPage-itemIcon tb-listViewPage-itemImage').
                            addClass('fa fa-' + icon));
                    }
                    $li.append($('<span></span>').addClass('tb-listViewPage-itemCaption').text(item.caption));
                    that.decorateItem && that.decorateItem.call(that, $li, item);
                    $li.appendTo($ul);
                });
                html = $ul[0].outerHTML;
            }
            return html;
        },

        _addObstacle: function() {
            $('<div id="tb-listViewPage-obstacle"></div>').addClass('ui-popup-obstacle').appendTo(document.body);
        },

        _removeObstacle: function() {
            $('#tb-listViewPage-obstacle').remove();
        },

        _createScroller: function() {
        	var scroller = new IScroll(this.$('.js-content')[0], {
        		tap: false,
        		preventDefault: true,
        		//preventDefaultException: { tagName: /^(INPUT|TEXTAREA|BUTTON|A|SELECT|LI)$/ },
        		click: false,

        		bounce: false,
        	});
        },

        remove: function() {
            this._removeObstacle();
            this.scroller && this.scroller.destroy();
            BasePageView.prototype.remove.apply(this, arguments);
        }



    });

});