﻿/**
 * @class DiscussionCardView
 */
define(['underscore', 'jquery', 'views/BaseEntityCardView', 'models/discussions/DiscussionTopicModel', 'utils/uiManager', 'text!views/discussions/DiscussionCardView_data.html', 'srv', 'views/common/WorkspaceListViewPage', 'utils/cache/structureCache', 'utils/cache/currentUserCache'], function (_, $, BaseEntityCardView, DiscussionTopicModel, uiManager, dataTemplate, srv, WorkspaceListViewPage, structureCache, currentUserCache) {

    var DiscussionCardView = BaseEntityCardView.extend({

        entityType: 'discussion',

        dataTemplate: dataTemplate,

        focusedFieldSelector: '.js-discussionTitle',

        emptyCard: {
            caption: 'Выберите обсуждение из списка, чтобы посмотреть более подробную информацию',
            icon: 'comments'
        },

        refData: {
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        initialize: function (options) {
            BaseEntityCardView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new DiscussionTopicModel();
        },

        _createControls: function ()
        {
        	var canEdit = true;
        	if (this.model.get('permissions'))
        		canEdit = this.model.get('permissions').canEdit;

        	this.ui.form.editInPlace(this.$('[bound-to="title"]'), {
        		emptyCaption: 'Тема обсуждения...',
        		viewerClasses: 'tb-discussion-topicTitle js-discussionTitle',
        		canEdit: canEdit
        	});

        	this.ui.form.editInPlace(this.$('[bound-to="body"]'), {
        		emptyCaption: 'Текст обсуждения...',
        		viewerClasses: 'tb-discussion-commentBody',
        		canEdit: canEdit
        	});

        	var structureItems = WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
        		//myRootCaption: 'Мои обсуждения'
        		excludeMyRoot: true
        	});

        	this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
        		listViewPageClass: WorkspaceListViewPage,
        		items: structureItems,
        		icon: 'suitcase',
        		emptyCaption: 'Проект не выбран',
        		onSelected: function (value, item)
        		{
        		}
        	});

        	var locationContext = this.entityLocationContext();
        	if (locationContext && locationContext.workspaceId)
        	{
        		this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);
        	}

        	this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
        },



        onSaved: function() {
            srv.clearCache('discussionTopic');
            srv.clearCache('discussion');
            //this.router.tryForceUpdateCurrentView();
        }

    });

    uiManager.registerCardViewClass(DiscussionCardView.prototype.entityType, DiscussionCardView);
    uiManager.registerCardViewClass('discussionTopic', DiscussionCardView);

    return DiscussionCardView;
});
