﻿/**
 * @class DiscussionListView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'text!views/discussions/DiscussionListView_data.html',
    'collections/discussions/DiscussionTopicCollection',
    'srv',
    'settings',
    'utils/locationContextUtil',
    'views/common/BaseLowerToolbar',
    'utils/uiManager'
], function (_, $, BaseToolView, dataTemplate, DiscussionTopicCollection, srv, settings, locationContextUtil, BaseLowerToolbar, uiManager) {
    return BaseToolView.extend({

        cardEntityType: 'discussion',

        rightButton: {
            icon: 'plus',
            handler: 'onNewDiscussionClicked'
        },

        headerText: 'Обсуждения',

        collection: null,

        locationContextUtil: locationContextUtil,

        events: _.extend({
            'click .js-discussion': 'onDiscussionClicked'
        }, BaseToolView.prototype.events),

        initialize: function (options) {
            BaseToolView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        }
                    ]
                });
            }
        },

        fetchUrl: function () {
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'discussionTopic');
        },
        updateData: function (updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            this.collection = new DiscussionTopicCollection();
            srv.fetch(_.defaults({
                collection: this.collection,
                url: this.fetchUrl(),
                success: function () {
                    settings.ajaxDebugging && that.log.debug('DiscussionListView: got discussion list (' + that.collection.length + ' items)');
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        renderData: function() {
            this.renderLocationContextCaption();
            this.$content().html(dataTemplate.call(this));
        },

        onNewDiscussionClicked: function(evt) {
            evt && evt.preventDefault();
            var locationContext = this.locationContext();
            uiManager.openCard('discussion', _.extend({
                router: this.router,
                entityId: 'new'
            }, (locationContext.workspaceId ? locationContext : {})));
            /*var locationContext = this.locationContext();
            var url;
            if (!_.isEmpty(locationContext)) {
                url = '#!/discussions/area/' + locationContext.areaId + '/workspace/' + locationContext.workspaceId + '/edit/new';
            } else {
                url = '#!/discussions/edit/new';
            }
            // TODO use urlBuilder
            this.router.navigate(url, {trigger: true, replace: false});*/
        },

        onDiscussionClicked: function(evt) {
            evt && evt.preventDefault();
            var entityId = $(evt.currentTarget).attr('entity-id');
            uiManager.openCard('discussion', {
                entityId: entityId,
                router: this.router
            });
        }






    });
});