﻿/**
 * @class DiscussionView
 */
define([
    'underscore',
    'jquery',
    'backbone',
    'collections/BaseCollection',
    'models/discussions/DiscussionTopicModel',
    //'collections/discussions/DiscussionTopicCollection',
    'views/BaseViewerView',
    'text!views/discussions/DiscussionView.html',
    'text!views/discussions/DiscussionView_data.html',
    'settings',
    'srv',
    'views/common/EmbeddableCommentsView',
    'views/common/CommentComposerPageView',
    'views/common/EmbeddableCommentActivatorView',
    'views/common/EmbeddableAttachmentListView',
    'views/common/BaseLowerToolbar',
    'views/common/CommentComposerPopupView',
    'views/deleteSupport',
    'utils/attachmentUtil',
    'utils/fileManager'
], function (
    _,
    $,
    Backbone,
    BaseCollection,
    DiscussionTopicModel,
    //DiscussionTopicCollection,
    BaseViewerView,
    contentTemplate,
    dataTemplate,
    settings,
    srv,
    EmbeddableCommentsView,
    CommentComposerPageView,
    EmbeddableCommentActivatorView,
    EmbeddableAttachmentListView,
    BaseLowerToolbar,
    CommentComposerPopupView,
    deleteSupport,
    attachmentUtil,
    fileManager
    ) {
    return BaseViewerView.extend({

        contentTemplate: contentTemplate,

        headerText: 'Обсуждение',

        editingAllowed: false,

        _modelLoaded: null,

        attachmentCollection: null,
        _attachmentCollectionLoaded: null,

        commentCollection: null,
        _commentCollectionLoaded: null,

        embeddableCommentsView: null,

        //embeddableCommentComposerView: null,
        embeddableCommentActivatorView: null,

        createModel: function () {
            //return new TaskModel();
            // todo
            return new DiscussionTopicModel();
        },

        initialize: function (options) {
            this.include(deleteSupport);
            BaseViewerView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        },
                        {
                            builtin: 'delete'
                        },
                        {
                            name: 'attach',
                            icon: 'paperclip',
                            onClicked: function($td, item) {
                                attachmentUtil.showAttachUI({
                                    onAttachmentCreated: function(imageUrl) {
                                        that.pubSub.trigger('update:start');
                                        fileManager.uploadAttachment(imageUrl, {
                                            target: {
                                                entityType: that.model.entityType,
                                                entityId: that.model.id
                                            },
                                            fileName: 'image-' + that.formatter.nowAsFileNameString() + '.jpg',
                                            success: function() {
                                                that.update({forceUpdate: true});
                                            }
                                        });
                                    }
                                });

                            }
                        },
                        {
                            name: 'comment',
                            icon: 'comment-o',
                            onClicked: function($td, item) {
                                that.ui.scrollToBottom(true, function() {
                                    var popupView = new CommentComposerPopupView({
                                        commentTarget: {
                                            entityType: that.model.entityType,
                                            entityId: that.model.id
                                        },
                                        onPosted: function() {
                                            that.updateComments();
                                        }
                                    });
                                    popupView.render();
                                });
                            }
                        }
                    ]
                });
            }
        },

        updateData: function (updateOptions) {
            var that = this;

            this.model = this.createModel();
            this.model.id = this.options.entityId;
            this._modelLoaded = false;

            this.attachmentCollection = new BaseCollection(); // TODO use dedicated collection type?
            this._attachmentCollectionLoaded = false;

            this.commentCollection = new BaseCollection(); // TODO use dedicated collection type?
            this._commentCollectionLoaded = false;

            srv.fetch(_.defaults({
                model: this.model,
                success: function () {
                    that._modelLoaded = true;
                    that.onDataUpdated();
                }
            }, updateOptions));


            srv.fetch(_.defaults({
                collection: this.attachmentCollection,
                url: that.model.url() + '/attachment',
                success: function() {
                    that._attachmentCollectionLoaded = true;
                    that.onDataUpdated();
                }
            }, updateOptions));

            srv.fetch(_.defaults({
                collection: this.commentCollection,
                url: that.model.url() + '/comment',
                success: function () {
                    that._commentCollectionLoaded = true;
                    that.onDataUpdated();
                }
            }, updateOptions));

        },

        onDataUpdated: function () {
            if (this._modelLoaded && this._commentCollectionLoaded && this._attachmentCollectionLoaded) {
                this.renderDataWrapper();
            }
        },

        renderData: function () {
            this.$content().find('.js-data').html(dataTemplate.call(this));
            this.renderEmbeddableAttachmentListView();
            this.renderEmbeddableCommentsView();
            //this.renderEmbeddableCommentActivatorView();
            this.$('.js-hiddenBeforeDataRendering').show();
        },


        renderEmbeddableCommentsView: function () {
            if (this.embeddableCommentsView) {
                this.embeddableCommentsView.remove();
            }
            this.embeddableCommentsView = this.createChildView(EmbeddableCommentsView, {
                collection: this.commentCollection
            });
            this.embeddableCommentsView.render();
            var $commentsContainer = this.$content().find('.js-comments');
            $commentsContainer.empty();
            this.embeddableCommentsView.$el.appendTo($commentsContainer);
        },

        updateComments: function () {
            var that = this;
            this.commentCollection = new BaseCollection();
            this._commentCollectionLoaded = false;
            srv.fetch({
                forceUpdate: true,
                collection: this.commentCollection,
                url: that.model.url() + '/comment',
                success: function () {
                    that._commentCollectionLoaded = true;
                    that.renderEmbeddableCommentsView();
                }
            });
        },

        /**
         * @deprecated
         */
        renderEmbeddableCommentActivatorView: function() {

            var that = this;
            if (this.embeddableCommentActivatorView) {
                //this.stopListening(this.embeddableCommentActivatorView);
                this.embeddableCommentActivatorView.remove();
            }
            this.embeddableCommentActivatorView = this.createChildView(EmbeddableCommentActivatorView, {
                onClicked: function() {

                }
            });
            this.embeddableCommentActivatorView.render();
            this.embeddableCommentActivatorView.$el.appendTo(this.$content().find('.js-commentActivator'));

        },

        renderEmbeddableAttachmentListView: function() {
            if (this.embeddableAttachmentListView) {
                this.embeddableAttachmentListView.remove();
            }
            this.embeddableAttachmentListView = this.createChildView(EmbeddableAttachmentListView, {
                collection: this.attachmentCollection,
                borderBottom: false
            });
            this.embeddableAttachmentListView.render();
            this.embeddableAttachmentListView.$el.appendTo(this.$content().find('.js-attachments'));
        }

    });
});