﻿/**
 * @class NewDiscussionView
 */
define([
    'underscore',
    'jquery',
    'views/BaseEditorView',
    'text!views/discussions/NewDiscussionView_data.html',
    'text!views/discussions/NewDiscussionView_header.html',
    'utils/cache/structureCache',
    'views/locationContextSupport',
    'utils/locationContextUtil',
    'models/discussions/DiscussionTopicModel',
    'srv',
    'views/common/WorkspaceListViewPage'
], function(
    _,
    $,
    BaseEditorView,
    dataTemplate,
    headerTemplate,
    structureCache,
    locationContextSupport,
    locationContextUtil,
    DiscussionTopicModel,
    srv,
    WorkspaceListViewPage
    ) {
    return BaseEditorView.extend({

        headerTemplate: headerTemplate,

        refData: {
            //users: allUsersCache.refDataLoader,
            structure: structureCache.refDataLoader

        },

        events: {
            'click .js-save': 'onSaveClicked',
            'click .js-back': 'onBack'
        },

        saveTriggerSelector: '.js-save',

        initialize: function (options) {
            this.include(locationContextSupport);
            BaseEditorView.prototype.initialize.apply(this, arguments);
        },

        createModel: function() {
            return new DiscussionTopicModel();
        },

        onSaveClicked: function(evt) {
            this.save();
            evt.preventDefault();
        },

        renderData: function() {
            var that = this;
            this.$content().html(dataTemplate.call(this));
            this.bindData();

            this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
                listViewPageClass: WorkspaceListViewPage,
                items: WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
                    excludeMyRoot: true
                }),
                icon: 'suitcase',
                emptyCaption: 'Проект не выбран'
            });

            var locationContext = this.locationContext();
            locationContext && locationContext.workspaceId && this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);

            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
            this.ui.form.autoSizeAllInContainer(this.$el);
        },

        beforeSave: function() {
            var locationContext = this.locationContext();
            if (!_.isEmpty(locationContext)) {
                this.model.set('workspaceRef', { id : locationContext.workspaceId });
            }
        },

        onSaved: function() {
            srv.clearCache('discussionTopic');

            var locationContext = this.locationContext();
            /*var url;
            if (!_.isEmpty(locationContext)) {
                url = '#!/discussions/area/' + locationContext.areaId + '/workspace/' + locationContext.workspaceId;
            } else {
                url = '#!/discussions';
            }*/
            this.router.navigate('#!/discussions/view/' +  this.model.id, {trigger: true, replace: true});
        }


    });
});