﻿/**
 * @class TaskCardView
 */
define(['underscore', 'jquery', 'views/BaseEntityCardView', 'models/files/FileItemModel', 'utils/uiManager', 'utils/cache/structureCache', 'utils/cache/currentUserCache', 'text!views/files/FileCardView_data.html', 'views/common/UserListViewPage', 'srv', 'views/common/WorkspaceListViewPage', 'utils/attachmentUtil', 'settings'], function (_, $, BaseEntityCardView, FileItemModel, uiManager, structureCache, currentUserCache, dataTemplate, UserListViewPage, srv, WorkspaceListViewPage, attachmentUtil, settings) {

    var FileCardView = BaseEntityCardView.extend({

        entityType: 'file',

        dataTemplate: dataTemplate,

        supportsAttachments: false,

        focusedFieldSelector: '.js-fileName',

        emptyCard: {
            caption: 'Ничего не выбрано',
            icon: 'folder-open'
        },

        refData: {
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        events: _.extend({
            'click .js-download': 'onDownloadClicked'
        }, BaseEntityCardView.prototype.events),

        initialize: function (options) {
            BaseEntityCardView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new FileItemModel();
        },

        _createControls: function () {
            var that = this;

            this.ui.form.editInPlace(this.$('[bound-to="name"]'), {
                viewerClasses: 'tb-cardView-title js-fileName',
                emptyCaption: 'Название файла...'
            });


            this.ui.form.editInPlace(this.$('[bound-to="description"]'), {
                viewerClasses: 'tb-cardView-description',
                emptyCaption: 'Описание файла...'
            });
            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
        },

        onSaved: function () {
            var that = this;
            srv.clearCache('folder');
            srv.clearCache('file');
        },

        onDownloadClicked: function(evt) {
            var that = this;
            evt && evt.preventDefault();
            //var entityId = $(evt.currentTarget).attr('data-entity-id');
            var entityId = this.model.id;
            attachmentUtil.processAttachmentWithTicket(entityId, function(fileId, ticket, expires) {
                var documentUrl = that._fileUrlWithTicket(fileId, ticket, expires);
                window.open(documentUrl, '_system', 'location=yes');
            });
        },

        _fileUrlWithTicket: function(fileId, ticket, expires) {
            return settings.restUrlBase + 'file/' + fileId +
                '/content?ticket=' + ticket + '&expires=' + expires + '&inline=true';
        },

        deleteEntity: function() {
            var that = this;
            var m = this._modelToDelete();
            srv.del({
                customData: {
                    fileIDs: [m.id],
                    folderIDs: []
                },
                entityType: 'file',
                success: function() {
                    srv.clearCache('folder');
                    srv.clearCache('file');
                    that._onDeleteSuccess();
                }
            });
        },

        reloadAfterSaveExplicitFragment: function() {
            return '#!/files/area/' + this.options.areaId + '/workspace/' + this.options.workspaceId + '/type/file/view/' + this.model.id;
        }

    });

    uiManager.registerCardViewClass(FileCardView.prototype.entityType, FileCardView);

    return FileCardView;
});
