﻿/**
 * @class FolderCardView
 */
define([
    'underscore',
    'jquery',
    'views/BaseEntityCardView',
    'models/files/FileItemModel',
    'utils/uiManager',
    'utils/cache/structureCache',
    'utils/cache/currentUserCache',
    'text!views/files/FolderCardView_data.html',
    'srv',
    'settings'
], function (_, $, BaseEntityCardView, FileItemModel, uiManager, structureCache, currentUserCache, dataTemplate, srv, settings) {

    var FolderCardView = BaseEntityCardView.extend({
        entityType: 'folder',

        dataTemplate: dataTemplate,

        supportsAttachments: false,
        supportsActivity: false,

        focusedFieldSelector: '.js-folderName',

        emptyCard: {
            caption: 'Ничего не выбрано',
            icon: 'folder-open'
        },

        refData: {
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        events: _.extend({
            'click .js-download': 'onDownloadClicked'
        }, BaseEntityCardView.prototype.events),

        initialize: function (options) {
            BaseEntityCardView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new FileItemModel();
        },

        fetchUrl: function() {
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'folder/' + this.model.id);
        },

        _createControls: function () {
            var that = this;

            this.ui.form.editInPlace(this.$('[bound-to="name"]'), {
                viewerClasses: 'tb-cardView-title js-folderName',
                emptyCaption: 'Название папки...'
            });


            this.ui.form.editInPlace(this.$('[bound-to="description"]'), {
                viewerClasses: 'tb-cardView-description',
                emptyCaption: 'Описание папки...'
            });
            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
        },

        onSaved: function () {
            var that = this;
            srv.clearCache('file');
            srv.clearCache('folder');
        }

    });

    uiManager.registerCardViewClass(FolderCardView.prototype.entityType, FolderCardView);

    return FolderCardView;

});