﻿/**
 * @class FolderListingView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'views/BasePageView',
    'text!views/files/FolderListingView_data.html',
    'text!views/files/FolderListingView_data_file.html',
    'text!views/files/FolderListingView_data_folder.html',
    'collections/files/FileItemCollection',
    'srv',
    'settings',
    'utils/locationContextUtil',
    'views/common/BaseLowerToolbar',
    'utils/uiManager',
    'utils/fileUtil',
    'utils/attachmentUtil',
    'utils/fileManager'
], function (_, $, BaseToolView, BasePageView, dataTemplate, dataFileTemplate, dataFolderTemplate, FileItemCollection, srv, settings, locationContextUtil, BaseLowerToolbar, uiManager, fileUtil, attachmentUtil, fileManager) {
    return BaseToolView.extend({

        dataFileTemplate: dataFileTemplate,
        dataFolderTemplate: dataFolderTemplate,

        cardEntityType: 'folder',

        rightButton: {
            icon: 'plus',
            handler: 'onAddClicked'
        },

        headerText: 'Файлы',

        collection: null,

        locationContextUtil: locationContextUtil,

        events: _.extend({
            'click .js-file': 'onFileClicked',
            'click .js-folder': 'onFolderClicked',
            'click .js-up': 'onUpClicked'
        }, BaseToolView.prototype.events),

        initialize: function (options) {
            BaseToolView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        }
                    ]
                });
            }
        },

        fetchUrl: function () {
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, (this.options.folder ? 'folder/' + this.options.folder + '/' : '') + 'file');
        },
        updateData: function (updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            this.collection = new FileItemCollection();
            srv.fetch(_.defaults({
                collection: this.collection,
                url: this.fetchUrl(),
                success: function () {
                    settings.ajaxDebugging && that.log.debug('FolderListingView: got item list (' + that.collection.length + ' items)');
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        render: function() {
            BasePageView.prototype.render.apply(this, arguments);
            this.tryShowCard({
                entityId: this.options.folder
            });
        },

        renderData: function() {
            this.renderLocationContextCaption();
            this.$content().html(dataTemplate.call(this));
        },

        onAddClicked: function(evt) {
            var that = this;
            evt && evt.preventDefault();
            attachmentUtil.showAttachUI({
                items: [
                    {
                        value: 'createFolder',
                        caption: 'Создать папку',
                        icon: 'folder-open',
                        onClick: function() {
                            // TODO
                            alert('Not implemented yet.');
                        }
                    },
                    {
                        value: 'takePhoto',
                        caption: 'Сделать снимок',
                        icon: 'camera'
                    },
                    {
                        value: 'loadFromGallery',
                        caption: 'Загрузить из галереи',
                        icon: 'photo'
                    }
                ],
                onAttachmentCreated: function(imageUrl) {
                    fileManager.uploadFile(imageUrl, {
                        target: {
                            areaId: that.options.areaId,
                            workspaceId: that.options.workspaceId,
                            folderId: that.options.folder
                        },
                        fileName: 'image-' + that.formatter.nowAsFileNameString() + '.jpg',
                        success: function() {
                            that.update({forceUpdate: true});
                            //that.updateAttachments();
                        }
                    });
                }
            });

        },

        onFileClicked: function(evt) {
            evt && evt.preventDefault();
            var entityId = $(evt.currentTarget).attr('data-entity-id');
            uiManager.openCard('file', {
                entityId: entityId,
                explicitFragment: '#!/files/area/' + this.options.areaId + '/workspace/' + this.options.workspaceId + '/type/file/view/' + entityId
            });
        },

        onFolderClicked: function(evt) {
            evt && evt.preventDefault();
            var entityId = $(evt.currentTarget).attr('data-entity-id');
            /*if (uiManager.isTabletMode()) {
                uiManager.openCard('folder', {
                    entityId: entityId
                });
            }*/
            this.router.navigate('#!/files/area/' + this.options.areaId + '/workspace/' + this.options.workspaceId + '/folder/' + entityId, {trigger: true, replace: false});
        },

        onUpClicked: function(evt) {
            evt && evt.preventDefault();
            history.back();
        },

        fileIconClass: function(fileModel) {
            var cssClass = fileUtil.cssClassByFileName(fileModel.get('name'));
            if (cssClass) {
                return 'tb-file-ico ' + cssClass;
            } else {
                return 'fa fa-file-o';
            }
        }

    });
});