﻿/**
 * @class NotificationView
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'text!views/notifications/NotificationView.html',
    'text!views/notifications/NotificationView_header.html',
	'text!views/notifications/NotificationView_item.html',
    'utils/cache/structureCache',
    'utils/navigationUtil',
	'utils/uiManager',
	'utils/entityUtil',
    'utils/locationContextUtil',
    'views/refDataSupport',
    'settings',
    'srv',
	'moment',
    'views/common/BaseLowerToolbar',
	'views/notifications/NotificationCardView'
], function (
    _,
    $,
    BasePageView,
    contentTemplate,
    headerTemplate,
	itemTemplate,
    structureCache,
    navigationUtil,
	uiManager,
	entityUtil, locationContextUtil,
    refDataSupport,
    settings,
    srv,
	moment,
    BaseLowerToolbar,
	NotificationCardView
    )
{
	return BasePageView.extend({

		cardEntityType: 'notification',

		contentTemplate: contentTemplate,
		headerTemplate: headerTemplate,
		itemTemplate: itemTemplate,

		mainData: null,

		locationContextUtil: locationContextUtil,
		entityUtil: entityUtil,

		events: _.extend({
			'click .js-toggleSidebar': 'onToggleSidebarClicked',
			'click .js-item': 'onItemClicked'
		}, BasePageView.prototype.events),

		refData: {
			structure: structureCache.refDataLoader
		},

		initialize: function (options)
		{
			var that = this;
			that.include(refDataSupport);
			BasePageView.prototype.initialize.apply(that, arguments);

			that._initializeLowerToolbar();

			that.listenTo(that.pubSub, 'Push:Message:Recieved', function ()
			{
				that._lastSelectedId = null;

				if (that.router.prevId)
					delete that.router.prevId;

				that.updateData();
			});

			that.listenTo(that.pubSub, 'Push:ClearNotifications', function () { that.onClearNotificationsClick(); });

			if (uiManager.isTabletMode())
				uiManager.openCard(that.cardEntityType, { router: that.router });

			

			that.refreshDateIntervalId = window.setInterval(function () { that.refreshDates(); }, 60000);
		},

		_initializeLowerToolbar: function ()
		{
			var that = this;
			if (!this.lowerToolbarClass)
			{
				this.lowerToolbarClass = BaseLowerToolbar.extend({
					items: [
                        {
                        	builtin: 'clearNotification'
                        }
					]
				});
			}
		},
		//renderLowerToolbar:function(){},

		updateData: function (updateOptions)
		{
			var that = this;

			that._mainDataLoaded = false;

			that.mainData = that.prepareResult();

			that.onDataUpdated();
		},

		prepareResult: function ()
		{
			return _.filter(tbPushClient.messages, function (item) { return item.activity && item.time > 0; });
		},

		renderData: function ()
		{
			var that = this;

			that.renderEmbeddableAgendaView();

			that.$('.ui-content-wrapper').css('bottom', '0');

			var needOpenMessage = _.find(that.mainData, function (item) { return item.open === true; });
			if (needOpenMessage)
			{
				delete needOpenMessage.open;
				that.openItem(needOpenMessage.activity.id);
			}
		},

		renderEmbeddableAgendaView: function ()
		{
			var that = this;
			var $list = that.$('.tb-activities').addClass('tb-emptyList').empty();

			if (that.mainData.length == 0)
			{
				$list.html('<li class="tb-emptyListCaption">Нет уведомлений</li>');
				//that.$('.j-clear-notifications').hide();
				return;
			}
			
			//that.$('.j-clear-notifications').show();

			var html = [];
			$list.removeClass('tb-emptyList');

			//viewed: false, time: new Date().getTime(), 
			//      data:{
			//		entityID: '',
			//		entityType:''
			//		title: '',
			//		areaID: '',
			//		workspaceID: '',
			//		message: '',
			//		activityIDs:[],
			//		activity:{{id: "1679d38d-500f-e511-80c0-00155d000311",…}
					//additionalRefDictionary: null
					//areaRef: {id: "60682429-224d-4908-bf16-fb15fef900a3", caption: "TeamBridge"}
					//createdDateTime: "2015-06-10T09:10:43.409Z"
					//dayTotalActivityCount: 0
					//dependentEntityRef: null
					//details: null
					//entityRef: {type: "file", deleted: false, id: "ae2cd01c-ede7-e411-80c0-00155d000311",…}
					//id: "1679d38d-500f-e511-80c0-00155d000311"
					//initiatorRef: {id: "65553a72-0344-4a16-9687-10e221814bd8", caption: "Башканков Николай"}
					//message: "<initiator>Башканков Николай</initiator> добавил(а) новую версию файла"
					//type: "update"
					//workspaceRef: {id: "141f8e88-4490-4848-9a9e-3032e2cb7f30", caption: "Разработка мобильного клиента"}}

			_.each(that.mainData, function (message)
			{

				html.push(that.itemTemplate.call(that, { m: message }));
			});

			$list[0].insertAdjacentHTML('beforeend', html.join(''));
		},

		onUpdateSuccess: function (cid)
		{
			if (cid != this.cid)
				return;

			var that = this;
			BasePageView.prototype.onUpdateSuccess.apply(that, arguments);

			// highlight not viewed
			_.each(that.mainData, function (item) { if (item.viewed === false) that.highLightElementById(item.activity.id); });

			tbPushClient.setAllViewed();
		},

		onToggleSidebarClicked: function (evt)
		{
			evt.preventDefault();
			this.showSidebar();
		},

		refreshDates: function ()
		{
			var that=this;
			that.$('.j-refresh-time').each(function ()
			{
				var $elem = $(this);
				$elem.text(that.formatDate($elem.data('value')));
			});
		},

		formatDate: function (value)
		{
			//return that.formatter.humanizeActivityDateTime(m.time)
			return moment(value).fromNow();
		},

		remove: function ()
		{
			if (this.refreshDateIntervalId)
				window.clearInterval(this.refreshDateIntervalId);

			BasePageView.prototype.remove.apply(this, arguments);
		},

		onClearNotificationsClick: function ()
		{
			tbPushClient.messages = [];
			this.updateData();
		},

		openItem: function (activityId)
		{
			if (!activityId)
				return;

			var message = _.find(this.mainData, function (item) { return item.activity.id === activityId; });
			if (!message)
				return;

			if (message.activity.type == 'delete' || message.activity.entityRef.deleted === true)
				return;

			uiManager.openCard(message.activity.entityRef.type, {
				entityId: message.activity.entityRef.id,
				router: this.router,
				activityId: message.activity.id
			});
		},

		onItemClicked: function (e)
		{
			this.openItem($(e.currentTarget).data('activityId'));
		},

		highLightElementById: function (activityId)
		{
			var that = this;
			if (!activityId)
				return;

			var $list = this.$('.ui-listView');
			if ($list.length > 0)
			{
				var $li = $list.find('a[data-activity-id="' + activityId + '"]').parent();

				if ($li && $li.length > 0)
				{
					$li.addClass('highlighted');
					setTimeout(function ()
					{
						$li && $li.length > 0 && $li.removeClass('highlighted');
					}, 5000);
				}
			}
		},
	});
});