﻿define(['underscore', 'jquery', 'settings', 'srv', 'pubSub'], function(_, $, settings, srv, pubSub) {

    return {

        _mainDataLoaded: false,

        /**
         * Объект для хранения дополнительных данных
         */
        refData: {},

        /**
         * Инициирует загрузку всех дополнительных данных
         */
        startRefDataLoading: function() {
            // Начать загрузку всех дополнительных данных
            var that = this;
            if (this.isAllDataReady()) {
                this.renderDataWrapper();
            } else {
                _.each(this.refData, function(value, name) {
                    if (_.isNull(value)) {
                        srv.fetch({
                            url: settings.restUrlBase + name,
                            success: function(result) {
                                that.onDataLoaded(name, result);
                            }
                        });
                    } else if (_.isString(value)) {
                        srv.fetch({
                            url: settings.restUrlBase + value,
                            success: function(result) {
                                that.onDataLoaded(name, result);
                            }
                        });
                    } else if (_.isFunction(value)) {
                        value.call(that, name);
                    }

                });
            }
        },

        prepareRefData: function() {
            this.refData = _.clone(Object.getPrototypeOf(this).refData);
        },

        onMainDataLoaded: function(refData) {
            // Переопределить, если нужно
            this._mainDataLoaded = true;
        },

        isMainDataLoaded: function() {
            // Переопределить, если нужно
            return this._mainDataLoaded;
        },

        onDataLoaded: function(refDataName, refData) {
            if (refDataName) {
                // Если указано имя, то это - дополнительные данные, сохраняем их во внутреннем объекте
                this.refData[refDataName] = refData;
            } else {
                // Загружены основные данные (например, модель)
                this.onMainDataLoaded(refData);
            }
            var allDataReady = this.isAllDataReady();
            // Если все необходимые данные загружены, вызываем рендеринг представления
            if (allDataReady) {
                this.renderDataWrapper();
            }
        },

        isAllDataReady: function() {
            var allDataReady = true;
            // check if all ref data loaded
            if (_.some(_.values(this.refData), function(value) { return !_.isObject(value) || _.isFunction(value); })) {
                allDataReady = false;
            }
            // check if main data (model) fetched
            if (!this.isMainDataLoaded()) {
                allDataReady = false;
            }
            return allDataReady;
        },

        getRefData: function(name) {
            var view = this;
            while (view) {
                if (view.refData && view.refData[name]) {
                    return view.refData[name];
                }
                view = view.container;
            }
            return undefined;
        }

    };

});