﻿/**
 * @class SearchView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'text!views/search/SearchView.html',
    'text!views/search/SearchView_searchHits.html',
    'settings',
    'srv',
    'collections/search/SearchHitCollection',
    'utils/renderUtil'
], function (_, $, BaseToolView, contentTemplate, searchHitsTemplate, settings, srv, SearchHitCollection, renderUtil) {
    return BaseToolView.extend({

        contentTemplate: contentTemplate,
        headerText: 'Поиск',

        query: null,

        collection: null,

        autoUpdate: false,

        events: _.extend({
            'click .js-search': 'onSearchClicked',
            'keydown .js-query': 'onQueryKeyDown'
        }, BaseToolView.prototype.events),

        initialize: function (options) {
            BaseToolView.prototype.initialize.apply(this, arguments);
            options.query && (this.query = options.query);
        },

        search: function(query) {
            this.query = query;
            this.update();
        },

        updateData: function(updateOptions) {
            var that = this;
            if (this.query) {
                this.collection = new SearchHitCollection();
                srv.fetch(_.defaults({
                    collection: this.collection,
                    url: settings.restUrlBase + 'search/' + encodeURI(this.query),
                    success: function() {
                        that.onDataUpdated();
                    }
                }, updateOptions));
            } else {
                that.onDataUpdated();
            }
        },

        renderData: function() {
            this.renderLocationContextCaption();
            this.$('.js-searchHits').html(searchHitsTemplate.call(this));
        },

        performSearching: function() {
            this.search(this.$('.js-query').val());
        },

        onSearchClicked: function(evt) {
            this.performSearching();
            evt.preventDefault();
        },

        onQueryKeyDown: function(evt) {
            if (evt.keyCode === 13) {
                this.performSearching();
                evt.preventDefault();
            }
        },

        renderHelper: {
            iconForEntityType: function(entityType) {
                renderUtil.iconForEntityType(entityType);
            },

            generateMarkup: function(brief) {
                return renderUtil.generateMarkup(brief);
            }
        }





    });
});