﻿/**
 * @class NewTaskCardView
 */
define([
    'underscore',
    'jquery',
    'views/BaseCardEditorView',
    'text!views/tasks/NewTaskCardView_data.html',
    'text!views/tasks/NewTaskCardView_header.html',
    'settings',
    'models/tasks/TaskModel',
    'views/locationContextSupport',
    'utils/cache/structureCache',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'utils/locationContextUtil',
    'utils/userUtil',
    'utils/uiManager',
    'srv',
    'views/common/WorkspaceListViewPage',
    'views/common/UserListViewPage'
], function (_, $, BaseCardEditorView, dataTemplate, headerTemplate, settings, TaskModel, locationContextSupport, structureCache, allUsersCache, currentUserCache, locationContextUtil, userUtil, uiManager, srv, WorkspaceListViewPage, UserListViewPage) {
    var NewTaskCardView = BaseCardEditorView.extend({

        headerTemplate: headerTemplate,

        refData: {
            //users: allUsersCache.refDataLoader,
            //users: userUtil.userListRefDataLoader,
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader

        },

        events: {
            'click .js-save': 'onSaveClicked',
            'click .js-back': 'onBack',
            'click .js-assignedToPickerStub': 'onAssignedToPickerStub'
        },

        saveTriggerSelector: '.js-save',

        initialize: function (options) {
            this.include(locationContextSupport);
            BaseCardEditorView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new TaskModel();
        },

        onSaveClicked: function (evt) {
            this.save();
            evt.preventDefault();
        },

        renderData: function () {
            var that = this;
            this.$('.js-content').html(dataTemplate.call(this));
            this.bindData();

            if (this.isNewEntity()) {
                this.$('.js-hideIfNew').hide();
            }

            this.ui.form.datePicker(this.$('.js-date'), {
                emptyCaption: 'Дата не задана',
                supportsEmptyValue: true
            });

            this.ui.form.stateSwitch(this.$('[bound-to="priority"]'), {
                initializeRoot: function ($root, options, $hidden) {
                    $('<i class="fa fa-flag"></i>').appendTo($root);
                },
                states: [
                    {
                        value: 'normal',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.fa').css('color', '#a3a3a3');
                        }
                    },
                    {
                        value: 'high',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.fa').css('color', 'red');
                        }
                    }
                ]
            });
            this.ui.form.stateSwitch(this.$('[bound-to="status"]'), {
                initializeRoot: function ($root, options, $hidden) {
                    $('<i class="fa fa-check"></i>').appendTo($root);
                },
                states: [
                    {
                        value: 'active',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.fa').css('color', '#a3a3a3');
                        }
                    },
                    {
                        value: 'completed',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.fa').css('color', 'green');
                        }
                    }
                ]
            });

            var structureItems = WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
                myRootCaption: 'Мои задачи'
            });

            this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
                listViewPageClass: WorkspaceListViewPage,
                items: structureItems,
                icon: 'suitcase',
                emptyCaption: 'Проект не выбран',
                onSelected: function (value, item) {
                    if (item.myRootWorkspace) {
                        var user = that.refData.currentUser;
                        that.ui.form.destroy(that.$('[bound-to="assignedToRef"]'));
                        that.ui.form.staticPicker(that.$('[bound-to="assignedToRef"]'), {
                            items: [
                                {
                                    id: user.id,
                                    caption: user.name
                                }
                            ],
                            icon: 'user'
                        });
                    } else {
                        that._updateUserList({
                            area: item.areaId,
                            workspace: item.id
                        });
                    }
                }
            });

            var locationContext = this.locationContext();
            if (locationContext && locationContext.workspaceId) {
                var workspaceItem = _.find(structureItems, function (item) {
                    return item.type === 'workspace' && item.id === locationContext.workspaceId;
                });
                if (workspaceItem && workspaceItem.myRootWorkspace) {
                    var user = that.refData.currentUser;
                    that.ui.form.destroy(that.$('[bound-to="assignedToRef"]'));
                    that.ui.form.staticPicker(that.$('[bound-to="assignedToRef"]'), {
                        items: [
                            {
                                id: user.id,
                                caption: user.name
                            }
                        ],
                        icon: 'user'
                    });
                } else {
                    this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);
                    this._updateUserList(locationContext);
                }
            }

            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
            this.ui.form.autoSizeAllInContainer(this.$el);
        },

        _updateUserList: function (selectedLocationContext) {
            var that = this;
            if (_.isEmpty(selectedLocationContext)) {
                selectedLocationContext = this.locationContext();
            }
            var url = srv.makeUrlWithLocationContext(selectedLocationContext, 'user') + '?include=roles;allUsers';
            srv.fetch({
                url: url,
                success: function (users) {
                    that._renderUserList(users);
                }
            });
        },

        _renderUserList: function (users) {
            var $hidden = this.$('[bound-to="assignedToRef"]');
            this.$('.js-assignedToPickerStub').remove();
            this.ui.form.destroy($hidden);
            this.ui.form.listViewPicker($hidden, {
                listViewPageClass: UserListViewPage,
                items: UserListViewPage.prototype.usersToItems(users),
                icon: 'user',
                usePhotos: true,
                emptyCaption: 'Никому не назначена',
                headerText: 'Выберите ответственного'
            });
            $hidden/*.val('')*/.trigger('ui:refresh');
        },

        beforeSave: function () {
            var locationContext = this.locationContext();
            if (!_.isEmpty(locationContext)) {
                this.model.set('workspaceRef', { id: locationContext.workspaceId });
            }
        },

        onSaved: function () {
            srv.clearCache('task');
            srv.clearCache('calendar');

            /*var locationContext = this.locationContext();
             var url;
             if (!_.isEmpty(locationContext)) {
             url = '#!/tasks/area/' + locationContext.areaId + '/workspace/' + locationContext.workspaceId;
             } else {
             url = '#!/tasks';
             }
             // TODO use urlBuilder
             this.router.navigate(url, {trigger: true, replace: false});*/

            this.router.navigate('#!/tasks/view/' + this.model.id, {trigger: true, replace: true});
        },

        onAssignedToPickerStub: function (evt) {
            evt.preventDefault();
            this.ui.popup.message({
                text: 'Укажите проект, чтобы выбрать ответственного'
            });
        }

    });

    uiManager.registerCardViewClass('newTask', NewTaskCardView);

    return NewTaskCardView;
});

