﻿/**
 * @class TaskCardView
 */
define(['underscore', 'jquery', 'views/BaseEntityCardView', 'models/tasks/TaskModel', 'utils/uiManager', 'utils/cache/structureCache', 'utils/cache/currentUserCache', 'text!views/tasks/TaskCardView_data.html', 'views/common/UserListViewPage', 'srv', 'views/common/WorkspaceListViewPage', 'utils/uiExtensions'], function (_, $, BaseEntityCardView, TaskModel, uiManager, structureCache, currentUserCache, dataTemplate, UserListViewPage, srv, WorkspaceListViewPage, uiExtensions)
{

    var TaskCardView = BaseEntityCardView.extend({

        entityType: 'task',

        dataTemplate: dataTemplate,

        focusedFieldSelector: '.js-taskTitle',

        emptyCard: {
            caption: 'Выберите задачу из списка, чтобы посмотреть более подробную информацию',
            icon: 'check'
        },

        refData: {
            structure: structureCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        },

        events: _.extend({
            'click .js-parentTask': 'onParentTaskClicked',
            'click .js-assignedToPickerStub': 'onAssignedToPickerStub',
            'click .js-coAssigneeRefsPickerStub': 'onCoAssigneeRefsPickerStub'
        }, BaseEntityCardView.prototype.events),

        initialize: function (options) {
            BaseEntityCardView.prototype.initialize.apply(this, arguments);
        },

        createModel: function () {
            return new TaskModel();
        },

        _createControls: function () {
            var that = this;

            this.ui.form.editInPlace(this.$('[bound-to="title"]'), {
                viewerClasses: 'tb-cardView-title js-taskTitle',
                emptyCaption: 'Название задачи...'
            });


            this.ui.form.editInPlace(this.$('[bound-to="description"]'), {
                viewerClasses: 'tb-cardView-description',
                emptyCaption: 'Описание задачи...'
            });

            this.ui.form.stateSwitch(this.$('[bound-to="priority"]'), {
                initializeRoot: function ($root, options, $hidden) {
                    $('<i class="ui-pickerIcon fa fa-flag"></i>').appendTo($root);
                    $('<span class="ui-pickerValue"></span>').appendTo($root);
                },
                states: [
                    {
                        value: 'normal',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.ui-pickerIcon').css('color', '#a3a3a3');
                            $root.find('.ui-pickerValue').text('Нормальный приоритет');
                        }
                    },
                    {
                        value: 'high',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.ui-pickerIcon').css('color', 'red');
                            $root.find('.ui-pickerValue').text('Высокий приоритет');
                        }
                    }
                ]
            });
            this.ui.form.stateSwitch(this.$('[bound-to="status"]'), {
                initializeRoot: function ($root, options, $hidden) {
                    $('<i class="ui-pickerIcon fa fa-check"></i>').appendTo($root);
                    $('<span class="ui-pickerValue"></span>').appendTo($root);
                },
                states: [
                    {
                        value: 'active',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.ui-pickerIcon').css('color', '#a3a3a3');
                            $root.find('.ui-pickerValue').text('Активная');
                        }
                    },
                    {
                        value: 'completed',
                        decorateState: function (state, $root, options, $hidden) {
                            $root.find('.ui-pickerIcon').css('color', 'green');
                            $root.find('.ui-pickerValue').text('Завершена');
                        }
                    }
                ]
            });

            if (this.isNew())
            {
            	this.ui.form.datePicker(this.$('.js-date'), {
            		emptyCaption: 'Дата не задана',
            		supportsEmptyValue: true
            	});
            }
            else
            {
            	var dueOn = this.model.get('dueOn');
            	if (!dueOn || _.isUndefined(dueOn.dateTime))
            		this.ui.form.datePicker(this.$('.js-date'), {
            			emptyCaption: 'Дата не задана',
            			supportsEmptyValue: true
            		});
            	else
            		uiExtensions.form.dateTimePicker(this.$('.js-date'), {
            			emptyCaption: 'Дата не задана',
            			icon: 'calendar',
            			supportsEmptyValue: true
            		});
            }

            var structureItems = WorkspaceListViewPage.prototype.structureToItems(this.refData.structure, {
                myRootCaption: 'Мои задачи'
            });

            this.ui.form.listViewPicker(this.$('[bound-to="workspaceRef"]'), {
                listViewPageClass: WorkspaceListViewPage,
                items: structureItems,
                icon: 'suitcase',
                ignoreOtherElements: uiManager.isTabletMode(),
                emptyCaption: 'Проект не выбран',
                onSelected: function (value, item) {
                    if (item.myRootWorkspace) {
                        var user = that.refData.currentUser;
                        that.ui.form.destroy(that.$('[bound-to="assignedToRef"]'));
                        that.ui.form.staticPicker(that.$('[bound-to="assignedToRef"]'), {
                            items: [
                                {
                                    id: user.id,
                                    caption: user.name
                                }
                            ],
                            icon: 'user'
                        });
                        that.$('[bound-to="coAssigneeRefs"]').val('');
                        that.ui.form.destroy(that.$('[bound-to="coAssigneeRefs"]'));
                    } else {
                        that._updateUserList({
                            area: item.areaId,
                            workspace: item.id
                        });
                    }
                }
            });

            var locationContext = this.entityLocationContext();
            if (locationContext && locationContext.workspaceId) {
                var workspaceItem = _.find(structureItems, function (item) {
                    return item.type === 'workspace' && item.id === locationContext.workspaceId;
                });
                if (workspaceItem && workspaceItem.myRootWorkspace) {
                    var user = that.refData.currentUser;
                    that.ui.form.destroy(that.$('[bound-to="assignedToRef"]'));
                    that.ui.form.staticPicker(that.$('[bound-to="assignedToRef"]'), {
                        items: [
                            {
                                id: user.id,
                                caption: user.name
                            }
                        ],
                        icon: 'user'
                    });
                    that.$('[bound-to="coAssigneeRefs"]').val('');
                    that.ui.form.destroy(that.$('[bound-to="coAssigneeRefs"]'));
                } else {
                    this.$('[bound-to="workspaceRef"]').val(locationContext.workspaceId);
                    this._updateUserList(locationContext);
                }
            }
            this.$('input[type="hidden"][bound-to]').trigger('ui:refresh');
        },

        _updateUserList: function (selectedLocationContext) {
            var that = this;
            if (_.isEmpty(selectedLocationContext)) {
                selectedLocationContext = this.locationContext();
            }
            var url = srv.makeUrlWithLocationContext(selectedLocationContext, 'user') + '?include=roles;allUsers';
            srv.fetch({
                url: url,
                success: function (users) {
                    that._renderUserList(users);
                }
            });
        },

        _renderUserList: function (users) {
            var $assignedToHidden = this.$('[bound-to="assignedToRef"]');
            this.$('.js-assignedToPickerStub').remove();
            this.ui.form.destroy($assignedToHidden);
            this.ui.form.listViewPicker($assignedToHidden, {
                listViewPageClass: UserListViewPage,
                items: UserListViewPage.prototype.usersToItems(users),
                icon: 'user',
                usePhotos: true,
                emptyCaption: 'Никому не назначена',
                headerText: 'Выберите ответственного',
                ignoreOtherElements: uiManager.isTabletMode()
            });
            $assignedToHidden.trigger('ui:refresh');

            var $coAssigneeRefsHidden = this.$('[bound-to="coAssigneeRefs"]');
            this.$('.js-coAssigneeRefsPickerStub').remove();
            this.ui.form.destroy($coAssigneeRefsHidden);
            this.ui.form.listViewPicker($coAssigneeRefsHidden, {
                listViewPageClass: UserListViewPage,
                items: UserListViewPage.prototype.usersToItems(users, 'Нет соисполнителей'),
                icon: 'users',
                usePhotos: true,
                emptyCaption: 'Нет соисполнителей',
                headerText: 'Соисполнители',
                multipleSelection: true,
                ignoreOtherElements: uiManager.isTabletMode()
            });
            $coAssigneeRefsHidden.trigger('ui:refresh');
        },

        onSaved: function () {
            var that = this;
            srv.clearCache('task');
            srv.clearCache('calendar');
            srv.clearCache('planning');
            //this.router.tryForceUpdateCurrentView();
        },

        onParentTaskClicked: function(evt) {
        	if (uiManager.isTabletMode())
        	{
        		var locationContext = this.locationContext();
        		uiManager.openCard('task', _.extend({
        			router: this.router,
        			entityId: this.model.get('parentTaskRef').id
        		}, (locationContext.workspaceId ? locationContext : {})));
        	}
        	else
        		this.model.get('parentTaskRef') && this.router.navigate(this.urlBuilder.task(this.model.get('parentTaskRef').id), { trigger: true, replace: false });
        },

        onAssignedToPickerStub: function (evt) {
            //evt.preventDefault();
            this.ui.popup.message({
                text: 'Укажите проект, чтобы выбрать ответственного',
                classes: 'ui-popup-message ui-popup-cardArea'
            });
        },

        onCoAssigneeRefsPickerStub: function (evt) {
            //evt.preventDefault();
            this.ui.popup.message({
                text: 'Укажите проект, чтобы выбрать соисполнителей',
                classes: 'ui-popup-message ui-popup-cardArea'
            });
        }

    });

    uiManager.registerCardViewClass(TaskCardView.prototype.entityType, TaskCardView);

    return TaskCardView;
});
