﻿/**
 * @class TaskListView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'text!views/tasks/TaskListView.html',
    'settings',
    'srv',
    'collections/tasks/TaskCollection',
    'views/common/EmbeddableAgendaView',
    'utils/routingUtil',
    'utils/formatter',
    'utils/uiManager',
    'moment',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache',
    'views/tasks/TaskFiltersPopupView',
    'views/tasks/TaskSortPopupView',
    'storage',
    'views/common/BaseLowerToolbar',
    'views/filterSupport'
], function (_, $, BaseToolView, contentTemplate, settings, srv, TaskCollection, EmbeddableAgendaView, routingUtil, formatter, uiManager, moment, allUsersCache, currentUserCache, TaskFiltersPopupView, TaskSortPopupView, storage, BaseLowerToolbar, filterSupport) {
    return BaseToolView.extend({

        contentTemplate: contentTemplate,

        collection: null,

        embeddableAgendaView: null,

        filterPopupClass: TaskFiltersPopupView,

        rightButton: {
            icon: 'plus',
            handler: 'onNewTaskClicked'
        },

        cardEntityType: 'task',

        refData: _.extend({
            allUsers: allUsersCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        }, BaseToolView.prototype.refData),

        events: _.extend({
            //'click .js-filtersActivator': 'onActivateFilters'
            /*'click .js-refresh': function() {
             this.updateWithIndicator();
             }*/
        }, BaseToolView.prototype.events),

        onSyncLog: function (evt) {
            var that = this;
            evt.preventDefault();
            this.log.sync({
                success: function () {
                    that.ui.popup.message({
                        text: 'Спасибо! Информация успешно отправлена'
                    });
                },
                error: function () {
                    that.ui.popup.message({
                        text: 'Увы! Не получилось ничего отправить'
                    });
                }
            });
        },

        headerText: function () {
            return 'Задачи';
        },

        initialize: function (options) {
            this.include(filterSupport);
            BaseToolView.prototype.initialize.apply(this, arguments);
            //this.filterModel = new TaskFilterModel();

            // restore filters value from storage
            this.restoreFilters();
            this._restoreSortParams();
            this._initializeLowerToolbar();
        },

        _restoreSortParams: function() {
            var sortStorageKey = TaskSortPopupView.prototype.sortStorageKey;
            var sortParams = storage.get(sortStorageKey);
            if (sortParams) {
                this.sortParams = JSON.parse(sortParams)
            } else {
                sortParams = TaskSortPopupView.prototype.sortModelClass.prototype.defaults;
                storage.set(sortStorageKey, sortParams);
                this.sortParams = sortParams;
            }
        },

        _initializeLowerToolbar: function () {
            function iconForSortBy(sortBy) {
                switch (sortBy) {
                    case 'title':
                        return 'font';
                    case 'dueOn':
                    case 'dueDate': // for backward compatibility
                    case 'completedDateTime':
                    case 'createdDateTime':
                        return 'calendar';
                    case 'assignedTo':
                        return 'user';
                    case 'priority':
                    default:
                        return 'flag';
                }
            }
            function iconForSortDirection(sortDirection) {
                return sortDirection === 'desc' ? 'long-arrow-up' : 'long-arrow-down';
            }
            function removeFaIcon($el) {
                var el = $el[0];
                var classes = el.className.split(/\s+/);
                for (var i = 0; i < classes.length; i++) {
                    var cls = classes[i];
                    if (cls.indexOf('fa-') === 0) {
                        $el.removeClass(cls);
                        break;
                    }
                }
            }
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh',
                            width: '33%'
                        },
                        {
                            builtin: 'filter',
                            width: '33%'
                        },
                        {
                            name: 'sort',
                            /*icon: 'sort-amount-asc',*/
                            onClicked: function ($td, item) {
                                var that = this;
                                var view = new TaskSortPopupView({
                                    applySortCallback: function(sortParams) {
                                        that.parentView.applySort(sortParams);
                                        var $sortBy = that.$('.js-sortBy');
                                        removeFaIcon($sortBy);
                                        $sortBy.addClass('fa-' + iconForSortBy(sortParams.sortBy));
                                        var $sortDirection = that.$('.js-direction');
                                        removeFaIcon($sortDirection);
                                        $sortDirection.addClass('fa-' + iconForSortDirection(sortParams.sortDirection));
                                    },
                                    parentView: this.parentView
                                });
                                view.render();
                            },
                            decorateItem: function ($td, item) {
                                var sortBy = this.parentView.sortParams ? this.parentView.sortParams.sortBy : undefined;
                                $('<i class="fa"></i>').addClass('js-sortBy fa-'+iconForSortBy(sortBy)).appendTo($td).css('margin-right', '0.1em');
                                var sortDirection = this.parentView.sortParams ? this.parentView.sortParams.sortDirection : undefined;
                                $('<i class="fa"></i>').addClass('js-direction fa-'+iconForSortDirection(sortDirection)).appendTo($td);
                            }
                        }
                    ]
                });
            }
        },

        applySort: function(sortParams) {
            this.sortParams = sortParams;
            var that = this;
            setTimeout(function ()
            {
            	that.update();
            }, 1);
            /*if (this.lowerToolbar) {
                var $sortEl = this.lowerToolbar.getItemElement$('sort');
                if (this.sortParams && this.sortParams.sortBy) {
                    var $fa = $sortEl.find('.fa').removeClass('fa-sort-amount-asc fa-sort-amount-desc');
                    $fa.addClass('fa-sort-amount-' + this.sortParams.sortDirection);
                }
            }*/
        },

        routeArgsToOptions: function (args) {
            function createRangeFilter(filterParam, valueFrom, valueTo, filterParamCaption, valueCaption) {
                return {
                    type: 'range',
                    param: filterParam,
                    valueFrom: valueFrom,
                    valueTo: valueTo,
                    paramCaption: filterParamCaption,
                    valueCaption: valueCaption
                };
            }

            function createListFilter(filterParam, values, filterParamCaption, valueCaption) {
                return {
                    range: 'list',
                    param: filterParam,
                    values: values,
                    paramCaption: filterParamCaption,
                    valueCaption: valueCaption
                };
            }

            var options = {};
            options.flags = {};
            var filters = [];
            var dateTimeFrom, dateTimeTo;
            if (args['list']) {
                if (args['list'] === 'today') {
                    var today = formatter.todayAsString();
                    dateTimeFrom = formatter.startOfDay(today);
                    dateTimeTo = formatter.endOfDay(today);
                    filters.push(createRangeFilter('date', dateTimeFrom, dateTimeTo, 'Дата', 'сегодня'));
                    options.flags.filteredByToday = true;
                }
            }
            if (args['date']) {
                var date = args['date'];
                dateTimeFrom = formatter.startOfDay(date);
                dateTimeTo = formatter.endOfDay(date);
                var humanDate = formatter.serviceDateToHuman(date);
                filters.push(createRangeFilter('date', dateTimeFrom, dateTimeTo, 'Дата', humanDate));
            }
            if (args['assignedTo']) {
                if (args['assignedTo'] === 'all') {
                    // ничего не добавляем к фильтру
                }
            } else {
                // если явно не указано, то мы должны получить только задачи текущего пользователя
                filters.push(createListFilter('assignedTo', 'me', 'Исполнитель', 'я'));
            }
            options.filters = filters;
            args['area'] && (options.areaId = args['area']);
            args['workspace'] && (options.workspaceId = args['workspace']);
            return options;
        },

        fetchParams: function () {
            var params = {};

            // TODO do we really need support for this.options.filters now?
            _.each(this.options.filters, function (filter) {
                if (filter.param === 'date') {
                    _.extend(params, {
                        dueDateTimeFrom: filter.valueFrom,
                        dueDateTimeTo: filter.valueTo
                    });
                }
            });

            if (this.filters) {
                if (this.filters.assignedTo) {
                    if (this.filters.assignedTo === 'createdByMe') {
                        params.createdBy = 'me';
                    } else {
                        params.assignedTo = this.filters.assignedTo
                    }
                }
                this.filters.dueDate && (params.dueDate = this.filters.dueDate);
                this.filters.status && (params.status = this.filters.status);
            }

            if (this.sortParams && this.sortParams.sortBy) {
                params.orderBy = this.sortParams.sortBy + ((this.sortParams.sortDirection !== 'desc') ? '+' : '-');
            }

            return params;
        },

        fetchUrl: function () {
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'task');
        },

        updateData: function (updateOptions) {
            var that = this;
            this._mainDataLoaded = false;
            this.collection = new TaskCollection([], {
                // TODO need this location context here?
                locationContext: {
                    area: this.options.areaId,
                    workspace: this.options.workspaceId
                }
            });
            srv.fetch(_.defaults({
                collection: this.collection,
                url: this.fetchUrl(),
                urlParams: this.fetchParams(),
                success: function () {
                    settings.ajaxDebugging && that.log.debug('TaskListView: got task list (' + that.collection.length + ' items)');
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        renderData: function () {
            this.renderLocationContextCaption();
            this.renderEmbeddableAgendaView();
        },

        renderEmbeddableAgendaView: function () {
            if (this.embeddableAgendaView) {
                this.embeddableAgendaView.remove();
            }
            this.embeddableAgendaView = this.createChildView(EmbeddableAgendaView, {
                tasks: this.collection,
                supportsEmptyCaption: true,
                emptyCaption: 'Пока нет задач. Измените фильтр или создайте новую задачу',
                showTaskDetails: true,
                showLocationContext: _.isEmpty(this.locationContext())
                /*format: 'full',
                 headerText: false*/
            });
            this.embeddableAgendaView.render();
            this.embeddableAgendaView.$el.appendTo(this.$('.js-embeddedTaskList'));
        },

        onNewTaskClicked: function (evt) {
            evt && evt.preventDefault();
            var locationContext = this.locationContext();
            uiManager.openCard('task', _.extend({
                router: this.router,
                entityId: 'new'
            }, (locationContext.workspaceId ? locationContext : {})));
            /*var locationContext = this.locationContext();
             var url;
             if (!_.isEmpty(locationContext)) {
             url = '#!/tasks/area/' + locationContext.areaId + '/workspace/' + locationContext.workspaceId + '/edit/new';
             } else {
             url = '#!/tasks/edit/new';
             }
             // TODO use urlBuilder
             this.router.navigate(url, {trigger: true, replace: false});*/
        },

        /**
         * @deprecated
         * @param evt
         */
        onActivateFilters: function (evt) {

        }





    });
});

