﻿/**
 * @class TaskView
 */
define([
    'underscore',
    'jquery',
    'backbone',
    'srv',
    'settings',
    'views/common/BasePlanItemView',
    'text!views/tasks/TaskView_data.html',
    'models/tasks/TaskModel'
], function (
    _,
    $,
    Backbone,
    srv,
    settings,
    BasePlanItemView,
    dataTemplate,
    TaskModel
    ) {
    return BasePlanItemView.extend({

        dataTemplate: dataTemplate,

        headerText: 'Задача',

        events: _.extend({
            'click .js-parentTask': 'onParentTaskClicked',
            'click [data-subtask-id]': 'onSubTaskClicked',
            'change .js-checkbox.js-completed': 'onCheckboxChanged'
        }, BasePlanItemView.prototype.events),

        createModel: function() {
            return new TaskModel();
        },

        renderData: function() {
            BasePlanItemView.prototype.renderData.apply(this, arguments);
            this.ui.form.checkbox(this.$('.js-checkbox'));
        },

        fetchUrl: function() {
            return _.result(this.model, 'url') + '?include=subTasks';
        },

        editorUrl: function() {
            if (this.model.has('areaRef') && this.model.has('workspaceRef')) {
                return '#!/tasks/area/' + this.model.get('areaRef').id + '/workspace/' + this.model.get('workspaceRef').id + '/edit/' + this.model.get('id');
            } else {
                return '#!/tasks/edit/' + this.model.get('id');
            }
        },

        onParentTaskClicked: function(evt) {
            evt.preventDefault();
            this.model.get('parentTaskRef') && this.router.navigate(this.urlBuilder.task(this.model.get('parentTaskRef').id), {trigger: true, replace: false});
        },

        onSubTaskClicked: function(evt) {
            evt.preventDefault();
            var id = $(evt.currentTarget).attr('data-subtask-id');
            this.router.navigate(this.urlBuilder.task(id), {trigger: true, replace: false});
        },

        onCheckboxChanged: function(evt) {
            var that = this;
            srv.clearCache('task');
            srv.clearCache('calendar');
            var $input = $(evt.currentTarget);
            var checked = ($input.val() === 'true');
            setTimeout(function() {
                //var model = that.collection.get(entityId);
                that.model.set('status', checked ? 'completed' : 'active');
                srv.save({
                    //patch: true,
                    model: that.model,
                    url: settings.restUrlBase + 'task/' + that.model.id,
                    success: function() {
                        that.update();
                    }
                });
            }, 1);
            /*var $li = $input.closest('li');
            if (checked !== $li.hasClass('tb-completed')) {
                var entityId = $li.attr('entity-id');

            }
            checked ? $li.addClass('tb-completed') : $li.removeClass('tb-completed');*/
        }

    });
});

