﻿/**
 * @class ProfileView
 */
define([
    'underscore',
    'jquery',
    'views/BasePageView',
    'text!views/users/ProfileView_data.html',
    'text!views/users/ProfileView_header.html',
    'settings',
    'srv',
    'version',
    'authentication',
	'storage',
	'pubSub',
    'models/users/UserModel',
    'utils/cache/currentUserCache',
    'utils/cache/allUsersCache',
    'utils/cache/structureCache',
    'utils/uiTheme',
	'utils/uiManager',
    'views/common/BaseLowerToolbar',
	'views/users/ProfileCardView',
], function (_, $, BasePageView, dataTemplate, headerTemplate, settings, srv, version, authentication, storage, pubSub, UserModel, currentUserCache, allUsersCache, structureCache, uiTheme, uiManager, BaseLowerToolbar, ProfileCardView)
{
    return BasePageView.extend({

    	cardEntityType: 'profile',

        hasSidebar: true,

        headerTemplate: headerTemplate,

        events: _.extend({
            'click .js-toggleSidebar': 'onToggleSidebarClicked',
            'click .js-logout': 'onLogout',
            'click .js-syncLog': 'onSyncLog',
            'click .j-clear-cache-all': 'onClearAllCache',
            'click .j-layout-switcher': 'onLayoutSwitcherClick',
            'click .j-push-switcher': 'onPushSwitcherClick',
            'click .j-test-push': 'onTestPushClick',
        }, BasePageView.prototype.events),

        model: null,

        versionNumber: version.get(),

        initialize: function() {
            BasePageView.prototype.initialize.apply(this, arguments);
            this._initializeLowerToolbar();

            if (uiManager.isTabletMode())
            	uiManager.openCard('profile', { router: this.router });
        },

        _initializeLowerToolbar: function() {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh',
                            width: '33%'
                        },
                        {
                            name: 'feedback',
                            icon: 'question-circle',
                            width: '33%',
                            onClicked: function() {
                                this.parentView && this.parentView.router.navigate('#!/feedback', {
                                    trigger: true,
                                    replace: false
                                });
                            }
                        },
                        {
                            name: 'logout',
                            icon: 'sign-out',
                            classes: 'tb-logoutButton',
                            caption: 'Выход',
                            onClicked: function($td, item) {
                                that.onLogout();
                            }

                        }
                    ]
                });
            }
        },

        updateData: function(updateOptions) {
            var that = this;
            this.model = new UserModel();
            srv.fetch(_.defaults({
                model: this.model,
                url: settings.restUrlBase + 'user/current',
                success: function() {
                    that.onDataUpdated();
                }
            }, updateOptions));

            // additionally update the profile cache
            setTimeout(function() {
                currentUserCache.update();
            }, 1);


        },

        renderData: function ()
        {
        	this.showLayoutSwitcher = false;

        	var pixelRatio = uiManager.getDevicePixelRatio();
        	var $w = $(window);
        	var actualWidth = $w.width() * pixelRatio;
        	var actualHeight = $w.height() * pixelRatio;

        	var userLayout = storage.get('UserLayoutMode');

        	if (actualHeight >= 800 || actualWidth >= 800)
        		this.showLayoutSwitcher = true;

        	this.$content().html(dataTemplate.call(this));
			
        	var $ctrl = this.$content().find('.j-layout-switcher');
        	if ($ctrl.length > 0)
        	{
        		if (!userLayout)
        		{
        			$ctrl.addClass('ui-checkbox-on').find('i.fa').removeClass('fa-square-o').addClass('fa-check-square-o');
        			//$ctrl.find('span.ui-paramValue').text('Две колонки');
        		}
        	}

        	var PushDisabled = storage.get('PushDisabled');
        	var $pushSwither = this.$content().find('.j-push-switcher');
        	if (PushDisabled)
        	{
        		$pushSwither.removeClass('ui-checkbox-on').find('i.fa').addClass('fa-square-o');
        		$pushSwither.find('span').text('Выключено');
        	}
        	else
        	{
        		$pushSwither.addClass('ui-checkbox-on').find('i.fa').addClass('fa-check-square-o');
        		$pushSwither.find('span').text('Включено');
        	}

        	var email = this.model.get('email');
        	if (email == 'bashkankov@teambridge.ru' || email == 'bukatin@teambridge.ru')
        	{
        		var messages = [];
        		_.each(tbPushClient.messages, function (item) { messages.push('<li>-message: ' + JSON.stringify(item) + '</li>'); });

        		this.$content().find('.j-push-activity').html(tbPushClient.htmlStack.join('') + messages.join(''));
        	}
        },

        onLayoutSwitcherClick: function (e)
        {
        	var $ctrl = $(e.currentTarget);
        	if ($ctrl.hasClass('ui-checkbox-on'))
        	{
        		// too one column
        		storage.set('UserLayoutMode', uiManager.modes.smartphone);

        		$ctrl.removeClass('ui-checkbox-on').find('i.fa').removeClass('fa-check-square-o').addClass('fa-square-o');
        		//$ctrl.find('span.ui-paramValue').text('Одна колонка');

        		$(document.body).hide();

        		setTimeout(function ()
        		{
        			uiManager.onOrientationChange();

        			$(document.body).show();
        			$(window).trigger('resize');

        		}, 100);
        	}
        	else
        	{
        		// to auto
        		storage.unset('UserLayoutMode');

        		$ctrl.addClass('ui-checkbox-on').find('i.fa').removeClass('fa-square-o').addClass('fa-check-square-o');
        		//$ctrl.find('span.ui-paramValue').text('Две колонки');

        		$(document.body).hide();

        		setTimeout(function ()
        		{
        			uiManager.onOrientationChange();

        			$(document.body).show();
        			$(window).trigger('resize');

        		}, 100);
        	}
        },

        onPushSwitcherClick: function (e)
        {
        	var $pushSwither = $(e.currentTarget);
        	if ($pushSwither.hasClass('ui-checkbox-on'))
        	{
        		tbPushClient.deleteUserRegistration().then(function ()
        		{
        			storage.set('PushDisabled', true);

        			$pushSwither.removeClass('ui-checkbox-on').find('i.fa').removeClass('fa-check-square-o').removeClass('fa-square-o').addClass('fa-square-o');
        			$pushSwither.find('span').text('Выключено');
        		});
        	}
        	else
        	{
        		storage.unset('PushDisabled');

        		tbPushClient.saveUserRegistration(function ()
        		{
        			$pushSwither.addClass('ui-checkbox-on').find('i.fa').removeClass('fa-check-square-o').removeClass('fa-square-o').addClass('fa-check-square-o');
        			$pushSwither.find('span').text('Включено');
        		});
        	}
        },

        onToggleSidebarClicked: function(evt) {
            this.showSidebar();
            evt.preventDefault();
        },

        onLogout: function ()
        {
        	var that = this;
        	tbPushClient.deleteUserRegistration().always(function ()
        	{
        		authentication.logout();
        		currentUserCache.clear();
        		allUsersCache.clear();
        		structureCache.clear();
        		srv.clearCache('ajax', true);

        		that.router.redirectToLogin();
        	});
        },

        onSyncLog: function() {
            this.log.sync(function() {
                alert('Ага, выполнено успешно.');
            });
        },

        onClearAllCache: function (e)
        {
        	var $button = $(e.currentTarget)
        	if ($button.prop('disabled')===true)
        		return false;

        	$button.html('<i class="fa fa-refresh fa-spin"></i> Очистка...').prop('disabled', true);
        	storage.clearAllAjax();

        	var that = this;
        	setTimeout(function ()
        	{
        		$button.html('<i class="fa fa-refresh fa-spin"></i> Обновление данных...');

        		that.listenToOnce(pubSub, 'structure:updated', function ()
        		{
        			$button.html('Очистить кэш').prop('disabled',false);
        		});

        		structureCache.update();

        		allUsersCache.update();
        	}, 500);
    	},


        onTestPushClick: function (e)
        {
        	// TODO: чтоб сервер длину коммента резал на 255 символов с ...

        	tbPushClient.onNotificationGCM({
        		event: 'message',
        		foreground: true,
        		payload: {
        			data: {
        				areaRef: { id: "6cedd444-8d34-42c7-8843-017fa820fc94", caption: "qwerqw1234" },
        				dependentEntityRef: { type: "comment", id: "510206b0-1910-e511-9900-080027f9a43f", caption: "Вот такой вот комментарий" },
        				entityRef: { type: "task", id: "4a0206b0-1910-e511-9900-080027f9a43f", caption: "111111111" },
        				workspaceRef: { id: "6ce71ce8-e9be-4f6e-ae9e-1cbc6daebf52", caption: "test invite" },
        				operation: 'updated',
        			},
        			title: 'Добавлен комментарий к задаче'
        		}
        	});

        	tbPushClient.onNotificationGCM({
        		event: 'message',
        		foreground: true,
        		payload: {
        			data: {
        				areaRef: { id: "6cedd444-8d34-42c7-8843-017fa820fc94", caption: "qwerqw1234" },
        				dependentEntityRef: null,
        				entityRef: { type: "task", id: "35e9e2d3-790e-e511-9900-080027f9a43f", caption: "rtyert" },
        				workspaceRef: { id: "6ce71ce8-e9be-4f6e-ae9e-1cbc6daebf52", caption: "test invite" },
        				operation: 'created',
        			},
        			title: 'Создана задача'
        		}
        	});
        	tbPushClient.onNotificationGCM({
        		event: 'message',
        		foreground: false,
        		payload: {
        			data: {
        				areaRef: { id: "a3454a7e-276c-4d52-a49e-4ff1f6005b75", caption: "TeamBridge" },
        				dependentEntityRef: null,
        				entityRef: { type: "task", id: "46c97970-f20d-e511-9900-080027f9a43f", caption: "qwerqwe 12312 qwer" },
        				workspaceRef: { id: "58638e1c-76ae-41fd-b339-41d2042d7eb4", caption: "Запуск новой версии TeamBridge" },
        				operation: 'updated',
        			},
        			title: 'Изменена задача'
        		}
        	});

        	setTimeout(function ()
        	{
        		tbPushClient.onNotificationGCM({
        			event: 'message',
        			foreground: true,
        			payload: {
        				data: {
        					areaRef: { id: "a3454a7e-276c-4d52-a49e-4ff1f6005b75", caption: "TeamBridge" },
        					dependentEntityRef: null,
        					entityRef: { type: "task", id: "441a4960-f00d-e511-9900-080027f9a43f", caption: "twqewrqwe" },
        					workspaceRef: { id: "58638e1c-76ae-41fd-b339-41d2042d7eb4", caption: "Запуск новой версии TeamBridge" },
        					operation: 'created',
        				},
        				title: 'Создана задача'
        			}
        		});
        		setTimeout(function ()
        		{
        			tbPushClient.onNotificationGCM({
        				event: 'message',
        				foreground: true,
        				payload: {
        					data: {
        						areaRef: { id: "932cb08c-4deb-451a-ac08-27f477e94c36", caption: "Портал ТимБридж2" },
        						dependentEntityRef: null,
        						entityRef: { type: "task", id: "a16ea832-650b-e511-9900-080027f9a43f", caption: "twer" },
        						workspaceRef: { id: "21be4c66-9e79-49d5-bf96-8aea173693c5", caption: "Project 1" },
        						operation: 'created',
        					},
        					title: 'Создана задача'
        				}
        			});
        		},1000);
        	}, 5000);
        },
    });
});
